@@ -1,6 +1,6 @@
 /** \file app_interrogate.h */
 
-// Copyright (C) 2021 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2021-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef APP_INTERROGATE_H_
@@ -8,6 +8,7 @@
 
 #include "cmdline/parsed_cmd.h"
 
-void interrogate(Parsed_Cmd * parsed_cmd);
+void app_interrogate(Parsed_Cmd * parsed_cmd);
+void init_app_interrogate();
 
 #endif /* APP_INTERROGATE_H_ */
