@@ -10,8 +10,6 @@
 #include <config.h>
 
 #include <assert.h>
-#include <base/base_services.h>
-#include <base/drm_connector_state.h>
 #include <ctype.h>
 #include <errno.h>
 #include <glib-2.0/glib.h>
@@ -34,9 +32,6 @@
 #include "util/file_util.h"
 #include "util/glib_string_util.h"
 #include "util/i2c_util.h"
-#ifdef USE_LIBDRM
-#include "util/libdrm_util.h"
-#endif
 #include "util/linux_util.h"
 #include "util/regex_util.h"
 #include "util/report_util.h"
@@ -59,6 +54,9 @@
 #include "base/ddc_packets.h"
 #include "base/display_retry_data.h"
 #include "base/displays.h"
+#ifdef USE_LIBDRM
+#include "base/drm_connector_state.h"
+#endif
 #include "base/dsa2.h"
 #include "base/linux_errno.h"
 #include "base/monitor_model_key.h"
@@ -67,6 +65,7 @@
 #include "base/sleep.h"
 #include "base/status_code_mgt.h"
 #include "base/tuned_sleep.h"
+#include "base/base_services.h"
 
 #include "vcp/parse_capabilities.h"
 #include "vcp/persistent_capabilities.h"
@@ -77,7 +76,6 @@
 
 #include "sysfs/sysfs_dpms.h"
 #include "sysfs/sysfs_sys_drm_connector.h"
-// #include "sysfs/i2c_sysfs_i2c_info.h"
 #include "sysfs/sysfs_top.h"
 #include "sysfs/sysfs_base.h"
 
@@ -103,8 +101,10 @@
 #include "ddc/ddc_vcp_version.h"
 #include "ddc/ddc_vcp.h"
 
+#ifdef WATCH_DISPLAYS
 #include "dw/dw_main.h"
 #include "dw/dw_services.h"
+#endif
 
 #include "cmdline/cmd_parser_aux.h"    // for parse_feature_id_or_subset(), should it be elsewhere?
 #include "cmdline/cmd_parser.h"
@@ -332,36 +332,48 @@ verify_i2c_access() {
       while ( (busno = bs256_iter_next(iter)) >= 0)  {
          char fnbuf[20];   // oversize to avoid -Wformat-truncation error
          snprintf(fnbuf, sizeof(fnbuf), "/dev/i2c-%d", busno);
-         if ( access(fnbuf, R_OK|W_OK) < 0 ) {
-            int errsv = errno;   // EACCESS if lack permissions, ENOENT if file doesn't exist
-            if (errsv == ENOENT) {
+         if (running_as_root) {
+            struct stat stat_buf;
+            int rc = stat(fnbuf, &stat_buf);
+            if (rc != 0) {
                buses_without_devices = bs256_insert(buses_without_devices, busno);
                fprintf(stderr, "Device %s does not exist. Error = %s\n",
-                              fnbuf, linux_errno_desc(errsv));
-            }
-            else {
-               inaccessible_devices = bs256_insert(inaccessible_devices, busno);
-               fprintf(stderr, "Device %s is not readable and writable.  Error = %s\n",
-                           fnbuf, linux_errno_desc(errsv) );
-               include_open_failures_reported(busno);
+                               fnbuf, linux_errno_desc(-ENOENT));
+             }
+         }
+         else {
+            if ( access(fnbuf, R_OK|W_OK) < 0 ) {
+               int errsv = errno;   // EACCESS if lack permissions, ENOENT if file doesn't exist
+               if (errsv == ENOENT) {
+                  buses_without_devices = bs256_insert(buses_without_devices, busno);
+                  fprintf(stderr, "Device %s does not exist. Error = %s\n",
+                                 fnbuf, linux_errno_desc(errsv));
+               }
+               else {
+                  inaccessible_devices = bs256_insert(inaccessible_devices, busno);
+                  fprintf(stderr, "Device %s is not readable and writable.  Error = %s\n",
+                              fnbuf, linux_errno_desc(errsv) );
+                  include_open_failures_reported(busno);
+               }
             }
          }
       }
       bs256_iter_free(iter);
-      buses_without_devices_ct = bs256_count(buses_without_devices);
-      inaccessible_devices_ct = bs256_count(inaccessible_devices);
+   }
 
-      if (buses_without_devices_ct > 0) {
-         fprintf(stderr, "/sys/bus/i2c buses without /dev/i2c-N devices: %s\n",
-                bs256_to_string_decimal_t(buses_without_devices, "/sys/bus/i2c/devices/i2c-", " ") );
-         fprintf(stderr, "Driver i2c_dev must be loaded or builtin\n");
-         fprintf(stderr, "See https://www.ddcutil.com/kernel_module\n");
-      }
-      if (inaccessible_devices_ct > 0) {
-         fprintf(stderr, "Devices possibly used for DDC/CI communication cannot be opened: %s\n",
-                bs256_to_string_decimal_t(inaccessible_devices, "/dev/i2c-", " "));
-         fprintf(stderr, "See https://www.ddcutil.com/i2c_permissions\n");
-      }
+   buses_without_devices_ct = bs256_count(buses_without_devices);
+   inaccessible_devices_ct = bs256_count(inaccessible_devices);
+
+   if (buses_without_devices_ct > 0) {
+      fprintf(stderr, "/sys/bus/i2c buses without /dev/i2c-N devices: %s\n",
+             bs256_to_string_decimal_t(buses_without_devices, "/sys/bus/i2c/devices/i2c-", " ") );
+      fprintf(stderr, "Driver i2c_dev must be loaded or builtin\n");
+      fprintf(stderr, "See https://www.ddcutil.com/kernel_module\n");
+   }
+   if (inaccessible_devices_ct > 0) {
+      fprintf(stderr, "Devices possibly used for DDC/CI communication cannot be opened: %s\n",
+             bs256_to_string_decimal_t(inaccessible_devices, "/dev/i2c-", " "));
+      fprintf(stderr, "See https://www.ddcutil.com/i2c_permissions\n");
    }
 
    int result = buses_ct - (buses_without_devices_ct + inaccessible_devices_ct);
@@ -392,20 +404,29 @@ int verify_i2c_access_for_single_bus(int busno) {
    else {
        char fnbuf[20];   // oversize to avoid -Wformat-truncation error
        snprintf(fnbuf, sizeof(fnbuf), "/dev/i2c-%d", busno);
-       if ( access(fnbuf, R_OK|W_OK) < 0 ) {
-          int errsv = errno;   // EACCESS if lack permissions, ENOENT if file doesn't exist
-          if (errsv == ENOENT) {
-             fprintf(stderr, "Device %s does not exist. Error = %s\n",
-                               fnbuf, linux_errno_desc(errsv));
-          }
-          else {
-             fprintf(stderr, "Device %s is not readable and writable.  Error = %s\n",
-                            fnbuf, linux_errno_desc(errsv) );
-             include_open_failures_reported(busno);
+       if (running_as_root) {
+          struct stat stat_buf;
+          int statrc = stat(fnbuf, &stat_buf);
+          if (statrc == 0) {
+             result = 1;
           }
        }
-       else
-          result = 1;
+       else {
+          if ( access(fnbuf, R_OK|W_OK) < 0 ) {
+             int errsv = errno;   // EACCESS if lack permissions, ENOENT if file doesn't exist
+             if (errsv == ENOENT) {
+                fprintf(stderr, "Device %s does not exist. Error = %s\n",
+                                  fnbuf, linux_errno_desc(errsv));
+             }
+             else {
+                fprintf(stderr, "Device %s is not readable and writable.  Error = %s\n",
+                               fnbuf, linux_errno_desc(errsv) );
+                include_open_failures_reported(busno);
+             }
+          }
+          else
+             result = 1;
+       }
    }
 
    DBGTRC_DONE(debug, TRACE_GROUP, "Returning %d.", result);
@@ -651,7 +672,8 @@ execute_cmd_with_optional_display_handle(
       {
          assert(dh);
          if (app_check_dynamic_features(dh->dref)) {
-            ensure_vcp_version_set(dh);
+            if (parsed_cmd->output_level > DDCA_OL_TERSE)
+               ensure_vcp_version_set(dh);
 
             Public_Status_Code psc = app_show_feature_set_values_by_dh(dh, parsed_cmd);
             main_rc = (psc==0) ? EXIT_SUCCESS : EXIT_FAILURE;
@@ -785,6 +807,7 @@ DDCA_Syslog_Level preparse_syslog_level(int argc, char** argv) {
 }
 
 
+#ifdef WATCH_DISPLAYS
 /** Terminates watch displays thread in case of CTRL-C etc. then
  *  terminates execution.
  */
@@ -801,6 +824,7 @@ void interrupt_handler(int sig) {
    }
    exit(0);
 }
+#endif
 
 
 //
@@ -841,7 +865,9 @@ main(int argc, char *argv[]) {
    add_local_rtti_functions();      // add entries for this file
    init_base_services();            // so tracing related modules are initialized
    init_ddc_services();             // initializes i2c, usb, ddc, vcp, dynvcp
+#ifdef WATCH_DISPLAYS
    init_dw_services();              // initializes subdir dw
+#endif
    init_app_ddcutil_services();
 #ifdef ENABLE_ENVCMDS
    init_app_sysenv_services();
@@ -1069,11 +1095,15 @@ main(int argc, char *argv[]) {
    // xdg_tests(); // for development
 
    if (parsed_cmd->flags2 & CMD_FLAG2_F2) {
+      bool saved_prefix_report_output = rpt_set_ornamentation_enabled(false);
       consolidated_i2c_sysfs_report(0);
+#ifdef USE_LIBDRM
       if (use_drm_connector_states)
          report_drm_connector_states(0);
+#endif
       // rpt_label(0, "*** Tests Done ***");
       // rpt_nl();
+      rpt_set_ornamentation_enabled(saved_prefix_report_output);
    }
 
    Call_Options callopts = CALLOPT_NONE;
@@ -1113,6 +1143,7 @@ main(int argc, char *argv[]) {
       main_rc = EXIT_SUCCESS;
    }
 
+#ifdef WATCH_DISPLAYS
    else if (parsed_cmd->cmd_id == CMDID_C1) {
       bool saved_prefix_report_output = rpt_set_ornamentation_enabled(false);
 
@@ -1142,9 +1173,13 @@ main(int argc, char *argv[]) {
 
       rpt_set_ornamentation_enabled(saved_prefix_report_output);
    }
+#endif
 
    else if (parsed_cmd->cmd_id == CMDID_C2 ||
             parsed_cmd->cmd_id == CMDID_C3 ||
+#ifndef WATCH_DISPLAYS
+            parsed_cmd->cmd_id == CMDID_C1 ||
+#endif
             parsed_cmd->cmd_id == CMDID_C4)
    {
       bool saved_prefix_report_output = rpt_set_ornamentation_enabled(false);
