@@ -8,7 +8,6 @@
 
 /** \cond */
 #include <assert.h>
-#include <base/drm_connector_state.h>
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
@@ -29,6 +28,9 @@
 #include "util/sysfs_util.h"
 
 #include "base/core.h"
+#ifdef USE_LIBDRM
+#include "base/drm_connector_state.h"
+#endif
 #include "base/linux_errno.h"
 #include "base/rtti.h"
 /** \endcond */
@@ -749,19 +751,13 @@ void report_one_connector(
 }
 
 
-void query_drm_using_sysfs()
-{
+void query_drm_using_sysfs() {
    bool debug = false;
    DBGTRC_STARTING(debug, TRACE_GROUP, "");
    int depth = 1;
    int d0 = depth;
    rpt_nl();
-   char * dname =
-#ifdef TARGET_BSD
-             "/compat/linux/sys/class/drm";
-#else
-             "/sys/class/drm";
-#endif
+   char * dname = SYS"/sys/class/drm";
 
    rpt_vstring(d0, "*** Examining %s ***", dname);
    dir_filtered_ordered_foreach(
@@ -774,11 +770,11 @@ void query_drm_using_sysfs()
    DBGTRC_DONE(debug, TRACE_GROUP, "");
 }
 
+
 //
 // Functions for probing /sys
 //
 
-
 void show_top_level_sys_entries(int depth) {
    rpt_label(depth, "*** Character device major numbers of interest: ***");
    char * names[] = {"i2c", "drm", "ddcci", "aux", "vfio", NULL};
@@ -872,7 +868,9 @@ void dump_sysfs_i2c(Env_Accumulator * accum) {
    execute_shell_cmd("find /sys/devices -name class | xargs grep \".*\"");
    rpt_nl();
 
+#ifdef USE_LIBDRM
    report_drm_connector_states(0);
+#endif
 
    if (accum->is_arm) {
       rpt_label(0,"*** Extended dump of sysfs video devices for ARM architecture ***");
@@ -921,8 +919,6 @@ void dump_sysfs_i2c(Env_Accumulator * accum) {
    // g_ptr_array_set_free_func(displaylink_devices,free);  //redundant
    g_ptr_array_free(displaylink_devices,true);
 
-
-
    DBGTRC_DONE(debug, TRACE_GROUP, "");
 }
 
