@@ -13,7 +13,7 @@
  * - debug and trace messages
  */
 
-// Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2025 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
@@ -26,7 +26,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-// #include <threads.h>  // requires glibc 2.28, apparently unused
 
 #ifdef TARGET_BSD
 #include <pthread_np.h>
@@ -61,8 +60,16 @@
 
 #include "base/core.h"
 
+
+//
+// Globals
+//
+
+bool timestamp_in_syslog_debug_msgs = false;
 bool tracing_initialized = false;
 bool library_disabled = false;
+bool running_as_root = false;
+
 
 //
 // Standard call options
@@ -70,11 +77,9 @@ bool library_disabled = false;
 
 Value_Name_Table callopt_bitname_table2 = {
       VN(CALLOPT_ERR_MSG),
- //   VN(CALLOPT_ERR_ABORT),
       VN(CALLOPT_RDONLY),
       VN(CALLOPT_WARN_FINDEX),
       VN(CALLOPT_WAIT),
-      VN(CALLOPT_FORCE_SLAVE_ADDR),
       VN(CALLOPT_NONE),                // special entry
       VN_END
 };
@@ -602,14 +607,17 @@ static bool vdbgtrc(
 
          // if (trace_to_syslog || (options & DBGTRC_OPTIONS_SYSLOG)) {
          if (test_emit_syslog(DDCA_SYSLOG_DEBUG) || dbgtrc_trace_to_syslog_only) {
-#ifdef PREV
-            char * syslog_msg = g_strdup_printf("%s%s(%-30s) %s%s%s",
-                        thread_prefix, elapsed_prefix, funcname, retval_info, base_msg,
-                        (tag_output) ? " (J)" : "");
-#endif
-            char * syslog_msg = g_strdup_printf("%s(%-30s) %s%s%s",
-                        thread_prefix, funcname, retval_info, base_msg,
-                        (tag_output) ? " (J)" : "");
+            char * syslog_msg = NULL;
+            if (timestamp_in_syslog_debug_msgs) {
+               syslog_msg = g_strdup_printf("%s%s(%-30s) %s%s%s",
+                           thread_prefix, elapsed_prefix, funcname, retval_info, base_msg,
+                           (tag_output) ? " (J)" : "");
+            }
+            else {
+               syslog_msg = g_strdup_printf("%s(%-30s) %s%s%s",
+                           thread_prefix, funcname, retval_info, base_msg,
+                           (tag_output) ? " (J)" : "");
+            }
             syslog(LOG_DEBUG, "%s", syslog_msg);
             free(syslog_msg);
          }
@@ -1118,7 +1126,7 @@ end_capture(void) {
    // printf("(%s) Starting.\n", __func__);
    assert(fdesc->in_memory_file);
    if (fflush(fdesc->in_memory_file) < 0) {
-      set_ferr_to_default();
+      fclose(fdesc->in_memory_file);
       SEVEREMSG("flush() failed. errno=%d", errno);
       // return g_strdup(result);
       result = g_strdup("\0");
@@ -1127,7 +1135,6 @@ end_capture(void) {
       // n. open_memstream() maintains a null byte at end of buffer, not included in in_memory_bufsize
       result = g_strdup(fdesc->in_memory_bufstart);
       if (fclose(fdesc->in_memory_file) < 0) {
-         set_ferr_to_default();
          SEVEREMSG("fclose() failed. errno=%d", errno);
          result = g_strdup("\0");
       }
@@ -1138,7 +1145,7 @@ end_capture(void) {
    }
    set_fout_to_default();
    if (fdesc->flags & DDCA_CAPTURE_STDERR)
-   set_ferr_to_default();
+      set_ferr_to_default();
    redirect_reports_to_syslog = fdesc->saved_rpt_to_syslog;
    fdesc->in_memory_capture_active = false;
    // traced_function_stack_suspended = false;
