@@ -55,9 +55,16 @@
 #define ASSERT_MARKER(_struct_ptr, _marker_value) \
    assert(_struct_ptr && memcmp(_struct_ptr->marker, _marker_value, 4) == 0)
 
-// Indicates that all tracing facilities have been configured
+
+//
+// Global Variables
+// 
+
+// Indicates that all tracing facilities have been configured:
 extern bool tracing_initialized;
 extern bool library_disabled;
+extern bool timestamp_in_syslog_debug_msgs;
+extern bool running_as_root;
 
 
 //
@@ -68,12 +75,9 @@ extern bool library_disabled;
 typedef Byte Call_Options;
 #define CALLOPT_NONE         0x00    ///< no options
 #define CALLOPT_ERR_MSG      0x80    ///< issue message if error
-// #define CALLOPT_ERR_ABORT    0x40    ///< terminate execution if error
 #define CALLOPT_RDONLY       0x20    ///< open read-only
 #define CALLOPT_WARN_FINDEX  0x10    ///< issue warning msg re hiddev_field_info.field_index change
-#define CALLOPT_FORCE        0x08    ///< ignore various validity checks
 #define CALLOPT_WAIT         0x04    ///< wait on locked resources, if false then fail
-#define CALLOPT_FORCE_SLAVE_ADDR 0x02 ///< use op I2C_SLAVE_FORCE (not currently used)
 
 char * interpret_call_options_t(Call_Options calloptions);
 
@@ -244,34 +248,38 @@ bool dbgtrc_returning_string(
  * It is reported to not exist on Termux.
  * However, if  "assert(#_assertion);" is used instead of "__assert_fail(...);",
  * the program does not terminate.
+ * 05/2025:
+ * Using exit(1) to terminate apparently causes powerdevil to terminate with status
+ * success, and as a result it does not automatically restart.
+ * Revert to using __assert_fail() to terminate.
+ * tmux source itself does not reference ddcutil, so it's unclear what the
+ * compatibility issue is.  Failure to build in obscure environments where
+ * __assert_fail() does not exist is less important than powerdevil/kde integration.
  */
 // n. using ___LINE__ instead of line in __assert_fail() causes compilation error
 #ifdef NDEBUG
-#define TRACED_ASSERT(_assertion) \
-   do { \
-   } while (0)
+#define TRACED_ASSERT(_assertion) do {} while (0)
+#define TRACED_ASSERT_IFF(_cond1,_cond2) do {} while (0)
 #else
 #define TRACED_ASSERT(_assertion) \
    do { \
-      if (_assertion) { \
+         if (_assertion) { \
          ;              \
       }                 \
       else {           \
-         /* int line = __LINE__; */  \
+         int line = __LINE__;  \
          dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_NONE, __func__, __LINE__, __FILE__,   \
                       "Assertion failed: \"%s\" in file %s at line %d",  \
                       #_assertion, __FILE__,  __LINE__);   \
          SYSLOG2(DDCA_SYSLOG_ERROR, "Assertion failed: \"%s\" in file %s at line %d",  \
                          #_assertion, __FILE__,  __LINE__);   \
          /* assert(#_assertion); */ \
-         /*  __assert_fail(#_assertion, __FILE__, line, __func__); */  \
+         __assert_fail(#_assertion, __FILE__, line, __func__);  \
          /* don't need assertion info, dbgtrc() and dbgtrc() have been called */ \
-         exit(1); \
+         /* exit(1); */  \
       } \
    } while (0)
-#endif
 
-#ifndef TRACED_ASSERT_IFF
 #define TRACED_ASSERT_IFF(_cond1, _cond2) \
    TRACED_ASSERT( ( (_cond1) && (_cond2) ) || ( !(_cond1) && !(_cond2) ) )
 #endif
@@ -328,7 +336,8 @@ bool dbgtrc_returning_string(
 
 #define DBGTRC_STARTING(debug_flag, trace_group, format, ...) \
    do { \
-      push_traced_function(__func__); \
+      if (traced_function_stack_enabled) \
+         push_traced_function(__func__); \
       dbgtrc( (debug_flag) || trace_callstack_call_depth > 0 || is_traced_callstack_call(__func__) \
                  ? DDCA_TRC_ALL : (trace_group), \
               DBGTRC_OPTIONS_STARTING, \
@@ -340,7 +349,8 @@ bool dbgtrc_returning_string(
       dbgtrc( (debug_flag) || trace_callstack_call_depth > 0  \
               ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
               __func__, __LINE__, __FILE__, "Done      "format, ##__VA_ARGS__); \
-      pop_traced_function(__func__);  \
+      if (traced_function_stack_enabled) \
+         pop_traced_function(__func__);  \
    } while (0)
 
 #define DBGTRC_DONE_WO_TRACED_FUNCTION_STACK(debug_flag , trace_group, format, ...) \
@@ -365,7 +375,8 @@ bool dbgtrc_returning_string(
              ? DDCA_TRC_ALL : (trace_group), \
           DBGTRC_OPTIONS_DONE, \
           __func__, __LINE__, __FILE__, _result, format, ##__VA_ARGS__); \
-       pop_traced_function(__func__);  \
+       if (traced_function_stack_enabled) \
+          pop_traced_function(__func__);  \
     } while (0)
 
 /* Notes on macros that have ENABLE_FAILSIM variants.
@@ -394,7 +405,8 @@ bool dbgtrc_returning_string(
       dbgtrc_ret_ddcrc( \
          (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
          __func__, __LINE__, __FILE__, rc, format, ##__VA_ARGS__); \
-      pop_traced_function(__func__);
+      if (traced_function_stack_enabled) \
+         pop_traced_function(__func__);
   } while (0)
 #define DBGTRC_RET_DDCRC2(debug_flag, trace_group, _rc, _data_pointer, format, ...) \
    do { \
@@ -412,7 +424,8 @@ bool dbgtrc_returning_string(
       dbgtrc_ret_ddcrc( \
          (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
          __func__, __LINE__, __FILE__, _rc, format, ##__VA_ARGS__); \
-      pop_traced_function(__func__);
+      if (traced_function_stack_enabled) \
+         pop_traced_function(__func__);
   } while (0)
 #else
 #define DBGTRC_RET_DDCRC(debug_flag, trace_group, rc, format, ...) \
@@ -421,7 +434,8 @@ bool dbgtrc_returning_string(
          (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), \
          DBGTRC_OPTIONS_DONE, \
          __func__, __LINE__, __FILE__, rc, format, ##__VA_ARGS__); \
-      pop_traced_function(__func__);  \
+      if (traced_function_stack_enabled) \
+         pop_traced_function(__func__);  \
    } while (0)
 
 #define DBGTRC_RET_DDCRC2(debug_flag, trace_group, rc, data_pointer, format, ...) \
@@ -429,7 +443,8 @@ bool dbgtrc_returning_string(
    dbgtrc_ret_ddcrc( \
       (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
       __func__, __LINE__, __FILE__, rc, format, ##__VA_ARGS__); \
-      pop_traced_function(__func__);  \
+      if (traced_function_stack_enabled) \
+         pop_traced_function(__func__);  \
    } while (0)
 #endif
 
@@ -446,7 +461,8 @@ bool dbgtrc_returning_string(
       dbgtrc_returning_errinfo( \
           (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
           __func__, __LINE__, __FILE__, errinfo_result, format, ##__VA_ARGS__); \
-      pop_traced_function(__func__); \
+      if (traced_function_stack_enabled) \
+         pop_traced_function(__func__); \
    } while(0)
 #define DBGTRC_RET_ERRINFO2(debug_flag, trace_group, errinfo_result, data_pointer, format, ...) \
    do { \
@@ -464,7 +480,8 @@ bool dbgtrc_returning_string(
       dbgtrc_returning_errinfo( \
           (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
           __func__, __LINE__, __FILE__, errinfo_result, format, ##__VA_ARGS__); \
-      pop_traced_function(__func__); \
+      if (traced_function_stack_enabled) \
+         pop_traced_function(__func__); \
    } while(0)
 #else
 #define DBGTRC_RET_ERRINFO(debug_flag, trace_group, errinfo_result, format, ...) \
@@ -472,7 +489,8 @@ bool dbgtrc_returning_string(
       dbgtrc_returning_errinfo( \
              (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), \
              DBGTRC_OPTIONS_DONE, __func__, __LINE__, __FILE__, errinfo_result, format, ##__VA_ARGS__); \
-      pop_traced_function(__func__);  \
+      if (traced_function_stack_enabled) \
+         pop_traced_function(__func__);  \
    } while (0)
 #define DBGTRC_RET_ERRINFO2(debug_flag, trace_group, errinfo_result, pointer, format, ...) \
       do { \
@@ -480,7 +498,8 @@ bool dbgtrc_returning_string(
              (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), \
              DBGTRC_OPTIONS_DONE, \
              __func__, __LINE__, __FILE__, errinfo_result, format, ##__VA_ARGS__); \
-         pop_traced_function(__func__);  \
+         if (traced_function_stack_enabled) \
+            pop_traced_function(__func__);  \
       } while (0)
 #endif
 
@@ -492,7 +511,8 @@ bool dbgtrc_returning_string(
          (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), \
          DBGTRC_OPTIONS_DONE, \
          __func__, __LINE__, __FILE__, SBOOL(bool_result), format, ##__VA_ARGS__); \
-     pop_traced_function(__func__);  \
+     if (traced_function_stack_enabled) \
+        pop_traced_function(__func__);  \
   } while (0)
 
 // typedef (*dbg_struct_func)(void * structptr, int depth);
@@ -505,7 +525,8 @@ bool dbgtrc_returning_string(
             _dbgfunc(_structptr, 1); \
          } \
       } \
-      pop_traced_function(__func__);  \
+      if (traced_function_stack_enabled) \
+         pop_traced_function(__func__);  \
    } while (0)
 
 #define DBGTRC_RET_STRUCT(_flag, _trace_group, _structname, _dbgfunc, _structptr) \
@@ -518,7 +539,8 @@ do { \
          _dbgfunc(_structptr, 1); \
       } \
    } \
-   pop_traced_function(__func__);  \
+   if (traced_function_stack_enabled) \
+      pop_traced_function(__func__);  \
 } while(0)
 
 #define DBGTRC_RET_STRUCT_VALUE(_flag, _trace_group, _structname, _dbgfunc, _structval) \
@@ -529,7 +551,8 @@ do { \
              "Returning %s value:", #_structname); \
       _dbgfunc(_structval, 2); \
    } \
-   pop_traced_function(__func__);  \
+   if (traced_function_stack_enabled) \
+      pop_traced_function(__func__);  \
 } while(0)
 
 
@@ -544,7 +567,8 @@ do { \
          _dbgfunc(*_structptr_loc, 1);                                        \
       }     \
    } \
-   pop_traced_function(__func__);  \
+   if (traced_function_stack_enabled) \
+      pop_traced_function(__func__);  \
 } while(0)
 
 
