@@ -1,6 +1,6 @@
 /** @file displays.c   Monitor identifier, reference, handle  */
 
-// Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2025 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
@@ -33,6 +33,7 @@
 #include "i2c_bus_base.h"
 #include "monitor_model_key.h"
 #include "per_display_data.h"
+#include "sleep.h"
 #include "rtti.h"
 #include "vcp_version.h"
 
@@ -42,8 +43,6 @@ GPtrArray * all_display_refs = NULL;         // all detected displays, array of
 GMutex      all_display_refs_mutex;
 bool        debug_locks = false;
 
-bool      terminate_watch_thread = false;
-
 // *** DDCA_IO_Path ***
 
 /** Tests 2 #DDCA_IO_Path instances for equality
@@ -320,7 +319,6 @@ char * did_repr(Display_Identifier * pdid) {
 }
 
 
-
 /** Frees a #Display_Identifier instance
  *
  * \param pdid pointer to #Display_Identifier to free
@@ -381,7 +379,6 @@ char * io_mode_name(DDCA_IO_Mode val) {
 }
 
 
-
 /** A simple function allowing for the assignment of a value to a
  *  #DDCA_IO_Path instance in a single line of code.
  *
@@ -442,28 +439,43 @@ char * dpath_short_name_t(DDCA_IO_Path * dpath) {
  *  suitable for diagnostic messages. The returned value is valid until the
  *  next call to this function on the current thread.
  *
- *  \param  dpath  pointer to ##DDCA_IO_Path
+ *  \param  dpath  pointer to #DDCA_IO_Path
  *  \return string representation of #DDCA_IO_Path
  */
 char * dpath_repr_t(DDCA_IO_Path * dpath) {
    static GPrivate  dpath_repr_key = G_PRIVATE_INIT(g_free);
 
    char * buf = get_thread_fixed_buffer(&dpath_repr_key, 100);
-   switch(dpath->io_mode) {
-   case DDCA_IO_I2C:
-      if (dpath->path.i2c_busno == BUSNO_NOT_SET)
-         snprintf(buf, 100, "Display Path not set");
-      else
-         snprintf(buf, 100, "Display_Path[/dev/i2c-%d]", dpath->path.i2c_busno);
-      break;
-   case DDCA_IO_USB:
-      snprintf(buf, 100, "Display_Path[/dev/usb/hiddev%d]", dpath->path.hiddev_devno);
+   if (dpath) {
+      switch(dpath->io_mode) {
+      case DDCA_IO_I2C:
+         if (dpath->path.i2c_busno == BUSNO_NOT_SET)
+            snprintf(buf, 100, "Display Path not set");
+         else
+            snprintf(buf, 100, "Display_Path[/dev/i2c-%d]", dpath->path.i2c_busno);
+         break;
+      case DDCA_IO_USB:
+         snprintf(buf, 100, "Display_Path[/dev/usb/hiddev%d]", dpath->path.hiddev_devno);
+      }
    }
+   else
+      snprintf(buf, 100, "NULL Display Path");
    return buf;
 }
 
 
-// *** Display_Ref ***
+//
+// *** Published Display Reference Table
+//
+// Converts between the internal (pointer) and external (integer) display
+// reference form, and maintains a table of all display references that
+// have been exposed by the API. This table is used to validate display
+// reference arguments on API calls.
+//
+// The external display reference is the unique dref_id stored in each
+// Display_Reference when it is created. Table published_dref_hash has
+// entries whose keys are the external DDCA_Display_Ref and whose values
+// are pointers to the internal Display_Ref.
 
 static uint max_dref_id = 0;
 static GMutex max_dref_id_mutex;
@@ -606,10 +618,12 @@ DDCA_Display_Ref dref_to_ddca_dref(Display_Ref * dref) {
 }
 
 
+// *** Display_Ref ***
 
 Display_Ref * create_base_display_ref(DDCA_IO_Path io_path) {
    bool debug = false;
    DBGTRC_STARTING(debug, DDCA_TRC_BASE, "io_path=%s", dpath_repr_t(&io_path));
+
    Display_Ref * dref = calloc(1, sizeof(Display_Ref));
    memcpy(dref->marker, DISPLAY_REF_MARKER, 4);
    dref->io_path = io_path;
@@ -620,6 +634,7 @@ Display_Ref * create_base_display_ref(DDCA_IO_Path io_path) {
    // Per_Display_Data * pdd = pdd_get_per_display_data(io_path, true);
    // dref->pdd = pdd;
    g_mutex_init(&dref->access_mutex);
+
    // DBGTRC_RET_STRUCT(debug, DDCA_TRC_BASE, "Display_Ref", dbgrpt_display_ref, dref);
    DBGTRC_DONE(debug, DDCA_TRC_BASE, "Returning %p", dref);
    return dref;
@@ -635,14 +650,11 @@ Display_Ref * create_base_display_ref(DDCA_IO_Path io_path) {
 Display_Ref * create_bus_display_ref(int busno) {
    bool debug = false;
    DBGTRC_STARTING(debug, DDCA_TRC_BASE, "busno=%d", busno);
+
    DDCA_IO_Path io_path;
    io_path.io_mode   = DDCA_IO_I2C;
    io_path.path.i2c_busno = busno;
    Display_Ref * dref = create_base_display_ref(io_path);
-
-#ifdef OLD
-   dref->driver_name = get_i2c_sysfs_driver_by_busno(busno);
-#endif
    if (debug) {
       DBGMSG("Done.  Constructed bus display ref %s:", dref_repr_t(dref));
       dbgrpt_display_ref(dref, true, 0);
@@ -666,11 +678,11 @@ Display_Ref * create_usb_display_ref(int usb_bus, int usb_device, char * hiddev_
    bool debug = false;
    DBGTRC_STARTING(debug, DDCA_TRC_BASE, "usb_bus=%d, usb_device=%d, hiddev_devname=%s",
          usb_bus, usb_device, hiddev_devname);
+
    DDCA_IO_Path io_path;
    io_path.io_mode      = DDCA_IO_USB;
    io_path.path.hiddev_devno = hiddev_name_to_number(hiddev_devname);
    Display_Ref * dref = create_base_display_ref(io_path);
-
    dref->usb_bus     = usb_bus;
    dref->usb_device  = usb_device;
    dref->usb_hiddev_name = g_strdup(hiddev_devname);
@@ -684,7 +696,9 @@ Display_Ref * create_usb_display_ref(int usb_bus, int usb_device, char * hiddev_
 
 Display_Ref * copy_display_ref(Display_Ref * dref) {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_BASE, "dref=%p, iopath=%s", dref, (dref) ? dpath_repr_t(&dref->io_path) : NULL);
+   DBGTRC_STARTING(debug, DDCA_TRC_BASE, "dref=%p, iopath=%s",
+         dref, (dref) ? dpath_repr_t(&dref->io_path) : NULL);
+
    Display_Ref * copy = NULL;
    if (dref) {
       DDCA_IO_Path iopath = dref->io_path;
@@ -709,13 +723,12 @@ Display_Ref * copy_display_ref(Display_Ref * dref) {
       // do not set dfr
       // do not set actual_display
       copy->actual_display_path = dref->actual_display_path;
-#ifdef OLD
-      copy->driver_name = g_strdup(dref->driver_name);
-#endif
       // dont set pdd
       copy->drm_connector = g_strdup(dref->drm_connector);
       copy->drm_connector_id = dref->drm_connector_id;
+      copy->drm_connector_found_by = dref->drm_connector_found_by;
    }
+
    // DBGTRC_RET_STRUCT(debug, DDCA_TRC_BASE, "Display_Ref", dbgrpt_display_ref, copy);
    DBGTRC_DONE(debug, DDCA_TRC_BASE, "Returning %p", copy);
    return copy;
@@ -734,6 +747,7 @@ Display_Ref * copy_display_ref(Display_Ref * dref) {
 DDCA_Status free_display_ref(Display_Ref * dref) {
    bool debug = false;
    DBGTRC_STARTING(debug, DDCA_TRC_BASE, "dref=%p", dref);
+
    DDCA_Status ddcrc = 0;
    if (dref) {
       assert ( memcmp(dref->marker, DISPLAY_REF_MARKER, 4) == 0);
@@ -753,9 +767,6 @@ DDCA_Status free_display_ref(Display_Ref * dref) {
                free_parsed_edid(dref->pedid);  // private copy
             }
             dfr_free(dref->dfr);
-#ifdef OLD
-            free(dref->driver_name);
-#endif
             free(dref->drm_connector);
             free(dref->communication_error_summary);
             g_mutex_clear(&dref->access_mutex);
@@ -765,21 +776,57 @@ DDCA_Status free_display_ref(Display_Ref * dref) {
          }
       }
    }
+
    DBGTRC_RET_DDCRC(debug, DDCA_TRC_BASE, ddcrc, "");
    return ddcrc;
 }
 
 
-void dref_lock(Display_Ref * dref) {
+/** Locks a display reference.
+ *
+ *  Repeatedly calls g_mutex_trylock() until the lock is obtained,
+ *  or the maximum elapsed time is elapsed.
+ *
+ *  The maximum elapsed time and frequency of calls to g_mutex_trylock() is
+ *  hardcoded.
+ *
+ *  @param  dref         display refrence
+ *  @retval DDCRC_OK     lock was obtained
+ *  @retval DDCRC_LOCKED maximum elapsed time exceeded
+ */
+DDCA_Status dref_lock(Display_Ref * dref) {
    bool debug = false;
    DBGTRC_STARTING(debug, DDCA_TRC_NONE, "locking dref %s ...", dref_reprx_t(dref));
-   bool was_locked = !g_mutex_trylock(&(dref->access_mutex));
-   if (was_locked ) {
-      DBGTRC_NOPREFIX(true, DDCA_TRC_NONE, "dref %s is locked,  waiting ... ", dref_reprx_t(dref));
-      g_mutex_lock(&(dref->access_mutex));
-      DBGTRC_NOPREFIX(true, DDCA_TRC_NONE, "obtained lock on %s",  dref_reprx_t(dref));
+
+   int max_wait_millisec = 1000;
+   int wait_interval_millisec = 50;
+   int total_elapsed_millisec = 0;
+   bool lock_succeeded = false;
+   int trylock_ctr = 0;
+   while (!lock_succeeded && total_elapsed_millisec < max_wait_millisec) {
+      lock_succeeded = g_mutex_trylock(&(dref->access_mutex));
+      trylock_ctr++;
+      if (!lock_succeeded) {
+         if (total_elapsed_millisec == 0)
+            DBGTRC_NOPREFIX(true, DDCA_TRC_NONE, "dref %s is locked,  waiting ... ", dref_reprx_t(dref));
+         SLEEP_MILLIS_WITH_STATS(wait_interval_millisec);
+         total_elapsed_millisec += wait_interval_millisec;
+         wait_interval_millisec *= 2;
+      }
    }
-   DBGTRC_DONE(debug, DDCA_TRC_NONE, "dref %s", dref_reprx_t(dref));
+   DDCA_Status ddcrc = (lock_succeeded) ? 0 : DDCRC_LOCKED;
+
+   if (total_elapsed_millisec == 0)
+      SYSLOG2(DDCA_SYSLOG_DEBUG, "dref %s, lock succeeded with no wait", dref_reprx_t(dref));
+   else if (total_elapsed_millisec < max_wait_millisec)
+      SYSLOG2(DDCA_SYSLOG_DEBUG, "dref %s, lock succeeded after %d milliseconds, %d g_mutex_trylock() calls",
+              dref_reprx_t(dref), max_wait_millisec, trylock_ctr);
+   else
+      SYSLOG2(DDCA_SYSLOG_ERROR, "dref %s, max lock wait time exceeded, %d milliseconds, %d g_mutex_trylock() calls",
+            dref_reprx_t(dref), max_wait_millisec, trylock_ctr);
+
+   DBGTRC_RET_DDCRC(debug, DDCA_TRC_NONE, ddcrc, "dref %s", dref_reprx_t(dref));
+   return ddcrc;
 }
 
 
@@ -875,7 +922,6 @@ void dbgrpt_display_ref(Display_Ref * dref, bool include_businfo, int depth) {
       rpt_int("usb_device",      NULL, dref->usb_device,      d1);
       rpt_str("usb_hiddev_name", NULL, dref->usb_hiddev_name, d1);
    }
-
    rpt_vstring(d1, "vcp_version_xdf:     %s", format_vspec(dref->vcp_version_xdf) );
    rpt_vstring(d1, "vcp_version_cmdline: %s", format_vspec(dref->vcp_version_cmdline) );
    rpt_vstring(d1, "flags:               %s", interpret_dref_flags_t(dref->flags) );
@@ -884,10 +930,6 @@ void dbgrpt_display_ref(Display_Ref * dref, bool include_businfo, int depth) {
    rpt_vstring(d1, "dispno:              %d", dref->dispno);
    rpt_vstring(d1, "pedid:               %p", dref->pedid);
    report_parsed_edid(dref->pedid, /*verbose*/ false, depth+1);
-
-#ifdef OLD
-   rpt_vstring(d1, "driver:           %s", dref->driver_name);
-#endif
    rpt_vstring(d1, "actual_display:   %p", dref->actual_display);
    rpt_vstring(d1, "actual_display_path: %s",
          (dref->actual_display_path) ? dpath_repr_t(dref->actual_display_path) : "NULL");
@@ -899,17 +941,20 @@ void dbgrpt_display_ref(Display_Ref * dref, bool include_businfo, int depth) {
       }
    }
    rpt_vstring(d1, "drm_connector:   %s", dref->drm_connector);
+   rpt_vstring(d1, "drm_connector_found_by: %s",  drm_connector_found_by_name(dref->drm_connector_found_by));
    rpt_vstring(d1, "drm_connector_id: %d", dref->drm_connector_id);
    rpt_vstring(d1, "creation_timestamp: %s", formatted_time_t(dref->creation_timestamp));
 
    DBGTRC_DONE(debug, DDCA_TRC_NONE, "");
 }
 
+
 // for use by DBGTRC_RET_STRUCT()
 void dbgrpt_display_ref0(Display_Ref * dref, int depth) {
    dbgrpt_display_ref(dref, true, depth);
 }
 
+
 void dbgrpt_display_ref_summary(Display_Ref * dref, bool include_businfo, int depth) {
    bool debug = false;
    int d1 = depth+1;
@@ -939,6 +984,7 @@ void dbgrpt_display_ref_summary(Display_Ref * dref, bool include_businfo, int de
    DBGTRC_DONE(debug, DDCA_TRC_NONE, "");
 }
 
+
 /** Thread safe function that returns a short description of a #Display_Ref.
  *  The returned value is valid until the next call to this function on
  *  the current thread.
@@ -1136,6 +1182,7 @@ Display_Ref * get_dref_by_busno_or_connector(
    return result;
 }
 
+
 #ifdef UNUSED
 Display_Ref *
 ddc_get_display_ref_by_drm_connector(
@@ -1430,18 +1477,18 @@ void free_bus_open_error(Bus_Open_Error * boe) {
 // Monitor models for which DDC is disabled
 //
 
-static GPtrArray  * ddc_disabled_table = NULL;
+static GPtrArray  * ignored_mmk_table = NULL;
 
 
 /** Adds a Monitor Model Id to the list of monitors for which DDC is disabled
  *
- *  @param  mmid  monitor model key string
+ *  @param  mmid  monitor model key struct
  *  @return true  if mmid is defined, false if not
  *
  *  @remark
- *  If the **ddc_disabled_table** does not already exist, it is created.
+ *  If the **ignored_mmk_table** does not already exist, it is created.
  */
-bool add_disabled_display(Monitor_Model_Key * p_mmk) {
+bool ignore_mmk(Monitor_Model_Key * p_mmk) {
    bool debug = false;
    char * repr = NULL;
    if (debug) {
@@ -1453,18 +1500,18 @@ bool add_disabled_display(Monitor_Model_Key * p_mmk) {
    bool missing = true;
    if (p_mmk->defined) {  // if it's a valid monitor model id string
       DBGF(debug, "%s is valid:", repr);
-      if (!ddc_disabled_table)
-         ddc_disabled_table = g_ptr_array_new();
+      if (!ignored_mmk_table)
+         ignored_mmk_table = g_ptr_array_new();
       // n. g_ptr_array_find_with_equal_func() requires glib 2.54
-      for (int ndx = 0; ndx < ddc_disabled_table->len; ndx++) {
-         Monitor_Model_Key* p = g_ptr_array_index(ddc_disabled_table, ndx);
+      for (int ndx = 0; ndx < ignored_mmk_table->len; ndx++) {
+         Monitor_Model_Key* p = g_ptr_array_index(ignored_mmk_table, ndx);
          if (monitor_model_key_eq(*p_mmk, *p)) {
             missing = false;
             break;
          }
       }
       if (missing)
-         g_ptr_array_add(ddc_disabled_table, p_mmk);
+         g_ptr_array_add(ignored_mmk_table, p_mmk);
       result = true;
    }
 
@@ -1474,20 +1521,20 @@ bool add_disabled_display(Monitor_Model_Key * p_mmk) {
 }
 
 
-bool add_disabled_mmk_by_string(const char * mmid) {
+bool ignore_mmk_by_string(const char * mmid) {
    bool result = false;
    Monitor_Model_Key* p_mmk = mmk_new_from_string(mmid);
    if (p_mmk) {
-      add_disabled_display(p_mmk);
+      ignore_mmk(p_mmk);
       result = true;
    }
    return result;
 }
 
 
-void dbgrpt_ddc_disabled_table(int depth) {
+void dbgrpt_ignored_mmk_table(int depth) {
    const char * table_name = "ddc_disabled_table";
-   GPtrArray* table = ddc_disabled_table;
+   GPtrArray* table = ignored_mmk_table;
    if (table) {
       if (table->len == 0)
          rpt_vstring(depth, "%s: empty", table_name);
@@ -1509,16 +1556,16 @@ void dbgrpt_ddc_disabled_table(int depth) {
  *  @param mmk  monitor-model-id
  *  @return **true** if the display type is disabled, **false** if not
  */
-bool is_disabled_mmk(Monitor_Model_Key mmk) {
+bool is_ignored_mmk(Monitor_Model_Key mmk) {
    bool debug = false;
    DBGF(debug, "Starting. mmk=%s", mmk_repr(mmk));
 
   // dbgrpt_ddc_disabled_table(2);
 
    bool result = false;
-   if (ddc_disabled_table) {
-      for (int ndx = 0; ndx < ddc_disabled_table->len; ndx++) {
-         Monitor_Model_Key* p = g_ptr_array_index(ddc_disabled_table, ndx);
+   if (ignored_mmk_table) {
+      for (int ndx = 0; ndx < ignored_mmk_table->len; ndx++) {
+         Monitor_Model_Key* p = g_ptr_array_index(ignored_mmk_table, ndx);
          DBGF(debug, "Comparing vs p = %p -> %s", p, mmk_repr(*p));
          if (monitor_model_key_eq(mmk, *p)) {
             result = true;
