@@ -1,6 +1,6 @@
 /** @file displays.h  Display Specification  */
 
-// Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2025 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef DISPLAYS_H_
@@ -20,10 +20,10 @@
 #include "ddcutil_types_internal.h"
 #include "dynamic_features.h"
 #include "feature_set_ref.h"
+#include "i2c_bus_base.h"
 #include "monitor_model_key.h"
 #include "vcp_version.h"
 
-extern bool      terminate_watch_thread;
 
 extern GPtrArray * all_display_refs;         // all detected displays, array of Display_Ref *
 extern GMutex      all_display_refs_mutex;
@@ -35,7 +35,7 @@ Display Specification
 
 Monitors are specified in different ways in different contexts:
 
-1) Display_Identifier contains the identifiers specified on the command line.
+1) Display_Identifier represents an identifier specified on the command line.
 
 2) Display_Ref is a logical display identifier.   It can be an I2C identifier
 or a USB identifier.
@@ -59,11 +59,11 @@ void terminate_displays();
 
 #define BUSNO_NOT_SET 255
 
-char *  io_mode_name(DDCA_IO_Mode val);
-bool    dpath_eq(DDCA_IO_Path p1, DDCA_IO_Path p2);
-char *  dpath_short_name_t(DDCA_IO_Path * dpath);
-char *  dpath_repr_t(DDCA_IO_Path * dpath);  // value valid until next call
-int     dpath_hash(DDCA_IO_Path path);
+char *       io_mode_name(DDCA_IO_Mode val);
+bool         dpath_eq(DDCA_IO_Path p1, DDCA_IO_Path p2);
+char *       dpath_short_name_t(DDCA_IO_Path * dpath);
+char *       dpath_repr_t(DDCA_IO_Path * dpath);  // value valid until next call
+int          dpath_hash(DDCA_IO_Path path);
 DDCA_IO_Path i2c_io_path(int busno);
 DDCA_IO_Path usb_io_path(int hiddev_devno);
 
@@ -198,22 +198,20 @@ typedef struct _display_ref {
    uint64_t                 next_i2c_io_after;     // nanosec
    struct _display_ref *    actual_display;        // if dispno == -2
    DDCA_IO_Path *           actual_display_path;   // alt to actual_display
-#ifdef OLD
-   char *                   driver_name;           //
-#endif
    struct Per_Display_Data* pdd;
    char *                   drm_connector;         // e.g. card0-HDMI-A-1  // REDUNDANT - IDENTICAL TO Bus_Info.drm_connector
    int                      drm_connector_id;      // identical to Bus_Info.drm_connector_id
+   Drm_Connector_Found_By   drm_connector_found_by;  // identical to Bus_Info.drm_connector_found_by
    char *                   communication_error_summary;
    uint64_t                 creation_timestamp;
    GMutex                   access_mutex;
 } Display_Ref;
 
 
-void dbgrpt_published_dref_hash(const char * msg, int depth);
-void init_published_dref_hash();
-void reset_published_dref_hash();
-void add_published_dref_id_by_dref(Display_Ref * dref);
+void             dbgrpt_published_dref_hash(const char * msg, int depth);
+void             init_published_dref_hash();
+void             reset_published_dref_hash();
+void             add_published_dref_id_by_dref(Display_Ref * dref);
 Display_Ref *    dref_from_published_ddca_dref(DDCA_Display_Ref ddca_dref);
 DDCA_Display_Ref dref_to_ddca_dref(Display_Ref * dref);
 
@@ -236,7 +234,7 @@ char *        dref_reprx_t(Display_Ref * dref);  // value valid until next call
 char *        ddci_dref_repr_t(DDCA_Display_Ref * ddca_dref);  // value valid until next call
 DDCA_Status   free_display_ref(Display_Ref * dref);
 Display_Ref * copy_display_ref(Display_Ref * dref);
-void          dref_lock(Display_Ref * dref);
+DDCA_Status   dref_lock(Display_Ref * dref);
 void          dref_unlock(Display_Ref * dref);
 
 // Do two Display_Ref's identify the same device?
@@ -311,10 +309,9 @@ typedef enum {
 
 const char * watch_mode_name(DDC_Watch_Mode mode);
 
-bool add_disabled_display(Monitor_Model_Key * mmk);
-bool add_disabled_mmk_by_string(const char * mmid);
-void dbgrpt_ddc_disabled_table(int depth);
-bool is_disabled_mmk(Monitor_Model_Key mmk);
-
+bool ignore_mmk(Monitor_Model_Key * mmk);
+bool ignore_mmk_by_string(const char * mmid);
+void dbgrpt_ignored_mmk_table(int depth);
+bool is_ignored_mmk(Monitor_Model_Key mmk);
 
 #endif /* DISPLAYS_H_ */
