@@ -4,7 +4,7 @@
  * display-specific feature metadata.
  */
 
-// Copyright (C) 2018-2023 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2018-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -15,6 +15,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "util/debug_util.h"
 #include "util/glib_util.h"
 #include "util/report_util.h"
 /** \endcond */
@@ -53,6 +54,7 @@ char * nontable_vcp_value_repr_t(Nontable_Vcp_Value * vcp_value) {
 
 // Feature flags
 
+#ifdef UNUSED
 /** Creates a string representation of DDCA_Feature_Flags bitfield.
  *
  *  @param  flags      feature characteristics
@@ -67,13 +69,14 @@ interpret_feature_flags_t(DDCA_Version_Feature_Flags flags) {
    static GPrivate  buf_key = G_PRIVATE_INIT(g_free);
    char * buffer = get_thread_fixed_buffer(&buf_key, 100);
 
-   g_snprintf(buffer, 100, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
+   g_snprintf(buffer, 100, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
        flags & DDCA_RO               ? "Read-Only, "                   : "",
        flags & DDCA_WO               ? "Write-Only, "                  : "",
        flags & DDCA_RW               ? "Read-Write, "                  : "",
        flags & DDCA_STD_CONT         ? "Continuous (standard), "       : "",
        flags & DDCA_COMPLEX_CONT     ? "Continuous (complex), "        : "",
        flags & DDCA_SIMPLE_NC        ? "Non-Continuous (simple), "     : "",
+       flags & DDCA_EXTENDED_NC      ? "Non-Continuous (extended), "   : "",
        flags & DDCA_COMPLEX_NC       ? "Non-Continuous (complex), "    : "",
        flags & DDCA_NC_CONT          ? "Non-Continuous with continuous subrange, " :"",
        flags & DDCA_WO_NC            ? "Non-Continuous (write-only), " : "",
@@ -94,8 +97,125 @@ interpret_feature_flags_t(DDCA_Version_Feature_Flags flags) {
 
    return buffer;
 }
+#endif
 
 
+/** Creates a string representation of DDCA_Feature_Flags bitfield.
+ *
+ *  @param  flags      feature characteristics
+ *  @return string representation, valid until the next call
+ *          of this function in the current thread, do not free
+ *
+ *  @remark
+ *  DDCA_Feature_Flags is a union (DDCA_Version_Feature_Flags,DDCA_Global_Feature_Flags)
+ *  All are defined as uint16_t, so this function can be used to interpret
+ *  DDCA_Version_Feature_Flags and DDCA_Global_Feature_Flags as well as
+ *  DDCA_Feature_Flags.
+ */
+const char *
+interpret_ddca_feature_flags_symbolic_t(DDCA_Feature_Flags flags) {
+   bool debug = false;
+
+   static GPrivate  buf_key = G_PRIVATE_INIT(g_free);
+   char * buffer = get_thread_fixed_buffer(&buf_key, 100);
+
+   g_snprintf(buffer, 100, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
+       // Exactly 1 of the following should be set in DDCA_Version_Feature_Flags:
+       flags & DDCA_RO                  ? "DDCA_RO|"             : "",
+       flags & DDCA_WO                  ? "DDCA_WO|"             : "",
+       flags & DDCA_RW                  ? "DDCA_RW|"             : "",
+       // Exactly 1 of the following should be set in DDCA_Version_Feature_Flags:
+       flags & DDCA_STD_CONT            ? "DDCA_STD_CONT|"       : "",
+       flags & DDCA_COMPLEX_CONT        ? "DDCA_COMPLEX_CONT|"   : "",
+       flags & DDCA_SIMPLE_NC           ? "DDCA_SIMPLE_NC|"      : "",
+       flags & DDCA_EXTENDED_NC         ? "DDCA_EXTENDED_NC|"    : "",
+       flags & DDCA_COMPLEX_NC          ? "DDCA_COMPLEX_NC|"     : "",
+       flags & DDCA_NC_CONT             ? "DDCA_NC_CONT|"        : "",
+       flags & DDCA_WO_NC               ? "DDCA_WO_CONT|"        : "",
+       flags & DDCA_NORMAL_TABLE        ? "DDCA_NORMAL_TABLE|"   : "",
+       flags & DDCA_WO_TABLE            ? "DDCA_WO_TABLE|"       : "",
+       flags & DDCA_DEPRECATED          ? "DDCA_DEPRECATED|"     : "",
+
+       // Lifecycle in DDCA_Global_Feature_Flags:
+       flags & DDCA_PERSISTENT_METADATA ? "DDCA_PERSISTENT_METADATA|"    : "",
+       flags & DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY
+                                        ? "DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY|" : "",
+       // Provenance in DDCA_Global_Feature_Flags:
+       flags & DDCA_USER_DEFINED        ? "DDCA_USER_DEFINED|"   : "",
+       flags & DDCA_SYNTHETIC           ? "DDCA_SYNTHESIZED|"    : ""
+
+   );
+   // remove final comma and blank
+   if (strlen(buffer) > 0)
+      buffer[strlen(buffer)-1] = '\0';
+
+   DBGF(debug, "flags=0x%04x, returning %s", flags, buffer);
+
+   return buffer;
+}
+
+const char *
+interpret_ddca_global_feature_flags_symbolic_t(DDCA_Feature_Flags flags) {
+   bool debug = false;
+
+   static GPrivate  buf_key = G_PRIVATE_INIT(g_free);
+   char * buffer = get_thread_fixed_buffer(&buf_key, 100);
+
+   g_snprintf(buffer, 100, "%s%s%s%s",
+
+
+       // Lifecycle in DDCA_Global_Feature_Flags:
+       flags & DDCA_PERSISTENT_METADATA ? "DDCA_PERSISTENT_METADATA|"    : "",
+       flags & DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY
+                                        ? "DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY|" : "",
+       // Provenance in DDCA_Global_Feature_Flags:
+       flags & DDCA_USER_DEFINED        ? "DDCA_USER_DEFINED|"   : "",
+       flags & DDCA_SYNTHETIC           ? "DDCA_SYNTHESIZED|"    : ""
+
+   );
+   // remove final comma and blank
+   if (strlen(buffer) > 0)
+      buffer[strlen(buffer)-1] = '\0';
+
+   DBGF(debug, "flags=0x%04x, returning %s", flags, buffer);
+
+   return buffer;
+}
+
+
+const char *
+interpret_ddca_version_feature_flags_symbolic_t(DDCA_Feature_Flags flags) {
+   bool debug = false;
+
+   static GPrivate  buf_key = G_PRIVATE_INIT(g_free);
+   char * buffer = get_thread_fixed_buffer(&buf_key, 100);
+
+   g_snprintf(buffer, 100, "%s%s%s%s%s%s%s%s%s%s%s%s%s",
+       // Exactly 1 of the following should be set in DDCA_Version_Feature_Flags:
+       flags & DDCA_RO                  ? "DDCA_RO|"             : "",
+       flags & DDCA_WO                  ? "DDCA_WO|"             : "",
+       flags & DDCA_RW                  ? "DDCA_RW|"             : "",
+       // Exactly 1 of the following should be set in DDCA_Version_Feature_Flags:
+       flags & DDCA_STD_CONT            ? "DDCA_STD_CONT|"       : "",
+       flags & DDCA_COMPLEX_CONT        ? "DDCA_COMPLEX_CONT|"   : "",
+       flags & DDCA_SIMPLE_NC           ? "DDCA_SIMPLE_NC|"      : "",
+       flags & DDCA_EXTENDED_NC         ? "DDCA_EXTENDED_NC|"    : "",
+       flags & DDCA_COMPLEX_NC          ? "DDCA_COMPLEX_NC|"     : "",
+       flags & DDCA_NC_CONT             ? "DDCA_NC_CONT|"        : "",
+       flags & DDCA_WO_NC               ? "DDCA_WO_CONT|"        : "",
+       flags & DDCA_NORMAL_TABLE        ? "DDCA_NORMAL_TABLE|"   : "",
+       flags & DDCA_WO_TABLE            ? "DDCA_WO_TABLE|"       : "",
+       flags & DDCA_DEPRECATED          ? "DDCA_DEPRECATED|"     : ""
+   );
+   // remove final comma and blank
+   if (strlen(buffer) > 0)
+      buffer[strlen(buffer)-1] = '\0';
+
+   DBGF(debug, "flags=0x%04x, returning %s", flags, buffer);
+
+   return buffer;
+}
+
 
 // SL value tables
 
@@ -257,7 +377,6 @@ sl_value_table_lookup(
 
 
 
-
 /** Output a debug report of a #Dyn_Feature_Metadata instance
  *
  *  @param  md     instance to report
@@ -275,13 +394,14 @@ dbgrpt_dyn_feature_metadata(
    rpt_vstring(d1, "MCCS version:      %d.%d",  md->vcp_version.major, md->vcp_version.minor);
    rpt_vstring(d1, "Feature name:      %s",     md->feature_name);
    rpt_vstring(d1, "Description:       %s",     md->feature_desc);
-   rpt_vstring(d1, "Feature flags:     0x%04x", md->feature_flags);
-   rpt_vstring(d1, "Interpreted flags: %s", interpret_feature_flags_t(md->feature_flags));
+   rpt_vstring(d1, "Global feature flags:     0x%04x", md->global_feature_flags);
+   rpt_vstring(d1, "Interpreted global feature flags: %s", interpret_ddca_global_feature_flags_symbolic_t(md->global_feature_flags));
+   rpt_vstring(d1, "Version feature flags:     0x%04x", md->version_feature_flags);
+   rpt_vstring(d1, "Interpreted version feature flags: %s", interpret_ddca_version_feature_flags_symbolic_t(md->version_feature_flags));
    dbgrpt_sl_value_table(md->sl_values, "Feature values", d1);
 }
 
 
-
 //
 // Display_Feature_Metadata (used internally)
 //
@@ -307,8 +427,10 @@ dbgrpt_display_feature_metadata(
                       meta->vcp_version.major, meta->vcp_version.minor, format_vspec(meta->vcp_version));
       rpt_vstring(d1, "feature_name:    %s", meta->feature_name);
       rpt_vstring(d1, "feature_desc:    %s", meta->feature_desc);
-      char * s = interpret_feature_flags_t(meta->feature_flags);
-      rpt_vstring(d1, "flags:           0x%04x = %s", meta->feature_flags, s);
+      const char * s = interpret_ddca_global_feature_flags_symbolic_t(meta->global_feature_flags);
+      rpt_vstring(d1, "global flags:    0x%04x = %s", meta->global_feature_flags, s);
+      const char * t = interpret_ddca_version_feature_flags_symbolic_t(meta->version_feature_flags);
+      rpt_vstring(d1, "version flags:    0x%04x = %s", meta->version_feature_flags, t);
       dbgrpt_sl_value_table(meta->sl_values, "Feature values", d1);
       rpt_vstring(d1, "nontable_formatter:           %p - %s",
                       meta->nontable_formatter,
@@ -336,11 +458,11 @@ dfm_free(
 {
    bool debug = false;
    DBGTRC_STARTING(debug, DDCA_TRC_VCP, "meta=%p", meta);
-   if (debug)
-      dbgrpt_display_feature_metadata(meta, 2);
+   // if (debug)
+   //    dbgrpt_display_feature_metadata(meta, 2);
    if (meta) {
-      assert(memcmp(meta->marker, DISPLAY_FEATURE_METADATA_MARKER, 4) == 0);
       DBGTRC_NOPREFIX(debug, DDCA_TRC_VCP, "feature_code = 0x%02x", meta->feature_code);
+      assert(memcmp(meta->marker, DISPLAY_FEATURE_METADATA_MARKER, 4) == 0);
       meta->marker[3] = 'x';
       free(meta->feature_name);
       free(meta->feature_desc);
@@ -390,7 +512,7 @@ void dfm_set_feature_desc(Display_Feature_Metadata * meta, const char * feature_
 /** Converts a #Dyn_Feature_Metadata record, representing user supplied
  *  metadata, to a #Display_Feature_Metadata.
  *
- *  @param ddca_meta  instance to convert
+ *  @param  dyn_meta  instance to convert
  *  @result newly created #Display_Feature_Metadata
  *
  *  @remark
@@ -398,26 +520,40 @@ void dfm_set_feature_desc(Display_Feature_Metadata * meta, const char * feature_
  */
 Display_Feature_Metadata *
 dfm_from_dyn_feature_metadata(
-      Dyn_Feature_Metadata * ddca_meta)
+      Dyn_Feature_Metadata * dyn_meta)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting");
-   assert(ddca_meta);
-   assert(memcmp(ddca_meta->marker, DDCA_FEATURE_METADATA_MARKER, 4) == 0);
+   DBGTRC_STARTING(debug, DDCA_TRC_NONE, "ddc_meta=%p", dyn_meta);
+   assert(dyn_meta);
+   assert(memcmp(dyn_meta->marker, DDCA_FEATURE_METADATA_MARKER, 4) == 0);
+   if (debug)
+      dbgrpt_dyn_feature_metadata(dyn_meta, 2);
 
-   Display_Feature_Metadata * dfm = dfm_new(ddca_meta->feature_code);
+   Display_Feature_Metadata * dfm = dfm_new(dyn_meta->feature_code);
    dfm->display_ref = NULL;
-   dfm->feature_desc = (ddca_meta->feature_desc) ? g_strdup(ddca_meta->feature_desc) : NULL;
-   dfm->feature_name = (ddca_meta->feature_name) ? g_strdup(ddca_meta->feature_name) : NULL;
-   // dfm->feature_flags = ddca_meta->feature_flags & ~DDCA_SYNTHETIC_DDCA_FEATURE_METADATA;
-   dfm->feature_flags = ddca_meta->feature_flags & ~DDCA_PERSISTENT_METADATA;
+   dfm->feature_desc = (dyn_meta->feature_desc) ? g_strdup(dyn_meta->feature_desc) : NULL;
+   dfm->feature_name = (dyn_meta->feature_name) ? g_strdup(dyn_meta->feature_name) : NULL;
+
+   // ensure global flag values also defined as version flag values are not used:
+   assert(!(dyn_meta->global_feature_flags & DDCA_SYNTHETIC));
+   assert(!(dyn_meta->global_feature_flags & DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY));
+
+   assert(dyn_meta->global_feature_flags & DDCA_USER_DEFINED);
+   assert(dyn_meta->global_feature_flags & DDCA_PERSISTENT_METADATA);
+
+   dfm->global_feature_flags = dyn_meta->global_feature_flags;
+   dfm->version_feature_flags =  dyn_meta->version_feature_flags;
+
    dfm->nontable_formatter = NULL;
    dfm->nontable_formatter_sl = NULL;
    dfm->table_formatter = NULL;
    dfm->vcp_version =  DDCA_VSPEC_UNQUERIED;
-   dfm->sl_values = copy_sl_value_table(ddca_meta->sl_values);
+   dfm->sl_values = copy_sl_value_table(dyn_meta->sl_values);
    // dfm->latest_sl_values = copy_sl_value_table(ddca_meta->latest_sl_values);
-   DBGMSF(debug, "Done. dfm=%p");
+
+   if (debug)
+      dbgrpt_display_feature_metadata(dfm, 2);
+   DBGTRC_DONE(debug, DDCA_TRC_NONE, "Returning dfm=%p", dfm);
    return dfm;
 }
 
@@ -444,7 +580,7 @@ dbgrpt_ddca_feature_metadata(
    rpt_vstring(d1, "Feature name:      %s",     md->feature_name);
    rpt_vstring(d1, "Description:       %s",     md->feature_desc);
    rpt_vstring(d1, "Feature flags:     0x%04x", md->feature_flags);
-   rpt_vstring(d1, "Interpreted flags: %s", interpret_feature_flags_t(md->feature_flags));
+   rpt_vstring(d1, "Interpreted flags: %s", interpret_ddca_feature_flags_symbolic_t(md->feature_flags));
    dbgrpt_sl_value_table(md->sl_values, "Feature values", d1);
 }
 
@@ -462,7 +598,7 @@ dfm_to_ddca_feature_metadata(
       Display_Feature_Metadata * dfm)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting. dfm=%p", dfm);
+   DBGTRC_STARTING(debug, DDCA_TRC_NONE, "dfm=%p", dfm);
    if (debug)
       dbgrpt_display_feature_metadata(dfm, 2);
 
@@ -470,15 +606,18 @@ dfm_to_ddca_feature_metadata(
    memcpy(ddca_meta->marker, DDCA_FEATURE_METADATA_MARKER, 4);
    ddca_meta->feature_code  = dfm->feature_code;
    ddca_meta->vcp_version   = dfm->vcp_version;
-   ddca_meta->feature_flags = dfm->feature_flags;
-   ddca_meta->feature_flags &= ~DDCA_PERSISTENT_METADATA;
-   ddca_meta->feature_flags &= ~DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY;
+   ddca_meta->feature_flags = dfm->version_feature_flags;
+   if (dfm->global_feature_flags & DDCA_PERSISTENT_METADATA)
+      ddca_meta->feature_flags |= DDCA_PERSISTENT_METADATA;
+   // ddca_meta->feature_flags &= ~DDCA_PERSISTENT_METADATA;
+   // ddca_meta->feature_flags &= ~DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY;
    ddca_meta->feature_name = (dfm->feature_name) ? g_strdup(dfm->feature_name) : NULL;
    ddca_meta->feature_desc = (dfm->feature_desc) ? g_strdup(dfm->feature_desc) : NULL;
    DBGMSF(debug, "** dfm->sl_values = %p", dfm->sl_values);
    ddca_meta->sl_values = copy_sl_value_table(dfm->sl_values);
 
-   DBGMSF_RET_STRUCT(debug, DDCA_Feature_Metadata, dbgrpt_ddca_feature_metadata, ddca_meta);
+   DBGTRC_RET_STRUCT(debug, DDCA_TRC_NONE, DDCA_Feature_Metadata,
+                     dbgrpt_ddca_feature_metadata, ddca_meta);
    return ddca_meta;
 }
 
@@ -510,6 +649,7 @@ free_ddca_feature_metadata(DDCA_Feature_Metadata * metadata) {
 
 void init_feature_metadata() {
    RTTI_ADD_FUNC(dfm_free);
-   RTTI_ADD_FUNC(init_feature_metadata);
+   RTTI_ADD_FUNC(dfm_from_dyn_feature_metadata);
+   RTTI_ADD_FUNC(dfm_to_ddca_feature_metadata);
 }
 
