@@ -508,8 +508,10 @@ static bool parse_watch_mode(
 
       if (     is_abbrev(v2, "POLL", 3))
          parsed_cmd->watch_mode = Watch_Mode_Poll;
+#ifdef USE_X11
       else if (is_abbrev(v2, "XEVENT", 3))
          parsed_cmd->watch_mode = Watch_Mode_Xevent;
+#endif
    // else if (is_abbrev(v2, "UDEV", 3))
    //    parsed_cmd->watch_mode = Watch_Mode_Udev;
       else if (is_abbrev(v2, "DYNAMIC", 3))
@@ -920,8 +922,8 @@ parse_command(
                                                         : "Verify value set by setvcp";
    const char * noverify_expl = (DEFAULT_SETVCP_VERIFY) ? "Do not verify value by setvcp"
                                                         : "Do not verify value set by setvcp (default)";
-   gboolean verify_flag    = DEFAULT_SETVCP_VERIFY;
-   gboolean noverify_flag  = false;
+   gboolean verify_set_flag    = false;
+   gboolean noverify_set_flag  = false;
    gboolean async_flag     = false;
    // gboolean async_check_i2c_flag = true;
    gboolean report_freed_excp_flag = false;
@@ -948,7 +950,9 @@ parse_command(
    gboolean parse_only_flag    = false;
    gboolean x52_no_fifo_flag   = false;
    gboolean enable_dsa2_flag   = DEFAULT_ENABLE_DSA2;
-   gboolean traced_function_stack_flag = false;
+   gboolean enable_tfs_flag    = DEFAULT_ENABLE_TRACED_FUNCTION_STACK;
+   const char * enable_tfs_expl  = (DEFAULT_ENABLE_TRACED_FUNCTION_STACK) ? "Enable Traced Function Stack (default)" : "Enable Traced Function Stack";
+   const char * disable_tfs_expl = (DEFAULT_ENABLE_TRACED_FUNCTION_STACK) ? "Disable Traced Function Stack" : "Disable Traced Function Stack (default)";
    gboolean traced_function_stack_errors_fatal_flag = false;
    // int      i2c_bus_check_async_min = DEFAULT_I2C_BUS_CHECK_ASYNC_MIN;
    // int      ddc_check_async_min = DEFAULT_DDC_CHECK_ASYNC_MIN;
@@ -1017,6 +1021,7 @@ parse_command(
       disable_tgefs_expl = "do not try to get EDID from /sys (default)";
    }
 
+#ifdef WATCH_DISPLAYS
    DDC_Watch_Mode default_watch_mode = DEFAULT_WATCH_MODE;
    char * default_watch_mode_keyword;
    switch(default_watch_mode) {
@@ -1026,10 +1031,17 @@ parse_command(
    case Watch_Mode_Udev:     default_watch_mode_keyword = "UDEV";    break;
    }
    char watch_mode_expl[80];
+#ifdef USE_X11
    g_snprintf(watch_mode_expl, 80, "DYNAMIC|XEVENT|POLL, default: %s", default_watch_mode_keyword);
+#else
+   g_snprintf(watch_mode_expl, 80, "DYNAMIC|POLL, default: %s", default_watch_mode_keyword);
+#endif
    gboolean enable_watch_displays = true;
+#ifdef USE_X11
    gint     xevent_watch_loop_millis_work = DEFAULT_XEVENT_WATCH_LOOP_MILLISEC;
+#endif
    gint     poll_watch_loop_millis_work = DEFAULT_POLL_WATCH_LOOP_MILLISEC;
+#endif
 
    gboolean f1_flag         = false;
    gboolean f2_flag         = false;
@@ -1253,8 +1265,8 @@ parse_command(
 
       // Behavior options
       {"maxtries",'\0', 0, G_OPTION_ARG_STRING,   &maxtrywork,       "Max try adjustment",  "comma separated list" },
-      {"verify",  '\0', 0, G_OPTION_ARG_NONE,     &verify_flag,      verify_expl,        NULL},
-      {"noverify",'\0', 0, G_OPTION_ARG_NONE,     &noverify_flag,    noverify_expl,      NULL},
+      {"verify",  '\0', 0, G_OPTION_ARG_NONE,     &verify_set_flag,   verify_expl,        NULL},
+      {"noverify",'\0', 0, G_OPTION_ARG_NONE,     &noverify_set_flag, noverify_expl,      NULL},
 
       {"mccs",    '\0', 0, G_OPTION_ARG_STRING,   &mccswork,         "Tailor feature handling to specific MCCS version",   "major.minor" },
 
@@ -1281,17 +1293,21 @@ parse_command(
                             G_OPTION_ARG_NONE,    &try_get_edid_from_sysfs,   enable_tgefs_expl, NULL},
       {"disable-try-get-edid-from-sysfs", '\0', G_OPTION_FLAG_REVERSE,
                            G_OPTION_ARG_NONE,     &try_get_edid_from_sysfs,   disable_tgefs_expl, NULL},
+#ifdef WATCH_DISPLAYS
 //      {"enable-watch-displays",  '\0', 0, G_OPTION_ARG_NONE, &watch_displays_flag, "Watch for display hotplug events", NULL },
       {"disable-watch-displays", '\0', G_OPTION_FLAG_REVERSE,
                                 G_OPTION_ARG_NONE, &enable_watch_displays, "Do not watch for display change events", NULL },
-      {"disable-api", '\0', G_OPTION_FLAG_HIDDEN,
-                      G_OPTION_ARG_NONE, &disable_api_flag, "Completely disable API", NULL },
       {"watch-mode", '\0', G_OPTION_FLAG_HIDDEN,
                            G_OPTION_ARG_STRING, &watch_mode_work, "How to watch for display changes",  watch_mode_expl},
+#ifdef USE_X11
       {"xevent-watch-loop-millisec", '\0', G_OPTION_FLAG_HIDDEN,
                            G_OPTION_ARG_INT, &xevent_watch_loop_millis_work, "Loop delay for mode XEVENT", "milliseconds"},
+#endif
       {"poll-watch-loop-millisec", '\0', G_OPTION_FLAG_HIDDEN,
                            G_OPTION_ARG_INT, &poll_watch_loop_millis_work, "Loop delay for mode POLL", "milliseconds"},
+#endif
+      {"disable-api", '\0', G_OPTION_FLAG_HIDDEN,
+                                           G_OPTION_ARG_NONE, &disable_api_flag, "Completely disable API", NULL },
 #ifdef ENABLE_USB
       {"enable-usb", '\0', G_OPTION_FLAG_NONE,
                                G_OPTION_ARG_NONE, &enable_usb_flag,  enable_usb_expl, NULL},
@@ -1348,8 +1364,11 @@ parse_command(
       {"trcfunc",    '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING_ARRAY, &parsed_cmd->traced_functions,  "Trace functions","function name" },
       {"trcfrom",    '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING_ARRAY, &parsed_cmd->traced_calls,      "Trace call stack from function","function name" },
       {"trcfile",    '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING_ARRAY, &parsed_cmd->traced_files,      "Trace files",    "file name" },
-      {"enable-traced-function-stack",
-                  '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &traced_function_stack_flag, "Enable traced function stack", NULL},
+      {"enable-traced-function-stack", '\0', G_OPTION_FLAG_NONE,
+                                           G_OPTION_ARG_NONE, &enable_tfs_flag,  enable_tfs_expl, NULL},
+      {"disable-traced-function-stack",'\0', G_OPTION_FLAG_REVERSE,
+                                           G_OPTION_ARG_NONE, &enable_tfs_flag,  disable_tfs_expl, NULL},
+
 //    {"traced-function-stack-errors-fatal",
 //                '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &traced_function_stack_errors_fatal_flag, "Traced function stack errors are fatal", NULL},
 
@@ -1646,9 +1665,16 @@ parse_command(
       EMIT_PARSER_ERROR(errmsgs, "Deprecated option ignored: --async.");
       EMIT_PARSER_ERROR(errmsgs, "Use --i2c-bus-checks-async-min (experimental) or --ddc-checks-async-min");
    }
-   if (verify_flag && noverify_flag) {
+   if (verify_set_flag && noverify_set_flag) {
       EMIT_PARSER_ERROR(errmsgs, "Both --verify and --noverify specified");
+      parsing_ok = false;
    }
+   bool verify_flag = DEFAULT_SETVCP_VERIFY;
+   if (verify_set_flag)
+      verify_flag = true;
+   else if (noverify_set_flag)
+      verify_flag = false;
+
 
 #define LIBDDCUTIL_ONLY_OPTION(_name,_val) \
    do \
@@ -1662,7 +1688,9 @@ parse_command(
       LIBDDCUTIL_ONLY_OPTION("--trcapi",                parsed_cmd->traced_api_calls);
       LIBDDCUTIL_ONLY_OPTION("--profile-api",           profile_api_flag);
       LIBDDCUTIL_ONLY_OPTION("--libddcutil-trace-file", parsed_cmd->trace_destination);
+#ifdef WATCH_DISPLAYS
       LIBDDCUTIL_ONLY_OPTION("--disable-watch-displays", !enable_watch_displays);
+#endif
       LIBDDCUTIL_ONLY_OPTION("--disable-api",           disable_api_flag);
    }
 
@@ -1707,9 +1735,7 @@ parse_command(
    SET_CMDFLAG(CMD_FLAG_WALLTIME_TRACE,    wall_timestamp_trace_flag);
    SET_CMDFLAG(CMD_FLAG_THREAD_ID_TRACE,   thread_id_trace_flag);
    SET_CMDFLAG(CMD_FLAG_PROCESS_ID_TRACE,  process_id_trace_flag);
-   SET_CMDFLAG(CMD_FLAG_VERIFY,            verify_flag || !noverify_flag);
-   // if (verify_flag || !noverify_flag)
-   //    parsed_cmd->flags |= CMD_FLAG_VERIFY;
+   SET_CMDFLAG(CMD_FLAG_VERIFY,            verify_flag);
 #ifdef OLD
    SET_CMDFLAG(CMD_FLAG_NODETECT,          nodetect_flag);
    SET_CMDFLAG(CMD_FLAG_ASYNC_I2C_CHECK,   async_check_i2c_flag);
@@ -1732,7 +1758,9 @@ parse_command(
    SET_CMDFLAG(CMD_FLAG_DSA2,              enable_dsa2_flag);
    SET_CMDFLAG(CMD_FLAG_DEFER_SLEEPS,      deferred_sleep_flag);
 
+#ifdef WATCH_DISPLAYS
    SET_CMDFLAG(CMD_FLAG_WATCH_DISPLAY_EVENTS,    enable_watch_displays);
+#endif
    SET_CMDFLAG(CMD_FLAG_DISABLE_API,       disable_api_flag);
    SET_CMDFLAG(CMD_FLAG_X52_NO_FIFO,       x52_no_fifo_flag);
    SET_CMDFLAG(CMD_FLAG_SHOW_SETTINGS,     show_settings_flag);
@@ -1748,7 +1776,7 @@ parse_command(
    SET_CMDFLAG(CMD_FLAG_SKIP_DDC_CHECKS,   skip_ddc_checks_flag);
    SET_CMDFLAG(CMD_FLAG_FLOCK,             enable_flock_flag);
    SET_CMDFLAG(CMD_FLAG_ENABLE_TRACED_FUNCTION_STACK,
-                                           traced_function_stack_flag);
+                                           enable_tfs_flag);
    SET_CMDFLAG(CMD_FLAG_TRACED_FUNCTION_STACK_ERRORS_FATAL,
                                            traced_function_stack_errors_fatal_flag);
 
@@ -2054,6 +2082,8 @@ parse_command(
       }
    }
 
+#ifdef WATCH_DISPLAYS
+#ifdef USE_X11
    if (xevent_watch_loop_millis_work <= 0) {
       EMIT_PARSER_ERROR(errmsgs,
             "--xevent-watch-loop-millisec not a positive number: %d", xevent_watch_loop_millis_work);
@@ -2061,6 +2091,7 @@ parse_command(
    }
    else
       parsed_cmd->xevent_watch_loop_millisec = (uint16_t) xevent_watch_loop_millis_work;
+#endif
 
    if (poll_watch_loop_millis_work <= 0) {
       EMIT_PARSER_ERROR(errmsgs,
@@ -2069,7 +2100,7 @@ parse_command(
    }
    else
       parsed_cmd->poll_watch_loop_millisec = (uint16_t) poll_watch_loop_millis_work;
-
+#endif
 
    // All options processed.  Check for consistency, set defaults
 
