@@ -17,6 +17,7 @@
 #include "config.h"
 
 #include "util/debug_util.h"
+#include "util/drm_card_connector_util.h"
 #ifdef ENABLE_FAILSIM
 #include "util/failsim.h"
 #endif
@@ -26,15 +27,13 @@
 #include "util/sysfs_i2c_util.h"
 #include "util/timestamp.h"
 #include "util/traced_function_stack.h"
-#ifdef USE_LIBDRM
-#include "util/drm_common.h"
-#include "util/libdrm_util.h"
-#endif
 
 #include "base/core.h"
 #include "base/display_retry_data.h"
 #include "base/dsa2.h"
+#ifdef USE_LIBDRM
 #include "base/drm_connector_state.h"
+#endif
 #include "base/flock.h"
 #include "base/i2c_bus_base.h"
 #include "base/parms.h"
@@ -65,10 +64,12 @@
 #include "ddc_try_data.h"
 #include "ddc_vcp.h"
 
+#ifdef WATCH_DISPLAYS
 #include "dw/dw_common.h"
 #include "dw/dw_main.h"
 #include "dw/dw_poll.h"
 #include "dw/dw_udev.h"
+#endif
 
 #include "ddc_common_init.h"
 
@@ -193,6 +194,7 @@ init_tracing(Parsed_Cmd * parsed_cmd)
    g_ptr_array_free(errinfo_accumulator, true);
 
    traced_function_stack_enabled = parsed_cmd->flags & CMD_FLAG_ENABLE_TRACED_FUNCTION_STACK;
+   DBGF(debug, "traced_function_stack_enabled=&s", SBOOL(traced_function_stack_enabled));
    traced_function_stack_errors_fatal = parsed_cmd->flags & CMD_FLAG_TRACED_FUNCTION_STACK_ERRORS_FATAL;
    if (parsed_cmd->flags2 & CMD_FLAG2_F26)
       traced_function_stack_errors_fatal = true;
@@ -210,7 +212,7 @@ STATIC Error_Info * init_disabled_displays(Parsed_Cmd * parsed_cmd) {
     for (int ndx = 0; ndx < ntsa_length(parsed_cmd->ddc_disabled); ndx++) {
           // DBGF(debug, "Adding disabled_mmid: %s", parsed_cmd->ddc_disabled[ndx]);
           char * cur_mmid = parsed_cmd->ddc_disabled[ndx];
-          bool found = add_disabled_mmk_by_string(cur_mmid);
+          bool found = ignore_mmk_by_string(cur_mmid);
           if (!found) {
              Error_Info * err = errinfo_new(DDCRC_CONFIG_ERROR, "Invalid mmid: %s", cur_mmid);
              g_ptr_array_add(errinfo_accumulator, err);
@@ -219,7 +221,7 @@ STATIC Error_Info * init_disabled_displays(Parsed_Cmd * parsed_cmd) {
     }
 
    if (debug)
-      dbgrpt_ddc_disabled_table(2);
+      dbgrpt_ignored_mmk_table(2);
 
    if (errinfo_accumulator->len > 0)
       errinfo = errinfo_new_with_causes_gptr(
@@ -374,7 +376,7 @@ init_performance_options(Parsed_Cmd * parsed_cmd)
    DBGTRC_DONE(debug, DDCA_TRC_NONE, "");
 }
 
-
+#ifdef WATCH_DISPLAYS
 STATIC void
 init_display_watch_options(Parsed_Cmd* parsed_cmd) {
    watch_displays_mode        = parsed_cmd->watch_mode;
@@ -391,13 +393,16 @@ init_display_watch_options(Parsed_Cmd* parsed_cmd) {
    if (parsed_cmd->i7 >= 0 && (parsed_cmd->flags2 & CMD_FLAG2_I7_SET))
       stabilization_poll_millisec = parsed_cmd->i7;
 }
+#endif
 
 
 STATIC void init_algorithm_options(Parsed_Cmd * parsed_cmd) {
    try_get_edid_from_sysfs_first = parsed_cmd->flags & CMD_FLAG_TRY_GET_EDID_FROM_SYSFS;
 
+#ifdef WATCH_DISPLAYS
    if (parsed_cmd->flags2 & CMD_FLAG2_F17)
        use_sysfs_connector_id = false;
+#endif
 
    force_sysfs_unreliable = parsed_cmd->flags2 & CMD_FLAG2_F21;
    force_sysfs_reliable   = parsed_cmd->flags2 & CMD_FLAG2_F22;
@@ -414,8 +419,10 @@ init_experimental_options(Parsed_Cmd* parsed_cmd) {
 
    if (parsed_cmd->flags2 & CMD_FLAG2_F5)
       EDID_Read_Uses_I2C_Layer = !EDID_Read_Uses_I2C_Layer;
+#ifdef USE_LIBDRM
    if (parsed_cmd->flags2 & CMD_FLAG2_F6)
       use_drm_connector_states = true;
+#endif
    if (parsed_cmd->flags2 & CMD_FLAG2_F7)
       detect_phantom_displays = false;
    if (parsed_cmd->flags2 & CMD_FLAG2_F9)
@@ -440,13 +447,17 @@ init_experimental_options(Parsed_Cmd* parsed_cmd) {
       verify_sysfs_edid = true;
 #endif
 
+#ifdef WATCH_DISPLAYS
    if (parsed_cmd->flags2 & CMD_FLAG2_F19)
       stabilize_added_buses_w_edid = true;
+#endif
    if (parsed_cmd->flags2 & CMD_FLAG2_F23)
       primitive_sysfs = true;
 
+#ifdef WATCH_DISPLAYS
    if (parsed_cmd->flags2 & CMD_FLAG2_F24)
       enable_write_detect_to_status = true;
+#endif
 
    if (parsed_cmd->flags2 & CMD_FLAG2_I2_SET)
         multi_part_null_adjustment_millis = parsed_cmd->i2;
@@ -480,6 +491,12 @@ submaster_initializer(Parsed_Cmd * parsed_cmd) {
 
    redirect_reports_to_syslog = parsed_cmd->flags2 & CMD_FLAG2_F8;
 
+   uid_t ruid, euid, suid;
+   getresuid(&ruid, &euid, &suid);
+   DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "ruid=%d, euid=%d", ruid, euid);
+   if (euid == 0 && ruid != euid)
+      running_as_root = true;
+
    final_result = init_failsim(parsed_cmd);
    if (final_result)
       goto bye;      // main_rc == EXIT_FAILURE
@@ -533,10 +550,10 @@ submaster_initializer(Parsed_Cmd * parsed_cmd) {
    if (is_arm)
       primitive_sysfs = true;
 
+
+   all_video_adapters_implement_drm = false;
    uint64_t t0;
    uint64_t t1;
-   all_video_adapters_implement_drm = false;
-#ifdef USE_LIBDRM
    // For each video adapter node in sysfs, check that subdirectories drm/cardN/cardN-xxx exist
    t0 = cur_realtime_nanosec();
    all_video_adapters_implement_drm = check_all_video_adapters_implement_drm();  // in i2c_sysfs.c
@@ -555,17 +572,18 @@ submaster_initializer(Parsed_Cmd * parsed_cmd) {
 
    if (parsed_cmd->flags2 & CMD_FLAG2_F12)
       all_video_adapters_implement_drm = false;
-#endif
 
    subinit_i2c_bus_core();
 
    // rpt_nl();
    // get_sys_drm_connectors(false);  // initializes global sys_drm_connectors
 
+#ifdef USE_LIBDRM
    if (use_drm_connector_states)
       redetect_drm_connector_states();
    DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "use_drm_connector_states=%s, drm_enabled = %s",
          sbool(use_drm_connector_states), sbool(all_video_adapters_implement_drm));
+#endif
 
 #ifdef NOT_HERE
   // adding or removing MST device can change whether all drm connectors have connector_id
@@ -595,7 +613,9 @@ submaster_initializer(Parsed_Cmd * parsed_cmd) {
 #endif
    rpt_set_default_ornamentation_enabled(true); // applies to all new threads
    rpt_set_ornamentation_enabled(true);         // current thread
+#ifdef WATCH_DISPLAYS
    init_display_watch_options(parsed_cmd);
+#endif
    init_algorithm_options(parsed_cmd);
    init_experimental_options(parsed_cmd);
 
