@@ -15,7 +15,7 @@
 
 void         i2c_discard_caches(Cache_Types caches);
 Error_Info * init_tracing(Parsed_Cmd * parsed_cmd);
-bool         submaster_initializer(Parsed_Cmd * parsed_cmd);
+Error_Info * submaster_initializer(Parsed_Cmd * parsed_cmd);
 void         init_ddc_common_init();
 
 #endif /* DDC_COMMON_INIT_H_ */
