@@ -23,7 +23,6 @@
 
 #include "util/data_structures.h"
 #include "util/debug_util.h"
-#include "util/drm_common.h"
 #include "util/edid.h"
 #include "util/error_info.h"
 #include "util/failsim.h"
@@ -47,7 +46,6 @@
 
 #include "base/core.h"
 #include "base/ddc_packets.h"
-#include "base/drm_connector_state.h"
 #include "base/dsa2.h"
 #include "base/feature_metadata.h"
 #include "base/linux_errno.h"
@@ -78,7 +76,6 @@
 #include "ddc/ddc_serialize.h"
 #include "ddc/ddc_vcp_version.h"
 #include "ddc/ddc_vcp.h"
-// #include "ddc/ddc_dw_main.h"
 
 #include "ddc/ddc_displays.h"
 
@@ -115,7 +112,6 @@ threaded_initial_checks_by_dref(gpointer data) {
    DBGTRC_STARTING(debug, TRACE_GROUP, "dref = %s", dref_repr_t(dref) );
 
    Error_Info * erec = ddc_initial_checks_by_dref(dref, false);
-   // g_thread_exit(NULL);
    ERRINFO_FREE_WITH_REPORT(erec, debug);
    DBGTRC_DONE(debug, TRACE_GROUP, "Returning NULL. dref = %s,", dref_repr_t(dref) );
    free_current_traced_function_stack();
@@ -172,6 +168,7 @@ ddc_non_async_scan(GPtrArray * all_displays) {
       Error_Info * err = ddc_initial_checks_by_dref(dref, false);
       free(err);
    }
+
    DBGTRC_DONE(debug, TRACE_GROUP, "");
 }
 
@@ -217,6 +214,7 @@ ddc_get_filtered_display_refs(bool include_invalid_displays, bool include_remove
          g_ptr_array_add(result, cur);
       }
    }
+
    DBGTRC_DONE(debug, TRACE_GROUP, "Returning array of size %d", result->len);
    if (debug || IS_TRACING()) {
       ddc_dbgrpt_drefs("Display_Refs:", result, 2);
@@ -288,7 +286,6 @@ ddc_get_bus_open_errors() {
 }
 
 
-
 /** Emits a debug report of a list of #Bus_Open_Error.
  *
  *  @param open_errors  array of #Bus_Open_Error
@@ -412,6 +409,7 @@ ddc_detect_all_displays(GPtrArray ** i2c_open_errors_loc) {
             if (businfo->drm_connector_name) {
                dref->drm_connector = g_strdup(businfo->drm_connector_name);
             }
+            dref->drm_connector_found_by = businfo->drm_connector_found_by;
             dref->drm_connector_id = businfo->drm_connector_id;
             dref->pedid = copy_parsed_edid(businfo->edid);
             dref->mmid  = mmk_new(dref->pedid->mfg_id,
@@ -427,7 +425,6 @@ ddc_detect_all_displays(GPtrArray ** i2c_open_errors_loc) {
          if (!asleep & !(dpms_state&DPMS_STATE_X11_CHECKED)) {
              if (dpms_check_drm_asleep_by_dref(dref)) {
                 dpms_state |= DPMS_SOME_DRM_ASLEEP;
-                asleep = true;
              }
              else {
                 all_displays_asleep = false;
@@ -500,7 +497,6 @@ ddc_detect_all_displays(GPtrArray ** i2c_open_errors_loc) {
          if (!asleep & !(dpms_state&DPMS_STATE_X11_CHECKED)) {
              if (dpms_check_drm_asleep_by_dref(dref)) {
                 dpms_state |= DPMS_SOME_DRM_ASLEEP;
-                asleep = true;
              }
              else {
                 all_displays_asleep = false;
@@ -638,6 +634,7 @@ ddc_ensure_displays_detected() {
  *  - The list of errors in #display_open_errors is discarded
  *  - The list of detected I2C buses is discarded
  *  - The USB monitor list is discarded
+ *  - The list of display references recognized by the API is discarded.
  */
 void
 ddc_discard_detected_displays() {
@@ -713,7 +710,7 @@ ddc_is_known_display_ref(Display_Ref * dref) {
  *  indicating why a display ref is invalid.
  *
  *  @param   dref   display reference to validate
- *  #param   validation_options
+ *  @param   validation_options
  *  @retval  DDCRC_OK
  *  @retval  DDCRC_ARG             dref is null or does not point to a Display_Ref
  *  @retval  DDCRC_DISCONNECTED    display has been disconnected
