@@ -330,7 +330,7 @@ check_supported_feature(Display_Handle *      dh,
          {
             if (newly_added) {
                DBGTRC_NOPREFIX(debug, TRACE_GROUP, "Additional 1 second sleep for newly added display (A)");
-               DW_SLEEP_MILLIS(1000, "Additional 1 second sleep for newly added display (C)");
+               SLEEP_MILLIS_WITH_SYSLOG(1000, "Additional 1 second sleep for newly added display (C)");
             }
             // turn off optimization in case it's on
             if (dynamic_sleep_active ) {
@@ -550,7 +550,7 @@ ddc_initial_checks_by_dref(Display_Ref * dref, bool newly_added) {
    Error_Info * err = NULL;
    I2C_Bus_Info * businfo = NULL;
 
-   bool disabled_mmk = is_disabled_mmk(*dref->mmid); // is this monitor model disabled?
+   bool disabled_mmk = is_ignored_mmk(*dref->mmid); // is this monitor model disabled?
    if (disabled_mmk) {
       dref->flags |= DREF_DDC_DISABLED;
       dref->flags |= DREF_DDC_COMMUNICATION_CHECKED;
