@@ -188,6 +188,48 @@ static bool is_drm_conformant_driver(const char * driver_name) {
 // Open/Close Display
 //
 
+__thread GPtrArray * open_displays_for_thread;
+
+
+bool add_open_display_for_current_thread(Display_Handle * dh) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "open_displays_for_thread=%p, dh=%s",
+         open_displays_for_thread, dh_repr_p(dh));
+
+   bool found = false;
+   if (!open_displays_for_thread)
+      open_displays_for_thread = g_ptr_array_new();
+   else
+      found = g_ptr_array_find(open_displays_for_thread, dh, NULL);
+   if (!found) {
+      // DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "open_displays_for_thread=%p, dh=%p", open_displays_for_thread, dh);
+      g_ptr_array_add(open_displays_for_thread, dh);
+   }
+
+   DBGTRC_RET_BOOL(debug, TRACE_GROUP, !found, "dh=%s", dh_repr_p(dh));
+   return !found;
+}
+
+
+bool remove_open_display_for_current_thread(Display_Handle * dh) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "dh=%s", dh_repr_p(dh));
+
+   bool found = false;
+   if (open_displays_for_thread) {
+      found = g_ptr_array_remove(open_displays_for_thread, dh);
+      if (open_displays_for_thread->len == 0) {
+         g_ptr_array_free(open_displays_for_thread, true);
+         open_displays_for_thread = NULL;
+      }
+   }
+
+
+   DBGTRC_RET_BOOL(debug, TRACE_GROUP, found, "dh=%s", dh_repr_p(dh));
+   return found;
+}
+
+
 /** Opens a DDC display.
  *
  *  \param  dref            display reference
@@ -209,8 +251,11 @@ ddc_open_display(
       Display_Handle** dh_loc)
 {
    bool debug = false;
+   assert(dref);
+   // static int ctr = 0;
+   // ctr++;
    DBGTRC_STARTING(debug, TRACE_GROUP, "dref=%s, callopts=%s, dh_loc=%p",
-                      dref_reprx_t(dref), interpret_call_options_t(callopts), dh_loc );
+                      dref_reprx_t(dref), interpret_call_options_t(callopts), dh_loc);
    TRACED_ASSERT(dh_loc);
    // TRACED_ASSERT(1==5);    // for testing
 
@@ -218,6 +263,27 @@ ddc_open_display(
    Error_Info * err = NULL;
    int fd = -1;
 
+   // if (ctr % 8 == 0)
+   //    dref->detail = NULL;
+   if (dref->flags & DREF_REMOVED) {
+      SYSLOG2(DDCA_SYSLOG_ERROR, "Attempting to open disconnected display reference %s",
+            dref_repr_t(dref));
+      err = ERRINFO_NEW(DDCRC_DISCONNECTED, "Attempting to open disconnected display reference %s",
+            dref_repr_t(dref));
+      goto bye;
+   }
+   if (!dref->detail) {
+      SYSLOG2(DDCA_SYSLOG_ERROR, "Display_Ref.detail == NULL, but DREF_REMOVED not set, dref=%s",
+            dref_repr_t(dref));
+      // show_backtrace(1);
+      // backtrace_to_syslog(LOG_ERR, 1);
+      current_traced_function_stack_to_syslog(LOG_ERR, /*reverse*/ false);
+      dref->flags |= DREF_REMOVED;
+      err = ERRINFO_NEW(DDCRC_DISCONNECTED,
+            "Display_Ref.detail == NULL, but DREF_REMOVED not set, dref=%s", dref_repr_t(dref));
+      goto bye;
+   }
+
    const char * driver_name = dref_get_i2c_driver(dref);
    DBGTRC_NOPREFIX(false, DDCA_TRC_NONE, "driver_name: %s", driver_name);
    if (driver_name && is_drm_conformant_driver(driver_name) &&
@@ -233,7 +299,7 @@ ddc_open_display(
          if (tryct == 0) {
             free(status);
             // DBGTRC_NOPREFIX(debug, TRACE_GROUP, "status == disconnected, sleeping 1 sec and retrying");
-            DW_SLEEP_MILLIS(1000, "Delay before rechecking attribute status");
+            SLEEP_MILLIS_WITH_SYSLOG(1000, "Delay before rechecking attribute status");
             tryct++;
             goto retry_status;
          }
@@ -347,6 +413,9 @@ bye:
    if (err) {
       COUNT_STATUS_CODE(err->status_code);
    }
+   else {
+      add_open_display_for_current_thread(dh);
+   }
    *dh_loc = dh;
    TRACED_ASSERT_IFF( !err, *dh_loc );
    // dbgrpt_distinct_display_descriptors(0);
@@ -432,6 +501,7 @@ ddc_close_display(Display_Handle * dh) {
          dh_repr_p(dh), g_hash_table_size(open_displays) );
    g_hash_table_remove(open_displays, dh);
    g_mutex_unlock (&open_displays_mutex);
+   remove_open_display_for_current_thread(dh);
 
    free_display_handle(dh);
    DBGTRC_RET_ERRINFO(debug, TRACE_GROUP, err, "dref=%s", dref_repr_t(dref));
@@ -468,6 +538,27 @@ void ddc_close_all_displays() {
 }
 
 
+void ddc_close_all_displays_for_current_thread(bool error_if_open) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "");
+
+   int closed_ct = 0;
+   if (open_displays_for_thread) {
+      for (int ndx = 0; ndx < open_displays_for_thread->len; ndx++) {
+         Display_Handle * dh = g_ptr_array_index(open_displays_for_thread, ndx);
+         DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Closing %s...", dh_repr_p(dh));
+         if (error_if_open) {
+        	 SYSLOG2(DDCA_SYSLOG_ERROR,"Closing %s that should not be open", dh_repr_p(dh) );
+         }
+         closed_ct++;
+         ddc_close_display_wo_return(dh);
+      }
+   }
+
+   DBGTRC_DONE(debug, TRACE_GROUP, "Closed %d open display handles", closed_ct);
+}
+
+
 // work in progress
 // typedef for ddc_i2c_write_read_raw, ddc_adl_write_read_raw, ddc_write_read_raw
 
@@ -1110,6 +1201,9 @@ init_ddc_packet_io_func_name_table() {
    RTTI_ADD_FUNC(ddc_write_only);
    RTTI_ADD_FUNC(ddc_write_only_with_retry);
    RTTI_ADD_FUNC(ddc_validate_display_handle2);
+   RTTI_ADD_FUNC(add_open_display_for_current_thread);
+   RTTI_ADD_FUNC(remove_open_display_for_current_thread);
+   RTTI_ADD_FUNC(ddc_close_all_displays_for_current_thread);
 }
 
 
