@@ -24,11 +24,11 @@
 #include "util/coredefs.h"
 #include "util/data_structures.h"
 #include "util/debug_util.h"
-#include "util/drm_common.h"
 #include "util/file_util.h"
 #include "util/glib_string_util.h"
 #include "util/glib_util.h"
 #include "util/i2c_util.h"
+#include "util/libdrm_aux_util.h"
 #include "util/linux_util.h"
 #include "util/msg_util.h"
 #include "util/report_util.h"
@@ -59,7 +59,9 @@
 
 #include "dw_status_events.h"
 #include "dw_dref.h"
+#ifdef USE_X11
 #include "dw_xevent.h"
+#endif
 
 #include "dw_common.h"
 
@@ -72,9 +74,11 @@ uint16_t  udev_watch_loop_millisec       = DEFAULT_UDEV_WATCH_LOOP_MILLISEC;
 uint16_t  poll_watch_loop_millisec       = DEFAULT_POLL_WATCH_LOOP_MILLISEC;
 uint16_t  xevent_watch_loop_millisec     = DEFAULT_XEVENT_WATCH_LOOP_MILLISEC;
 
+bool      terminate_watch_thread         = false;
 bool      terminate_using_x11_event      = false;
 
 
+
 uint32_t dw_calc_watch_loop_millisec(DDC_Watch_Mode watch_mode) {
    assert(watch_mode != Watch_Mode_Dynamic);
    int final_answer = 0;
@@ -132,13 +136,14 @@ void dw_free_watch_displays_data(Watch_Displays_Data * wdd) {
    if (wdd) {
       assert( memcmp(wdd->marker, WATCH_DISPLAYS_DATA_MARKER, 4) == 0 );
       wdd->marker[3] = 'x';
-      free(wdd->evdata);
+#ifdef USE_X11
+      dw_deinit_xevent_screen_change_notification(wdd->evdata);
+#endif
       free(wdd);
    }
 }
 
 
-
 void dw_free_recheck_displays_data(Recheck_Displays_Data * rdd) {
    if (rdd) {
       assert( memcmp(rdd->marker, RECHECK_DISPLAYS_DATA_MARKER, 4) == 0 );
@@ -163,8 +168,6 @@ void dw_free_callback_displays_data(Callback_Displays_Data * cdd) {
 }
 
 
-
-
 #ifdef UNUSED
 void ddc_i2c_filter_sleep_events(GArray * events) {
    bool debug = false;
@@ -527,7 +530,7 @@ dw_stabilized_buses_bs(Bit_Set_256 bs_prior, bool some_displays_disconnected) {
          DBGTRC(debug, TRACE_GROUP, "%s", s);
          SYSLOG2(DDCA_SYSLOG_NOTICE, "%s", s);
          free(s);
-         DW_SLEEP_MILLIS(initial_stabilization_millisec,  "Initial stabilization delay");
+         SLEEP_MILLIS_WITH_SYSLOG(initial_stabilization_millisec,  "Initial stabilization delay");
       }
    }
 
@@ -535,7 +538,7 @@ dw_stabilized_buses_bs(Bit_Set_256 bs_prior, bool some_displays_disconnected) {
    bool stable = false;
    while (!stable) {
       // DW_SLEEP_MILLIS(stabilization_poll_millisec, "Loop until stable"); // TMI
-      sleep_millis(stabilization_poll_millisec);
+      SLEEP_MILLIS_WITH_STATS(stabilization_poll_millisec);
       BS256 bs_latest = i2c_buses_w_edid_as_bitset();
       if (bs256_eq(bs_latest, bs_prior))
             stable = true;
@@ -556,6 +559,44 @@ dw_stabilized_buses_bs(Bit_Set_256 bs_prior, bool some_displays_disconnected) {
 }
 
 
+GMutex       active_callback_threads_mutex;
+GHashTable * active_callback_threads;
+
+
+void record_active_callback_thread(GThread* pthread){
+	bool debug = false;
+	DBGTRC_STARTING(debug,TRACE_GROUP, "pthread=p", pthread);
+
+	g_mutex_lock(&active_callback_threads_mutex);
+	if (!active_callback_threads)
+		active_callback_threads = g_hash_table_new(g_direct_hash, g_direct_equal);
+    g_hash_table_add(active_callback_threads, pthread);
+    g_mutex_unlock(&active_callback_threads_mutex);
+
+    DBGTRC_DONE(debug, TRACE_GROUP, "pthread=%p");
+}
+
+
+void remove_active_callback_thread(GThread* pthread){
+	bool debug = false;
+	DBGTRC_STARTING(debug,TRACE_GROUP, "pthread=p", pthread);
+
+	if (active_callback_threads) {
+		g_hash_table_remove(active_callback_threads, pthread);
+	}
+
+    DBGTRC_DONE(debug, TRACE_GROUP, "pthread=%p");
+}
+
+
+int  active_callback_thread_ct() {
+	int ct = 0;
+	if (active_callback_threads)
+		ct = g_hash_table_size(active_callback_threads);
+	return ct;
+}
+
+
 void init_dw_common() {
 #ifdef WATCH_ASLEEP
    RTTI_ADD_FUNC(ddc_i2c_check_bus_asleep);
@@ -563,4 +604,7 @@ void init_dw_common() {
    RTTI_ADD_FUNC(dw_stabilized_buses_bs);
    RTTI_ADD_FUNC(dw_emit_deferred_events);
    RTTI_ADD_FUNC(dw_hotplug_change_handler);
+   RTTI_ADD_FUNC(record_active_callback_thread);
+   RTTI_ADD_FUNC(remove_active_callback_thread);
+   RTTI_ADD_FUNC(active_callback_thread_ct);
 }
