@@ -26,11 +26,11 @@
 #include "util/coredefs.h"
 #include "util/data_structures.h"
 #include "util/debug_util.h"
-#include "util/drm_common.h"
 #include "util/file_util.h"
 #include "util/glib_string_util.h"
 #include "util/glib_util.h"
 #include "util/i2c_util.h"
+#include "util/libdrm_aux_util.h"
 #include "util/linux_util.h"
 #include "util/report_util.h"
 #include "util/string_util.h"
@@ -145,7 +145,7 @@ dw_i2c_stabilized_single_bus_by_connector_name(char * drm_connector_name, bool p
          DBGTRC(debug, TRACE_GROUP, "%s", s);
          SYSLOG2(DDCA_SYSLOG_NOTICE, "%s", s);
          free(s);
-         DW_SLEEP_MILLIS(initial_stabilization_millisec, "Initial sleep");
+         SLEEP_MILLIS_WITH_SYSLOG(initial_stabilization_millisec, "Initial sleep");
       }
    }
 
@@ -153,7 +153,7 @@ dw_i2c_stabilized_single_bus_by_connector_name(char * drm_connector_name, bool p
    bool stable = false;
    while (!stable) {
       // usleep(1000*stabilization_poll_millisec);
-      DW_SLEEP_MILLIS(stabilization_poll_millisec, "Stabilization loop");
+      SLEEP_MILLIS_WITH_SYSLOG(stabilization_poll_millisec, "Stabilization loop");
 
       char * s = g_strdup_printf("/sys/class/drm/%s/edid", drm_connector_name);
       // DBGF(debug, "reading: %s", s);
@@ -250,7 +250,7 @@ Bit_Set_256 dw_i2c_check_bus_changes(
          bs_new_buses_w_edid = bs_stabilized_buses_w_edid;
 #endif
 
-         BS256 bs_new_buses_w_edid = dw_stabilized_buses_bs(bs_new_buses_w_edid, detected_displays_removed_flag);
+         bs_new_buses_w_edid = dw_stabilized_buses_bs(bs_new_buses_w_edid, detected_displays_removed_flag);
       }
    }
 
