@@ -20,15 +20,11 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
-#ifdef USE_X11
-// #include <X11/extensions/dpmsconst.h>
-#endif
 /** \endcond */
 
 #include "util/coredefs_base.h"
 #include "util/debug_util.h"
 #include "util/data_structures.h"
-#include "util/drm_common.h"
 #include "util/edid.h"
 #include "util/error_info.h"
 #include "util/failsim.h"
@@ -39,6 +35,7 @@
 #include "util/report_util.h"
 #include "util/string_util.h"
 #include "util/subprocess_util.h"
+#include "util/sysfs_filter_functions.h"
 #include "util/sysfs_i2c_util.h"
 #include "util/sysfs_util.h"
 #include "util/traced_function_stack.h"
@@ -53,7 +50,6 @@
 #include "base/core.h"
 #include "base/ddc_errno.h"
 #include "base/display_lock.h"
-#include "base/drm_connector_state.h"
 #include "base/flock.h"
 #include "base/i2c_bus_base.h"
 #include "base/linux_errno.h"
@@ -367,7 +363,7 @@ Error_Info * i2c_open_bus(
       if (total_wait_millisec > open_max_wait_millisec)
        DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Total wait %d exceeds max wait %d, tries=%d", total_wait_millisec, open_max_wait_millisec, tryctr);
       else {
-         DW_SLEEP_MILLIS(open_wait_interval_millisec, "");
+         SLEEP_MILLIS_WITH_SYSLOG(open_wait_interval_millisec, "");
          // usleep(wait_interval_millisec * 1000);
       }
    }
@@ -402,11 +398,14 @@ Status_Errno i2c_close_bus_basic(int busno, int fd, Call_Options callopts) {
          f0printf(ferr(), "Close failed for %s, errno=%s\n",
                           filename_for_fd_t(fd), linux_errno_desc(errsv));
       result = -errsv;
+      SYSLOG2(DDCA_SYSLOG_ERROR, "Close failed for %s, errno=%s\n",
+            filename_for_fd_t(fd), linux_errno_desc(errsv));
       // assert(rc == 0);     // don't bother with recovery for now
    }
    return result;
 }
 
+
 /** Closes an open I2C bus device.
  *
  * @param  busno     i2c_bus_number
@@ -435,7 +434,7 @@ Status_Errno i2c_close_bus(int busno, int fd, Call_Options callopts) {
       int rc = flock_unlock_by_fd(fd);
       if (rc < 0) {
          DBGTRC_NOPREFIX(true, TRACE_GROUP,
-               "%/dev/i2c-%d. Unexpected error from flock(..,LOCK_UN): %s",
+               "/dev/i2c-%d. Unexpected error from flock(..,LOCK_UN): %s",
                busno, psc_desc(rc));
       }
    }
@@ -627,30 +626,19 @@ i2c_detect_x37(int fd, char * driver) {
    //   a write (7/2018), so this function checks both
    Status_Errno_DDC rc = 0;
    int max_tries = DETECT_X37_MAX_TRIES;  //2;   // ***TEMP*** 3;
-   bool use_file_io = false;
    int poll_wait_millisec = DETECT_X37_RETRY_MILLISEC;  // 400;
-   char * s = (use_file_io) ? "i2c" : "ioctl";
    int loopctr;
    for (loopctr = 0; loopctr < max_tries; loopctr++) {  // retries seem to give no benefit
-
       // regard either a successful write() or a read() as indication slave address is valid
       Byte    writebuf = {0x00};
-
-      if (use_file_io)
-         rc = invoke_i2c_writer(fd, 0x37, 1, &writebuf);
-      else
-         rc = i2c_ioctl_writer(fd, 0x37, 1, &writebuf);
-      // rc = 6; // for testing
+      rc = invoke_i2c_writer(fd, 0x37, 1, &writebuf);
       DBGTRC_NOPREFIX(debug, TRACE_GROUP,
-                   "invoke_%s_writer() for slave address x37 returned %s", s, psc_name_code(rc));
+                   "invoke_i2c_writer() for slave address x37 returned %s", psc_name_code(rc));
       if (rc != 0) {
          Byte    readbuf[4];  //  4 byte buffer
-         if (use_file_io)
-            rc = invoke_i2c_reader(fd, 0x37, false, 4, readbuf);
-         else
-            rc = i2c_ioctl_reader(fd, 0x37, false, 4, readbuf);
+         rc = invoke_i2c_reader(fd, 0x37, false, 4, readbuf);
          DBGTRC_NOPREFIX(debug, TRACE_GROUP,
-                   "invoke_%s_reader() for slave address x37 returned %s", s, psc_name_code(rc));
+                   "invoke_i2c_reader() for slave address x37 returned %s", psc_name_code(rc));
       }
       if (rc == 0)
          break;
@@ -660,9 +648,10 @@ i2c_detect_x37(int fd, char * driver) {
          wait = 2000;
       DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "driver=%s, sleeping for %d millisec", driver, wait);
             // usleep(poll_wait_millisec*1000);
-      DW_SLEEP_MILLIS(wait, "Extra x37 sleep");
+      SLEEP_MILLIS_WITH_SYSLOG(wait, "Extra x37 sleep");
       // sleep_millis(wait);
    }
+
    DBGTRC_RET_DDCRC(debug, TRACE_GROUP, rc,"loopctr=%d", loopctr);
    return rc;
 }
@@ -705,7 +694,7 @@ Error_Info * i2c_check_open_bus_alive(Display_Handle * dh) {
          //       businfo->busno, tryctr, dh_repr(dh));
          // SYSLOG2(DDCA_SYSLOG_WARNING,
          //       "!!! (B) Retrying i2c_check_edid_exists_by_dh, tryctr=%d, dh=%s", tryctr, dh_repr(dh));
-         DW_SLEEP_MILLIS2(DDCA_SYSLOG_WARNING, CHECK_OPEN_BUS_ALIVE_RETRY_MILLISEC,
+         SLEEP_MILLIS_WITH_SYSLOG2(DDCA_SYSLOG_WARNING, CHECK_OPEN_BUS_ALIVE_RETRY_MILLISEC,
                           "Retrying i2c_check_edid_exists_by_dh() (c)");
       }
 #ifdef SYSFS_PROBLEMATIC   // apparently not by driver vfd on Raspberry pi
@@ -915,31 +904,47 @@ bool is_displaylink_device(int busno) {
 
 
 typedef struct {
-   char * connector_name;
-   int    connector_id;
+   char *                 connector_name;
+   int                    connector_id;
    Drm_Connector_Found_By found_by;
-} Find_Sys_Drm_Connector_Result;
+} Found_Sys_Drm_Connector;
 
 
-void free_find_sys_drm_connector_result_contents(Find_Sys_Drm_Connector_Result rec) {
+void free_found_sys_drm_connector_result_contents(Found_Sys_Drm_Connector rec) {
    free(rec.connector_name);
 }
 
-
-void dbgrpt_find_sys_drm_connector_result(Find_Sys_Drm_Connector_Result val, int depth) {
-   rpt_vstring(depth, "Find_Sys_Drm_Connector_Result:");
+void dbgrpt_found_sys_drm_connector(Found_Sys_Drm_Connector val, int depth) {
+   rpt_vstring(depth, "Found_Sys_Drm_Connector:");
    rpt_vstring(depth+1, "connector_name:   %s", val.connector_name);
    rpt_vstring(depth+1, "connector_id:     %d", val.connector_id);
    rpt_vstring(depth+1, "found_by:         %s", drm_connector_found_by_name(val.found_by));
 }
 
+
+/** Locates a drm-card-connector directory using either an
+ *  I2C bus number or EDID value.
+ *
+ *  @param  busno      (-1 for not set)
+ *  @param  edid_bytes pointer to 128 byte edid
+ *  @return #DRM
+ *
+ *  @remark
+ *  Either one of busno edid_bytes should be set.  Having both parameters
+ *  avoids having 2 separate functions, one for bus number and one for EDID,
+ *  with essentially the same logic.
+ *  @remark
+ *  Given its small size, the result is returned on the stack, not
+ *  the heap, avoiding the need for the caller to free.
+ */
 // n. result returned on stack
-Find_Sys_Drm_Connector_Result find_sys_drm_connector_by_busno_or_edid(
+Found_Sys_Drm_Connector find_sys_drm_connector_by_busno_or_edid(
                                  int busno, Byte * edid_bytes)
 {
    bool debug  = false;
    DBGTRC_STARTING(debug, DDCA_TRC_NONE, " busno = %d, edid = %p" , busno, edid_bytes);
-   DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "edid=%p -> %s", edid_bytes, edid_summary_from_bytes(edid_bytes));
+   DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "edid=%p -> %s",
+         edid_bytes, edid_summary_from_bytes(edid_bytes));
    int d = (IS_DBGTRC(debug, DDCA_TRC_NONE)) ? 1 : -1;
    if (busno == 255)  // happens somehow
       busno = -1;
@@ -947,7 +952,7 @@ Find_Sys_Drm_Connector_Result find_sys_drm_connector_by_busno_or_edid(
    bool check_edid = edid_bytes;
    assert(check_busno || check_edid);
 
-   Find_Sys_Drm_Connector_Result result;
+   Found_Sys_Drm_Connector result;
    result.connector_name = NULL;
    result.found_by = DRM_CONNECTOR_NOT_FOUND;
    result.connector_id = 0;
@@ -965,7 +970,8 @@ Find_Sys_Drm_Connector_Result find_sys_drm_connector_by_busno_or_edid(
             result.connector_name = strdup(cname);
             result.found_by = DRM_CONNECTOR_FOUND_BY_BUSNO;
             result.connector_id = cbn->connector_id;
-            DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Found connector %s by i2c bus number match for bus i2c-%d", cname, busno);
+            DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE,
+                  "Found connector %s by i2c bus number match for bus i2c-%d", cname, busno);
          }
          free_connector_bus_numbers(cbn);
       }
@@ -976,12 +982,14 @@ Find_Sys_Drm_Connector_Result find_sys_drm_connector_by_busno_or_edid(
             possibly_write_detect_to_status_by_connector_name(cname);
             RPT_ATTR_EDID(d, &edid_bytes_array, "/sys/class/drm", cname, "edid");
             if (edid_bytes_array && edid_bytes_array->len >= 128) {
-                DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Got edid from sysfs: %s", edid_summary_from_bytes(edid_bytes_array->data));
+                DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Got edid from sysfs: %s",
+                      edid_summary_from_bytes(edid_bytes_array->data));
                 if (memcmp(edid_bytes_array->data, edid_bytes, 128) == 0) {
                    found = true;
                    result.connector_name = strdup(cname);
                    result.found_by = DRM_CONNECTOR_FOUND_BY_EDID;
-                   DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Found connector %s by EDID match for bus i2c-%d", cname, busno);
+                   DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE,
+                         "Found connector %s by EDID match for bus i2c-%d", cname, busno);
                 }
                 g_byte_array_free(edid_bytes_array, true);
             }
@@ -990,9 +998,8 @@ Find_Sys_Drm_Connector_Result find_sys_drm_connector_by_busno_or_edid(
    }
    free_sysfs_connector_names_contents(cnames);
 
-
    if (IS_DBGTRC(debug, DDCA_TRC_NONE)) {
-      dbgrpt_find_sys_drm_connector_result(result, 1);
+      dbgrpt_found_sys_drm_connector(result, 1);
    }
    DBGTRC_DONE(debug, DDCA_TRC_NONE, "");
    return result;
@@ -1086,39 +1093,42 @@ Byte * get_connector_edid(const char * connector_name) {
 
     bool is_displaylink = is_displaylink_device(busno);
 
+    bool drm_card_connector_directories_exist = directory_exists("/sys/class/drm");
     // *** Try to find the drm connector by bus number
 
-    // n. will fail for MST
-    DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Finding DRM connector name for bus %s using busno", dev_name);
-    Find_Sys_Drm_Connector_Result res = find_sys_drm_connector_by_busno_or_edid(busno, NULL);
-    if (res.connector_name) {
-       drm_connector_name = strdup(res.connector_name);
-       free_find_sys_drm_connector_result_contents(res);
-    }
-    else {
-       DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "DRM connector not found by busno %d", busno);
-    }
+    if (drm_card_connector_directories_exist) {
+       // n. will fail for MST
+       DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Finding DRM connector name for bus %s using busno", dev_name);
+       Found_Sys_Drm_Connector res = find_sys_drm_connector_by_busno_or_edid(busno, NULL);
+       if (res.connector_name) {
+          drm_connector_name = strdup(res.connector_name);
+          free_found_sys_drm_connector_result_contents(res);
+       }
+       else {
+          DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "DRM connector not found by busno %d", busno);
+       }
 
-    // *** Possibly try to get the EDID from sysfs
-    bool checked_connector_for_edid = false;
-    if (drm_connector_name)  {   // i.e. DRM_CONNECTOR_FOUND_BY_BUSNO
-       if ((try_get_edid_from_sysfs_first && is_sysfs_reliable_for_busno(busno) && !primitive_sysfs ) ||
-             is_displaylink)   // X50 can't be read for DisplayLink, must use sysfs
-       {
-          checked_connector_for_edid = true;
-          Byte * edidbytes = get_connector_edid(drm_connector_name);
-          if (edidbytes) {
-             edid_exists = true;
-             free(edidbytes);
-             DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Retrieved edid using DRM connector %s", drm_connector_name);
-          }
-          else {
-             DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Failed to get edid using DRM connector %s", drm_connector_name);
+       // *** Possibly try to get the EDID from sysfs
+       bool checked_connector_for_edid = false;
+       if (drm_connector_name)  {   // i.e. DRM_CONNECTOR_FOUND_BY_BUSNO
+          if ((try_get_edid_from_sysfs_first && is_sysfs_reliable_for_busno(busno) && !primitive_sysfs ) ||
+                is_displaylink)   // X50 can't be read for DisplayLink, must use sysfs
+          {
+             checked_connector_for_edid = true;
+             Byte * edidbytes = get_connector_edid(drm_connector_name);
+             if (edidbytes) {
+                edid_exists = true;
+                free(edidbytes);
+                DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Retrieved edid using DRM connector %s", drm_connector_name);
+             }
+             else {
+                DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Failed to get edid using DRM connector %s", drm_connector_name);
+             }
           }
        }
+       if (checked_connector_for_edid)
+          goto bye;
     }
-    if (checked_connector_for_edid)
-       goto bye;
 
     // *** Open bus
 
@@ -1146,7 +1156,7 @@ Byte * get_connector_edid(const char * connector_name) {
 
  bye:
     free(drm_connector_name);
-    ERRINFO_FREE_WITH_REPORT(master_err, true);
+    ERRINFO_FREE_WITH_REPORT(master_err, IS_DBGTRC(debug, TRACE_GROUP) || is_report_ddc_errors_enabled() );
     DBGTRC_RET_BOOL(debug, TRACE_GROUP, edid_exists, "");
     return edid_exists;
  }
@@ -1190,6 +1200,12 @@ Byte * get_connector_edid(const char * connector_name) {
   }
 
 
+ /** Determines if a card-connector class attribute represents a
+  *  display controller.
+  *
+  *  @param  adapter_class  class value as string
+  *  @return true/false
+  */
  bool is_adapter_class_display_controller(const char * adapter_class) {
     bool debug = false;
     DBGTRC_STARTING(debug, DDCA_TRC_NONE, "class = %s", adapter_class);
@@ -1210,14 +1226,94 @@ Byte * get_connector_edid(const char * connector_name) {
  }
 
 
+#ifdef UNUSED
+ // TODO: MOVE ELSEWHERE
+ /** Scans a single connector directory of /sys/class/drm.
+  *
+  *  Has typedef Dir_Foreach_Func
+  *
+  *  Adds the card-connector (as a string) to a #GPtrArray.
+  *
+  *  @param   dirname
+  *  @param   fn
+  *  @param   accumulator  #GPtrArray to collect the names
+  *  @param   depth        ignored
+  */
+ void add_one_drm_connector_name(
+       const char *  dirname,      // /sys/class/drm
+       const char *  fn,           // e.g. card0-DP-1
+       void *        accumulator,
+       int           depth)
+ {
+    bool debug = false;
+    DBGTRC_STARTING(debug, TRACE_GROUP, "dirname=%s, fn=%s, depth=%d", dirname, fn, depth);
+
+    GPtrArray * connectors = accumulator;
+    g_ptr_array_add(connectors, g_strdup(fn));
+
+    DBGTRC_DONE(debug, TRACE_GROUP, "");
+ }
+
+
+ /** Returns a GPtrArray of all the card-connector directory names,
+  *  e.g. card1-HDMI-A-1
+  *
+  *  @return #GPtrArray of names, caller must free
+  */
+ GPtrArray *  get_drm_connector_names() {
+    bool debug = false;
+    DBGTRC_STARTING(debug, DDCA_TRC_I2C,"");
+
+    GPtrArray * connector_names = g_ptr_array_new_with_free_func(free_sys_drm_connector);
+    dir_filtered_ordered_foreach(
+          "/sys/class/drm",
+          is_drm_connector,      // filter function
+          NULL,                  // ordering function
+          add_one_drm_connector_name,
+          connector_names,    // accumulator, GPtrArray *
+          -1);
+    DBGTRC_DONE(debug, DDCA_TRC_I2C, "size of sys_drm_connectors: %d", sys_drm_connectors->len);
+    return connector_names;
+ }
+
+
+ /** Do card-connector directores exist?
+  *
+  *  @return true/false
+  */
+ bool drm_connectors_exist() {
+    bool debug = false;
+    DBGTRC_STARTING(debug, DDCA_TRC_I2C, "");
+    GPtrArray * connector_names = get_drm_connector_names();
+    bool result = (connector_names->len > 0);
+    g_ptr_array_free(connector_names, true);
+    DBGTRC_RET_BOOL(debug, DDCA_TRC_I2C, result, "");
+    return result;
+ }
+#endif
+
+
+ /** Sets the card-connector related fields in a #I2C_Bus_Info instance,
+  *  by searching for the EDID value in the DRM card-connector directories
+  *
+  *  @param businfo pointer to I2C_Bus_Info instance
+  *
+  *  @remark
+  *  Note that this function presumes that the video driver for the I2C bus
+  *  supports DRM. This is not the case for older drivers, particularly Nvidia.
+  *  @remark
+  *  Writes to the system and (possibly) to the terminal if the instance is
+  *  not found.
+  */
 void set_connector_for_businfo_using_edid(I2C_Bus_Info * businfo) {
    bool debug  = false;
    DBGTRC_STARTING(debug, DDCA_TRC_NONE,
-          "Finding DRM connector name for bus i2c-%d using EDID", businfo->busno);
+          "Finding DRM connector name for bus i2c-%d using EDID, connector_directories_exist=%s",
+          businfo->busno, SBOOL(sysfs_connector_directories_exist()));
    assert(businfo->edid);
 
    businfo->drm_connector_name = NULL;
-   Find_Sys_Drm_Connector_Result conres =    // n.b. struct returned on stack, not pointer
+   Found_Sys_Drm_Connector conres =    // n.b. struct returned on stack, not pointer
        find_sys_drm_connector_by_busno_or_edid(-1, businfo->edid->bytes);
    if (conres.connector_name) {
         businfo->drm_connector_name = conres.connector_name;
@@ -1228,20 +1324,47 @@ void set_connector_for_businfo_using_edid(I2C_Bus_Info * businfo) {
                businfo->busno, businfo->drm_connector_name);
    }
    else {
-       DBGTRC_NOPREFIX(true, DDCA_TRC_NONE,
+      DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE,
              "Failed to find connector name for /dev/i2c-%d using EDID %p",
              businfo->busno, businfo->edid->bytes);
-       start_capture(DDCA_CAPTURE_STDERR);
-       rpt_vstring(0, "Failed to find connector name for /dev/i2c-%d, %s at line %d in file %s. ",
-             businfo->busno,  __func__, __LINE__, __FILE__);
-       i2c_dbgrpt_bus_info(businfo, /*include_sysinfo*/ true, 1);
-       rpt_nl();
-       report_sys_drm_connectors(true, 1);
-       Null_Terminated_String_Array lines = end_capture_as_ntsa();
-       for (int ndx=0; lines[ndx]; ndx++) {
-          LOGABLE_MSG(DDCA_SYSLOG_ERROR, "%s", lines[ndx]);
+      static bool previously_written_to_log = false;
+
+       if (!previously_written_to_log) {
+#ifdef NO_LONGER_NEEDED
+          start_capture(DDCA_CAPTURE_STDERR);
+          rpt_vstring(0, "Failed to find connector name for /dev/i2c-%d, %s at line %d in file %s. ",
+                businfo->busno,  __func__, __LINE__, __FILE__);
+          i2c_dbgrpt_bus_info(businfo, /*include_sysinfo*/ true, 1);
+          rpt_nl();
+          report_sys_drm_connectors(true, 1);
+          Null_Terminated_String_Array lines = end_capture_as_ntsa();
+          for (int ndx=0; lines[ndx]; ndx++) {
+#ifdef OLD    // made unnecessary by   assert(directory_exists("/sys/class/drm"));
+             if (directory_exists("/sys/class/drm"))
+                LOGABLE_MSG(DDCA_SYSLOG_ERROR, "%s", lines[ndx]);
+             else {
+                SYSLOG2(DDCA_SYSLOG_INFO, "%s", lines[ndx]);
+                SYSLOG2(DDCA_SYSLOG_INFO, "Directory /sys/class/drm does not exist");
+             }
+#endif
+             SYSLOG2(DDCA_SYSLOG_ERROR, "%s", lines[ndx]);
+          }
+          ntsa_free(lines, true);
+#endif
+          previously_written_to_log = true;
+       }
+       else {
+          if (sysfs_connector_directories_exist())
+             LOGABLE_MSG(DDCA_SYSLOG_ERROR,
+                   "Failed to find connector name for /dev/i2c-%d, %s at line %d in file %s. ",
+                   businfo->busno,  __func__, __LINE__, __FILE__);
+          else {
+             SYSLOG2(DDCA_SYSLOG_INFO,
+                   "Failed to find connector name for /dev/i2c-%d, %s at line %d in file %s. ",
+                   businfo->busno,  __func__, __LINE__, __FILE__);
+             SYSLOG2(DDCA_SYSLOG_INFO, "drm connector directories do not exist");
+          }
        }
-       ntsa_free(lines, true);
    }
    DBGTRC_DONE(debug, DDCA_TRC_NONE,"");
 }
@@ -1344,7 +1467,7 @@ bool check_x37_for_businfo(int fd, I2C_Bus_Info * businfo) {
  *  @param  businfo  pointer to #I2C_Bus_Info struct in which information will be set
  *  @return status code
  */
-Status_Errno  i2c_check_bus(I2C_Bus_Info * businfo) {
+Error_Info * i2c_check_bus(I2C_Bus_Info * businfo) {
    bool debug = false;
    DBGTRC_STARTING(debug, TRACE_GROUP, "busno=%d, businfo=%p, primitive_sysfs=%s",
          businfo->busno, businfo, SBOOL(primitive_sysfs) );
@@ -1358,6 +1481,7 @@ Status_Errno  i2c_check_bus(I2C_Bus_Info * businfo) {
    assert(businfo->busno >= 0);
    assert(businfo->busno != 255);
    bool try_get_edid_from_sysfs_first = true;
+
    // int busno = businfo->busno;
    char sysfs_name[30];
    char dev_name[15];
@@ -1367,8 +1491,8 @@ Status_Errno  i2c_check_bus(I2C_Bus_Info * businfo) {
    g_snprintf(dev_name,   15, "/dev/%s", i2cN);
    DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "sysfs_name = |%s|, dev_name = |%s|", sysfs_name, dev_name);
    // int d = (IS_DBGTRC(debug, DDCA_TRC_NONE)) ? 1 : -1;
+   bool drm_card_connector_directories_exist = sysfs_connector_directories_exist();
 
-   int ddcrc = 0;
    businfo->flags |= I2C_BUS_PROBED;
    Error_Info *master_err = NULL;
    if (!i2c_device_exists(businfo->busno)) {
@@ -1415,25 +1539,28 @@ Status_Errno  i2c_check_bus(I2C_Bus_Info * businfo) {
 
    // *** Try to find the drm connector by bus number
 
-   if (!businfo->drm_connector_name) {  // i.e. this is a recheck
+   if (!businfo->drm_connector_name) {  // i.e. this is not a recheck
       //assert(businfo->drm_connector_found_by == DRM_CONNECTOR_NOT_CHECKED ||
       //       businfo->drm_connector_found_by == DRM_CONNECTOR_NOT_FOUND);
       businfo->drm_connector_found_by = DRM_CONNECTOR_NOT_CHECKED;
-      // n. will fail for MST
-      DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Finding DRM connector name for bus %s using busno", dev_name);
-      Find_Sys_Drm_Connector_Result res = find_sys_drm_connector_by_busno_or_edid(businfo->busno, NULL);
-      if (res.connector_name) {
-         businfo->drm_connector_name = strdup(res.connector_name);  // *** LEAKS ***
-         businfo->drm_connector_found_by = DRM_CONNECTOR_FOUND_BY_BUSNO;
-         businfo->drm_connector_id = res.connector_id;
-         DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Found DRM connector name %s by busno, found_by=%s",
-               businfo->drm_connector_name, drm_connector_found_by_name(businfo->drm_connector_found_by));
-         free_find_sys_drm_connector_result_contents(res);
-      }
-      else {
-         DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "DRM connector not found by busno %d", businfo->busno);
+      if (drm_card_connector_directories_exist) {
+         // n. will fail for MST
+         DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Finding DRM connector name for bus %s using busno", dev_name);
+         Found_Sys_Drm_Connector res = find_sys_drm_connector_by_busno_or_edid(businfo->busno, NULL);
+         if (res.connector_name) {
+            businfo->drm_connector_name = strdup(res.connector_name);  // *** LEAKS ***
+            businfo->drm_connector_found_by = DRM_CONNECTOR_FOUND_BY_BUSNO;
+            businfo->drm_connector_id = res.connector_id;
+            DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Found DRM connector name %s by busno, found_by=%s",
+                  businfo->drm_connector_name, drm_connector_found_by_name(businfo->drm_connector_found_by));
+            free_found_sys_drm_connector_result_contents(res);
+         }
+         else {
+            DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "DRM connector not found by busno %d", businfo->busno);
+         }
       }
    }
+
    // *** Possibly try to get the EDID from sysfs
    bool checked_connector_for_edid = false;
    if (businfo->drm_connector_name)  {   // i.e. DRM_CONNECTOR_FOUND_BY_BUSNO
@@ -1491,7 +1618,7 @@ Status_Errno  i2c_check_bus(I2C_Bus_Info * businfo) {
 
    // If there's an EDID on the bus and we don't yet have the connector name
    // based on a busno match, try EDID match
-   if (!businfo->drm_connector_name && businfo->edid) {
+   if (!businfo->drm_connector_name && businfo->edid && drm_card_connector_directories_exist) {
       set_connector_for_businfo_using_edid(businfo);
    }
 
@@ -1516,7 +1643,7 @@ Status_Errno  i2c_check_bus(I2C_Bus_Info * businfo) {
    else  if (businfo->edid) {  // start, x37 check
 
       Monitor_Model_Key mmk = mmk_value_from_edid(businfo->edid);
-      bool disabled_mmk = is_disabled_mmk(mmk);
+      bool disabled_mmk = is_ignored_mmk(mmk);
       if (disabled_mmk) {
          businfo->flags |= I2C_BUS_DDC_DISABLED;
       }
@@ -1541,34 +1668,17 @@ Status_Errno  i2c_check_bus(I2C_Bus_Info * businfo) {
    DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Closing bus...");
    i2c_close_bus(businfo->busno, fd, CALLOPT_ERR_MSG);
 
-    // doesn't really belong here
-    businfo->last_checked_dpms_asleep = dpms_check_drm_asleep_by_businfo(businfo);
-
     businfo->flags |= I2C_BUS_INITIAL_CHECK_DONE;
 
 bye:
-   if ( IS_DBGTRC(debug, TRACE_GROUP)) {
-      DBGTRC_NOPREFIX(true, TRACE_GROUP, "busno=%d, flags = %s",
-            businfo->busno, i2c_interpret_bus_flags_t(businfo->flags));
-
-      // DBGTRC_NOPREFIX(debug, TRACE_GROUP, "businfo:");
-      // i2c_dbgrpt_bus_info(businfo, 2);
-      if (master_err) {
-         DBGTRC_NOPREFIX(debug, TRACE_GROUP, "businfo:");
-         i2c_dbgrpt_bus_info(businfo, /* include_sysinfo */ true, 2);
-         ddcrc = master_err->status_code;
-         ERRINFO_FREE_WITH_REPORT(master_err, true);
-      }
-   }
-   else {
-      if (master_err) {
-         ddcrc = master_err->status_code;
-         ERRINFO_FREE_WITH_REPORT(master_err, false);
-      }
+   if ( IS_DBGTRC(debug, DDCA_TRC_NONE)) {
+      // DBGTRC_NOPREFIX(true, DDCA_TRC_NONE, "busno=%d, flags = %s",
+      //       businfo->busno, i2c_interpret_bus_flags_t(businfo->flags));
+      i2c_dbgrpt_bus_info(businfo, /* include_sysinfo */ true, 2);
    }
 
-   DBGTRC_RET_DDCRC(debug, TRACE_GROUP, ddcrc, "");
-   return ddcrc;
+   DBGTRC_RET_ERRINFO(debug, TRACE_GROUP, master_err, "");
+   return master_err;
 }  // i2c_check_bus
 
 
@@ -1730,12 +1840,12 @@ i2c_threaded_initial_checks_by_businfo(gpointer data) {
    TRACED_ASSERT(memcmp(businfo->marker, I2C_BUS_INFO_MARKER, 4) == 0 );
    DBGTRC_STARTING(debug, TRACE_GROUP, "bus = /dev/i2c-%d", businfo->busno );
 
-   i2c_check_bus(businfo);
+   Error_Info * err = i2c_check_bus(businfo);
    // g_thread_exit(NULL);
 
-   DBGTRC_DONE(debug, TRACE_GROUP, "Returning NULL. bus=/dev/i2c-%d", businfo->busno );
+   DBGTRC_RET_ERRINFO(debug, TRACE_GROUP, err, "bus=/dev/i2c-%d", businfo->busno );
    free_current_traced_function_stack();
-   return NULL;
+   return err;
 }
 
 
@@ -1766,7 +1876,8 @@ i2c_async_scan(GPtrArray * i2c_buses) {
    DBGMSF(debug, "Started %d threads", threads->len);
    for (int ndx = 0; ndx < threads->len; ndx++) {
       GThread * thread = g_ptr_array_index(threads, ndx);
-      g_thread_join(thread);  // implicitly unrefs the GThread
+      Error_Info * err = g_thread_join(thread);  // implicitly unrefs the GThread
+      ERRINFO_FREE_WITH_REPORT(err, IS_DBGTRC(debug, TRACE_GROUP) || is_report_ddc_errors_enabled() );
    }
    DBGMSF(debug, "Threads joined");
    g_ptr_array_free(threads, true);
@@ -1783,12 +1894,14 @@ void
 i2c_non_async_scan(GPtrArray * i2c_buses) {
    bool debug = false;
    DBGTRC_STARTING(debug, TRACE_GROUP, "checking %d buses", i2c_buses->len);
+   Error_Info * err = NULL;
 
    for (int ndx = 0; ndx < i2c_buses->len; ndx++) {
       I2C_Bus_Info * businfo = g_ptr_array_index(i2c_buses, ndx);
       DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE,
             "Calling i2c_check_bus() synchronously for bus %d", businfo->busno);
-      i2c_check_bus(businfo);
+      err = i2c_check_bus(businfo);
+      ERRINFO_FREE_WITH_REPORT(err, IS_DBGTRC(debug, TRACE_GROUP) || is_report_ddc_errors_enabled() );
    }
 
    DBGTRC_DONE(debug, TRACE_GROUP, "");
@@ -1799,10 +1912,6 @@ i2c_non_async_scan(GPtrArray * i2c_buses) {
 // Attached buses
 //
 
-
-// moved from udev_i2c_util.c
-
-
 /** Gets the numbers of I2C devices
  *
  *  \param  include_ignorable_devices  if true, do not exclude SMBus and other ignorable devices
@@ -1831,15 +1940,11 @@ get_i2c_device_numbers_using_udev(bool include_ignorable_devices) {
    char * s = bva_as_string(bva, /*as_hex*/ false, ",");
    DBGTRC_DONE(debug, TRACE_GROUP, "Returning I2C bus numbers: %s", s);
    free(s);
-   // bva_report(bva, "Returning I2c bus numbers:");
 
    return bva;
 }
 
 
-Bit_Set_256 attached_buses;
-
-
 /** Returns the bus numbers for /dev/i2c buses that could possibly be
  *  connected to a monitor.:
  *
@@ -1983,7 +2088,8 @@ I2C_Bus_Info * i2c_get_and_check_bus_info(int busno) {
    I2C_Bus_Info* businfo =  i2c_get_bus_info(busno, &new_info);
    if (!new_info)
       i2c_reset_bus_info(businfo);
-   i2c_check_bus(businfo);
+   Error_Info * err = i2c_check_bus(businfo);
+   ERRINFO_FREE_WITH_REPORT(err, IS_DBGTRC(debug, DDCA_TRC_NONE) || is_report_ddc_errors_enabled());
 #ifdef OLD
    if (new_info | !(businfo->flags&I2C_BUS_INITIAL_CHECK_DONE)) {
       i2c_check_bus(businfo);
@@ -1998,7 +2104,6 @@ I2C_Bus_Info * i2c_get_and_check_bus_info(int busno) {
 }
 
 
-
 /** Detect buses if not already detected.
  *
  *  Stores the result in global array all_i2c_buses and also
@@ -2021,7 +2126,6 @@ int i2c_detect_buses() {
 }
 
 
-
 // used only by main.c, not shared library, does not need mutex protection
 I2C_Bus_Info * i2c_detect_single_bus(int busno) {
    bool debug = false;
@@ -2035,7 +2139,8 @@ I2C_Bus_Info * i2c_detect_single_bus(int busno) {
       }
       businfo = i2c_new_bus_info(busno);
       businfo->flags = I2C_BUS_EXISTS;
-      i2c_check_bus(businfo);
+      Error_Info * err = i2c_check_bus(businfo);
+      ERRINFO_FREE_WITH_REPORT(err, IS_DBGTRC(debug, DDCA_TRC_I2C) || is_report_ddc_errors_enabled());
       if (debug)
          i2c_dbgrpt_bus_info(businfo, true, 0);
       g_ptr_array_add(all_i2c_buses, businfo);
@@ -2075,8 +2180,6 @@ Bit_Set_256 buses_bitset_from_businfo_array(GPtrArray * businfo_array, bool only
 }
 
 
-
-
 //
 // I2C Bus Inquiry
 //
@@ -2155,6 +2258,9 @@ void i2c_report_active_bus(I2C_Bus_Info * businfo, int depth) {
    DBGTRC_STARTING(debug, TRACE_GROUP, "businfo=%p", businfo);
    assert(businfo);
 
+#define DO_OUTPUT(_indent, _title_width, _title, _value) \
+      rpt_vstring(_indent, "%-*s%s", _title_width, _title, _value);
+
    int d1 = depth+1;
    DDCA_Output_Level output_level = get_output_level();
    if (output_level >= DDCA_OL_NORMAL)
@@ -2165,41 +2271,42 @@ void i2c_report_active_bus(I2C_Bus_Info * businfo, int depth) {
    // if (!(businfo->flags & I2C_BUS_DRM_CONNECTOR_CHECKED))
    //    i2c_check_businfo_connector(businfo);
 
-   int title_width = (output_level >= DDCA_OL_VERBOSE) ? 39 : 25;
+   int title_width = (output_level >= DDCA_OL_VERBOSE) ? 43 : 25;
+   int d = (output_level >= DDCA_OL_VERBOSE) ? d1 : depth;
    if (businfo->drm_connector_name && output_level >= DDCA_OL_NORMAL) {
-      int d = (output_level >= DDCA_OL_VERBOSE) ? d1 : depth;
-      rpt_vstring( d, "%-*s%s", title_width, "DRM connector:",
-                   (businfo->drm_connector_name) ? businfo->drm_connector_name : "Not found" );
-      if (output_level >= DDCA_OL_VERBOSE) {
-         if (businfo->drm_connector_name) {
-            char title_buf[100];
-            int tw = title_width; // 35;  // title_width;
-            char * attr_value = NULL;
+      DO_OUTPUT(d, title_width, "DRM_connector:",
+            (businfo->drm_connector_name) ? businfo->drm_connector_name : "Not found" );
+   }
 
-            char * attr = "dpms";
-            attr_value = i2c_get_drm_connector_attribute(businfo, attr);
-            g_snprintf(title_buf, 100, "/sys/class/drm/%s/%s", businfo->drm_connector_name, attr);
-            rpt_vstring(d, "%-*s%s", tw, title_buf, attr_value);
-            free(attr_value);
+   if (output_level >= DDCA_OL_VERBOSE) {
+      if (businfo->drm_connector_name) {
+         char title_buf[100];
+         int tw = title_width; // 35;  // title_width;
+         char * attr_value = NULL;
 
-            attr = "enabled";
-            attr_value = i2c_get_drm_connector_attribute(businfo, attr);
-            g_snprintf(title_buf, 100, "/sys/class/drm/%s/%s", businfo->drm_connector_name, attr);
-            rpt_vstring(d, "%-*s%s", tw, title_buf, attr_value);
-            free(attr_value);
+         char * attr = "dpms";
+         attr_value = i2c_get_drm_connector_attribute(businfo, attr);
+         g_snprintf(title_buf, 100, "/sys/class/drm/%s/%s", businfo->drm_connector_name, attr);
+         DO_OUTPUT(d, tw, title_buf, attr_value);
+         free(attr_value);
 
-            attr = "status";
-            attr_value = i2c_get_drm_connector_attribute(businfo, attr);
-            g_snprintf(title_buf, 100, "/sys/class/drm/%s/%s", businfo->drm_connector_name, attr);
-            rpt_vstring(d, "%-*s%s", tw, title_buf, attr_value);
-            free(attr_value);
+         attr = "enabled";
+         attr_value = i2c_get_drm_connector_attribute(businfo, attr);
+         g_snprintf(title_buf, 100, "/sys/class/drm/%s/%s", businfo->drm_connector_name, attr);
+         DO_OUTPUT(d, tw, title_buf, attr_value);
+         free(attr_value);
 
-            attr = "connector_id";
-            attr_value = i2c_get_drm_connector_attribute(businfo, attr);
-            g_snprintf(title_buf, 100, "/sys/class/drm/%s/%s", businfo->drm_connector_name, attr);
-            rpt_vstring(d, "%-*s%s", tw, title_buf, attr_value);
-            free(attr_value);
-         }
+         attr = "status";
+         attr_value = i2c_get_drm_connector_attribute(businfo, attr);
+         g_snprintf(title_buf, 100, "/sys/class/drm/%s/%s", businfo->drm_connector_name, attr);
+         DO_OUTPUT(d, tw, title_buf, attr_value);
+         free(attr_value);
+
+         attr = "connector_id";
+         attr_value = i2c_get_drm_connector_attribute(businfo, attr);
+         g_snprintf(title_buf, 100, "/sys/class/drm/%s/%s", businfo->drm_connector_name, attr);
+         DO_OUTPUT(d, tw, title_buf, attr_value);
+         free(attr_value);
       }
    }
 
@@ -2211,19 +2318,19 @@ void i2c_report_active_bus(I2C_Bus_Info * businfo, int depth) {
    // rpt_vstring(depth, "Supports DDC:    %s", sbool(businfo->flags & I2C_BUS_ADDR_0X37));
 
    if (output_level >= DDCA_OL_VERBOSE) {
-      rpt_vstring(d1, "Driver:                                %s", (businfo->driver) ? businfo->driver : "Unknown");
-#ifdef DETECT_SLAVE_ADDRS
-      rpt_vstring(d1, "I2C address 0x30 (EDID block#)  present: %-5s", srepr(businfo->flags & I2C_BUS_ADDR_0X30));
-#endif
-      rpt_vstring(d1, "EDID exists:                           %-5s", sbool(businfo->flags & I2C_BUS_HAS_EDID));
-      rpt_vstring(d1, "I2C address 0x37 (DDC)  responsive:    %-5s", sbool(businfo->flags & I2C_BUS_ADDR_X37));
+      DO_OUTPUT(d1, title_width, "Driver:", (businfo->driver) ? businfo->driver : "Unknown");
+// #ifdef DETECT_SLAVE_ADDRS
+      DO_OUTPUT(d1, title_width, "I2C address 0x30 (EDID block#)  present:", sbool(businfo->flags & I2C_BUS_ADDR_X30));
+// #endif
+      DO_OUTPUT(d1, title_width, "EDID exists:",  sbool(businfo->flags & I2C_BUS_HAS_EDID));
+      DO_OUTPUT(d1, title_width, "I2C address 0x37 (DDC) responsive:", sbool(businfo->flags & I2C_BUS_ADDR_X37));
 #ifdef OLD
       rpt_vstring(d1, "Is eDP device:                         %-5s", sbool(businfo->flags & I2C_BUS_EDP));
       rpt_vstring(d1, "Is LVDS device:                        %-5s", sbool(businfo->flags & I2C_BUS_LVDS));
 #endif
-      rpt_vstring(d1, "Is LVDS or EDP display:                %-5s", sbool(businfo->flags & I2C_BUS_LVDS_OR_EDP));
-      rpt_vstring(d1, "Is laptop display by EDID:             %-5s", sbool(businfo->flags & I2C_BUS_APPARENT_LAPTOP));
-      rpt_vstring(d1, "Is laptop display:                     %-5s", sbool(businfo->flags & I2C_BUS_LAPTOP));
+      DO_OUTPUT(d1, title_width, "Is LVDS or EDP display:", sbool(businfo->flags & I2C_BUS_LVDS_OR_EDP));
+      DO_OUTPUT(d1, title_width, "Is laptop display by EDID:", sbool(businfo->flags & I2C_BUS_APPARENT_LAPTOP));
+      DO_OUTPUT(d1, title_width, "Is laptop display:",          sbool(businfo->flags & I2C_BUS_LAPTOP));
 
       // if ( !(businfo->flags & (I2C_BUS_EDP|I2C_BUS_LVDS)) )
       // rpt_vstring(d1, "I2C address 0x37 (DDC) responsive:  %-5s", sbool(businfo->flags & I2C_BUS_ADDR_0X37));
@@ -2231,12 +2338,14 @@ void i2c_report_active_bus(I2C_Bus_Info * businfo, int depth) {
       char fn[PATH_MAX];     // yes, PATH_MAX is dangerous, but not as used here
       sprintf(fn, "/sys/bus/i2c/devices/i2c-%d/name", businfo->busno);
       char * sysattr_name = file_get_first_line(fn, /* verbose*/ false);
-      rpt_vstring(d1, "%-*s%s", title_width, fn, sysattr_name);
+      // rpt_vstring(d1, "%-*s%s", title_width, fn, sysattr_name);
+      DO_OUTPUT(d1, title_width, fn, sysattr_name);
       free(sysattr_name);
       sprintf(fn, "/sys/bus/i2c/devices/i2c-%d", businfo->busno);
       char * path = NULL;
       GET_ATTR_REALPATH(&path, fn);
-      rpt_vstring(d1, "PCI device path:                       %s", path);
+      // rpt_vstring(d1, "PCI device path:                       %s", path);
+      DO_OUTPUT(d1, title_width, "PCI device path:", path);
       free(path);
 
 #ifdef REDUNDANT
@@ -2250,6 +2359,8 @@ void i2c_report_active_bus(I2C_Bus_Info * businfo, int depth) {
 #endif
    }
 
+#undef DO_OUTPUT
+
    if (businfo->edid) {
       if (output_level == DDCA_OL_TERSE) {
          rpt_vstring(depth, "I2C bus:          /dev/"I2C"-%d", businfo->busno);
@@ -2308,7 +2419,5 @@ void subinit_i2c_bus_core() {
 void init_i2c_bus_core() {
    init_i2c_bus_core_func_name_table();
    open_failures_reported = EMPTY_BIT_SET_256;
-   attached_buses = EMPTY_BIT_SET_256;
-   // connected_buses = EMPTY_BIT_SET_256;
 }
 
