@@ -56,7 +56,7 @@ Status_Errno     i2c_close_bus(int busno, int fd, Call_Options callopts);
 // Bus inspection
 I2C_Bus_Info *   i2c_get_and_check_bus_info(int busno);
 bool             i2c_edid_exists(int busno);
-Status_Errno     i2c_check_bus(I2C_Bus_Info * businfo);
+Error_Info *     i2c_check_bus(I2C_Bus_Info * businfo);
 Error_Info *     i2c_check_open_bus_alive(Display_Handle * dh);
 
 // Bus inventory - detect and probe buses
