@@ -11,10 +11,12 @@
 
 #include <assert.h>
 #include <errno.h>
+#include <glib-2.0/glib.h>
 #include <stdio.h>
 #include <syslog.h>
 /** \endcond */
 
+#include "util/drm_card_connector_util.h"
 #include "util/file_util.h"
 #include "util/i2c_util.h"
 #include "util/string_util.h"
@@ -188,7 +190,7 @@ Status_Errno_DDC invoke_i2c_writer(
                  hexstring_t(bytes_to_write, bytect));
 
    // n. prior to gcc 11, declaration cannot immediately follow label
-   I2C_IO_Strategy * strategy = I2C_IO_STRATEGY_NOT_SET;
+   I2C_IO_Strategy * strategy = NULL;
 retry:
    strategy = i2c_get_io_strategy();
    DBGTRC_NOPREFIX(debug, TRACE_GROUP, "strategy = %s", strategy->strategy_name);
@@ -235,7 +237,7 @@ Status_Errno_DDC invoke_i2c_reader(
                    readbuf);
 
      // n. prior to gcc 11, declaration cannot immediately follow label
-     I2C_IO_Strategy * strategy = I2C_IO_STRATEGY_NOT_SET;
+     I2C_IO_Strategy * strategy = NULL;
 retry:
      strategy = i2c_get_io_strategy();
      DBGTRC_NOPREFIX(debug, TRACE_GROUP, "strategy = %s", strategy->strategy_name);
