@@ -60,8 +60,10 @@
 #include "ddc/ddc_try_data.h"
 #include "ddc/ddc_vcp.h"
 
+#ifdef WATCH_DISPLAYS
 #include "dw/dw_main.h"
 #include "dw/dw_services.h"
+#endif
 
 #include "libmain/api_error_info_internal.h"
 #include "libmain/api_base_internal.h"
@@ -188,7 +190,7 @@ ddca_build_options(void) {
 
 
 const char *
-ddca_libddcutil_filename(void) {
+ddci_libddcutil_filename(void) {
    Dl_info info = {NULL,NULL,NULL,NULL};
    static char fullname[PATH_MAX];
    static char * p = NULL;
@@ -201,9 +203,15 @@ ddca_libddcutil_filename(void) {
 }
 
 
+const char *
+ddca_libddcutil_filename(void) {
+	return ddci_libddcutil_filename();
+}
+
+
 bool increment_active_api_calls(const char * funcname) {
    bool debug = false;
-   DBGMSF(debug, "Starting. funcname=%s, active_calls=%d", funcname, active_calls);
+   DBGTRC_STARTING(debug, DDCA_TRC_NONE, "funcname=%s, active_calls=%d", funcname, active_calls);
 
    bool result = true;
    g_mutex_lock(&api_quiesced_mutex);  // blocks API calls from starting
@@ -218,14 +226,14 @@ bool increment_active_api_calls(const char * funcname) {
    g_mutex_unlock(&active_calls_mutex);
    g_mutex_unlock(&api_quiesced_mutex);
 
-   DBGMSF(debug, "funcname=%s, returning %s", funcname, SBOOL(result));
+   DBGTRC_DONE(debug, DDCA_TRC_NONE, "funcname=%s, returning %s", funcname, SBOOL(result));
    return result;
 }
 
 
 void decrement_active_api_calls(const char * funcname) {
    bool debug = false;
-   DBGMSF(debug, "Starting. funcname=%s, active_calls=%d", funcname, active_calls);
+   DBGTRC_STARTING(debug, DDCA_TRC_NONE, "funcname=%s, active_calls=%d", funcname, active_calls);
 
    bool oops = false;
    g_mutex_lock(&active_calls_mutex);
@@ -240,13 +248,17 @@ void decrement_active_api_calls(const char * funcname) {
       MSG_W_SYSLOG(DDCA_SYSLOG_ERROR, "Unmatched active call ct in %s", funcname);
    }
 
-   DBGMSF(debug, "Done    funcname=%s, oops=%s", funcname, SBOOL(oops));
+   DBGTRC_DONE(debug, DDCA_TRC_NONE, "funcname=%s, oops=%s", funcname, SBOOL(oops));
 }
 
 
 /** Quiesce the API.
  *
  *  When quiesced, API calls that can affect monitor state terminate immediately with status DDCRC_QUIESCED.
+ *
+ *  This function waits at most 3000 miliseconds for outstanding API calls to complete.
+ *  If calls are still outstanding, an error messages is written to the system log,
+ *  but this does not prevent queiescing,
  */
 void quiesce_api() {
    bool debug = false;
@@ -517,7 +529,9 @@ _libddcutil_constructor(void) {
    init_api_base();         // registers functions in RTTI table
    init_base_services();    // initializes tracing related modules
    init_ddc_services();     // initializes i2c, usb, ddc, vcp, dynvcp
+#ifdef WATCH_DISPLAYS
    init_dw_services();      // initializes dw
+#endif
    init_api_services();     // other files in directory libmain
 
 #ifdef TESTING_CLEANUP
@@ -631,12 +645,14 @@ _ddca_terminate(void) {
       ddc_discard_detected_displays();
       if (requested_stats)
          ddc_report_stats_main(requested_stats, per_display_stats, dsa_detail_stats, false, 0);
+#ifdef WATCH_DISPLAYS
       DDCA_Display_Event_Class active_classes;
       if (dw_is_watch_displays_executing())
          dw_stop_watch_displays(/*wait=*/ true, &active_classes);   // in case it was started
       DBGTRC_NOPREFIX(debug, DDCA_TRC_API, "After ddc_stop_watch_displays");
       // sleep(5); // still needed?
       terminate_dw_services();
+#endif
       terminate_ddc_services();
       terminate_base_services();
       free_regex_hash_table();
@@ -785,7 +801,7 @@ ddci_init(const char *      libopts,
       // special handling for start and termination msgs
       // always output if syslog is opened
       syslog(LOG_NOTICE, "Initializing libddcutil.  ddcutil version: %s, shared library: %s",
-                   get_full_ddcutil_version(), ddca_libddcutil_filename());
+                   get_full_ddcutil_version(), ddci_libddcutil_filename());
 
       syslog_level = syslog_level_arg;  // global in trace_control.h
 
@@ -910,6 +926,7 @@ ddca_init(const char *      libopts,
    return ddci_init(libopts, syslog_level_arg, opts, NULL);
 }
 
+
 DDCA_Status
 ddca_init2(const char *     libopts,
           DDCA_Syslog_Level syslog_level_arg,
@@ -926,11 +943,22 @@ ddca_start_watch_displays(DDCA_Display_Event_Class enabled_classes) {
    bool debug = false;
    API_PROLOGX(debug, RESPECT_QUIESCE, "enabled_classes=0x%02x", enabled_classes);
 
+#ifdef WATCH_DISPLAYS
    DBGTRC_NOPREFIX(debug, DDCA_TRC_API, "all_video_adapters_implement_drm=%s",
          sbool(all_video_adapters_implement_drm));
 
+
+#ifdef OUT
    if (enabled_classes == DDCA_EVENT_CLASS_ALL)
       enabled_classes = DDCA_EVENT_CLASS_DISPLAY_CONNECTION;
+   else
+#endif
+	   if (enabled_classes&DDCA_EVENT_CLASS_DPMS) {
+        // edet = new_ddca_error_detail(DDCRC_UNIMPLEMENTED, "Watching for DPMS state changes unimplemented");
+  	   // DBGMSG("Watching for DPMS state changes unimplemented");
+  	   MSG_W_SYSLOG(DDCA_SYSLOG_WARNING, "Watching for DPMS state changes unimplemented");
+        enabled_classes &=~DDCA_EVENT_CLASS_DPMS;
+     }
 
    DDCA_Error_Detail * edet = NULL;
 #ifdef ENABLE_UDEV
@@ -943,6 +971,8 @@ ddca_start_watch_displays(DDCA_Display_Event_Class enabled_classes) {
    }
    else if (enabled_classes&DDCA_EVENT_CLASS_DPMS) {
       edet = new_ddca_error_detail(DDCRC_UNIMPLEMENTED, "Watching for DPMS state changes unimplemented");
+	  // MSG_W_SYSLOG(DDCA_SYSLOG_WARNING, "Watching for DPMS state changes unimplemented");
+      //enabled_classes &=~DDCA_EVENT_CLASS_DPMS;
    }
    else if (enabled_classes != DDCA_EVENT_CLASS_DISPLAY_CONNECTION) {
       edet = new_ddca_error_detail (DDCRC_ARG, "Invalid event class specified");
@@ -961,6 +991,10 @@ ddca_start_watch_displays(DDCA_Display_Event_Class enabled_classes) {
       ddcrc = edet->status_code;
       save_thread_error_detail(edet);
    }
+#else
+   DDCA_Status ddcrc = DDCRC_UNIMPLEMENTED;
+#endif
+
    API_EPILOG_RET_DDCRC(debug, RESPECT_QUIESCE, ddcrc, "");
 }
 
@@ -969,8 +1003,12 @@ DDCA_Status
 ddca_stop_watch_displays(bool wait) {
    bool debug = false;
    API_PROLOGX(debug, NORESPECT_QUIESCE, "wait=%s", SBOOL(wait));
+#ifdef WATCH_DISPLAYS
    DDCA_Display_Event_Class active_classes;
    DDCA_Status ddcrc = dw_stop_watch_displays(wait, &active_classes);
+#else
+   DDCA_Status ddcrc = DDCRC_UNIMPLEMENTED;
+#endif
    API_EPILOG_RET_DDCRC(debug, NORESPECT_QUIESCE, ddcrc, "");
 }
 
@@ -979,7 +1017,11 @@ DDCA_Status
 ddca_get_active_watch_classes(DDCA_Display_Event_Class * classes_loc) {
    bool debug = false;
    API_PROLOGX(debug, NORESPECT_QUIESCE, "Starting classes_loc=%p", classes_loc);
+#ifdef WATCH_DISPLAYS
    DDCA_Status ddcrc = dw_get_active_watch_classes(classes_loc);
+#else
+   DDCA_Status ddcrc = DDCRC_UNIMPLEMENTED;
+#endif
    API_EPILOG_RET_DDCRC(debug, NORESPECT_QUIESCE, ddcrc, "*classes_loc=0x%02x", *classes_loc);
 }
 
@@ -988,11 +1030,15 @@ ddca_get_display_watch_settings(DDCA_DW_Settings * settings_buffer) {
    bool debug = false;
    API_PROLOGX(debug, NORESPECT_QUIESCE, "Starting");
 
+#ifdef WATCH_DISPLAYS
    DDCA_Status ddcrc = DDCRC_OK;
    if (!settings_buffer)
       ddcrc = DDCRC_ARG;
    else
       dw_get_display_watch_settings(settings_buffer);
+#else
+   DDCA_Status ddcrc = DDCRC_UNIMPLEMENTED;
+#endif
 
    API_EPILOG_RET_DDCRC(debug, NORESPECT_QUIESCE, ddcrc, "Done");
 }
@@ -1003,10 +1049,13 @@ ddca_set_display_watch_settings(DDCA_DW_Settings * settings_buffer) {
    bool debug = false;
    API_PROLOGX(debug, NORESPECT_QUIESCE, "Starting");
 
+#ifdef WATCH_DISPLAYS
    DDCA_Status ddcrc = DDCRC_ARG;
    if (settings_buffer)
       ddcrc = dw_set_display_watch_settings(settings_buffer);
-
+#else
+   DDCA_Status ddcrc = DDCRC_UNIMPLEMENTED;
+#endif
    API_EPILOG_RET_DDCRC(debug, NORESPECT_QUIESCE, ddcrc, "Done");
 }
 
@@ -1271,7 +1320,8 @@ double
 ddca_set_sleep_multiplier(double multiplier)
 {
    bool debug = false;
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+      reset_current_traced_function_stack();
    DBGTRC_STARTING(debug, DDCA_TRC_API, "Setting multiplier = %6.3f", multiplier);
 
    double old_value = -1.0;
@@ -1292,7 +1342,8 @@ double
 ddca_get_sleep_multiplier()
 {
    bool debug = false;
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+      reset_current_traced_function_stack();
    DBGTRC(debug, DDCA_TRC_API, "");
 
    Per_Thread_Data * ptd = ptd_get_per_thread_data();
@@ -1471,6 +1522,8 @@ void init_api_base() {
    RTTI_ADD_FUNC(ddca_end_capture);
    RTTI_ADD_FUNC(quiesce_api);
    RTTI_ADD_FUNC(unquiesce_api);
+   RTTI_ADD_FUNC(increment_active_api_calls);
+   RTTI_ADD_FUNC(decrement_active_api_calls);
 #ifdef REMOVED
    RTTI_ADD_FUNC(ddca_set_sleep_multiplier);
    RTTI_ADD_FUNC(ddca_set_default_sleep_multiplier);
