@@ -23,6 +23,7 @@
 
 #include "base/ddc_command_codes.h"
 #include "base/displays.h"
+#include "base/feature_lists.h"
 #include "base/feature_metadata.h"
 #include "base/rtti.h"
 #include "base/vcp_version.h"
@@ -236,7 +237,8 @@ ddca_free_parsed_capabilities(
       DDCA_Capabilities * pcaps)
 {
    bool debug = false;
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+      reset_current_traced_function_stack();
    DBGTRC_STARTING(debug, DDCA_TRC_API, "pcaps=%p", pcaps);
    if (pcaps) {
       assert(memcmp(pcaps->marker, DDCA_CAPABILITIES_MARKER, 4) == 0);
@@ -488,7 +490,14 @@ ddca_report_parsed_capabilities(
       DDCA_Capabilities *      p_caps,
       int                      depth)
 {
+   bool debug = false;
+   free_thread_error_detail();
+   API_PROLOG(debug, "Starting. p_caps=%p", //  ddca_dref=%s",
+                      p_caps);      // , dref_repr_t((Display_Ref*) ddca_dref));
+
    ddci_report_parsed_capabilities_by_dref(p_caps, NULL, depth);
+
+   API_EPILOG_NO_RETURN_BASIC(debug, "");
 }
 
 
@@ -501,7 +510,7 @@ ddca_report_parsed_capabilities_by_dh(
    bool debug = false;
    free_thread_error_detail();
    API_PROLOGX(debug, RESPECT_QUIESCE, "p_caps=%p, ddca_dh=%s, depth=%d",
-                      p_caps, ddca_dh_repr(ddca_dh), depth);
+                      p_caps, ddci_dh_repr(ddca_dh), depth);
    DDCA_Status ddcrc = 0;
    Display_Handle * dh = (Display_Handle *) ddca_dh;
    if (dh == NULL || memcmp(dh->marker, DISPLAY_HANDLE_MARKER, 4) != 0 ) {
@@ -571,7 +580,7 @@ ddca_feature_list_from_capabilities(
    DDCA_Feature_List result = {{0}};
    for (int ndx = 0; ndx < parsed_caps->vcp_code_ct; ndx++) {
       DDCA_Cap_Vcp curVcp = parsed_caps->vcp_codes[ndx];
-      ddca_feature_list_add(&result, curVcp.feature_code);
+      feature_list_add(&result, curVcp.feature_code);
    }
    return result;
 }
@@ -583,4 +592,5 @@ void init_api_capabilities() {
    RTTI_ADD_FUNC(ddci_report_parsed_capabilities_by_dref);
    RTTI_ADD_FUNC(ddca_report_parsed_capabilities_by_dref);
    RTTI_ADD_FUNC(ddca_report_parsed_capabilities_by_dh);
+   RTTI_ADD_FUNC(ddca_report_parsed_capabilities);
 }
