@@ -36,6 +36,7 @@
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_vcp_version.h"
 
+#include "dw/dw_common.h"
 #include "dw/dw_main.h"
 #include "dw/dw_status_events.h"
 #include "dw/dw_udev.h"
@@ -46,6 +47,10 @@
 #include "libmain/api_displays_internal.h"
 
 
+// Trace class for this file
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_API;
+
+
 static inline bool valid_display_handle(Display_Handle * dh) {
    return (dh && memcmp(dh->marker, DISPLAY_HANDLE_MARKER, 4) == 0);
 }
@@ -128,19 +133,6 @@ DDCA_Status ddci_validate_ddca_display_ref2(
 }
 
 
-#ifdef UNUSED
-Display_Handle * validated_ddca_display_handle(DDCA_Display_Handle ddca_dh) {
-   Display_Handle * dh = (Display_Handle *) ddca_dh;
-   if (dh) {
-      if (memcmp(dh->marker, DISPLAY_HANDLE_MARKER, 4) != 0  ||
-          !ddc_is_valid_display_handle(dh) )
-         dh=NULL;
-   }
-   return dh;
-}
-#endif
-
-
 /** Validates an opaque #DDCA_Display_Handle, returning the corresponding
  *  #Display_Handle if successful.
  *
@@ -190,7 +182,8 @@ ddca_create_dispno_display_identifier(
       DDCA_Display_Identifier* did_loc)
 {
    free_thread_error_detail();
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+     reset_current_traced_function_stack();
    // assert(did_loc);
    API_PRECOND(did_loc);
    Display_Identifier* did = create_dispno_display_identifier(dispno);
@@ -207,7 +200,8 @@ ddca_create_busno_display_identifier(
 {
    free_thread_error_detail();
    // assert(did_loc);
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+      reset_current_traced_function_stack();
    API_PRECOND(did_loc);
    Display_Identifier* did = create_busno_display_identifier(busno);
    *did_loc = did;
@@ -224,7 +218,8 @@ ddca_create_mfg_model_sn_display_identifier(
       DDCA_Display_Identifier* did_loc)
 {
    free_thread_error_detail();
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+      reset_current_traced_function_stack();
    // assert(did_loc);
    API_PRECOND(did_loc);
    *did_loc = NULL;
@@ -263,7 +258,8 @@ ddca_create_edid_display_identifier(
 {
    // assert(did_loc);
    free_thread_error_detail();
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+      reset_current_traced_function_stack();
    API_PRECOND(did_loc);
    *did_loc = NULL;
    DDCA_Status rc = 0;
@@ -287,7 +283,8 @@ ddca_create_usb_display_identifier(
 {
    // assert(did_loc);
    free_thread_error_detail();
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+      reset_current_traced_function_stack();
    API_PRECOND(did_loc);
    Display_Identifier* did = create_usb_display_identifier(bus, device);
    *did_loc = did;
@@ -303,7 +300,8 @@ ddca_create_usb_hiddev_display_identifier(
 {
    // assert(did_loc);
    free_thread_error_detail();
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+      reset_current_traced_function_stack();
    API_PRECOND(did_loc);
    Display_Identifier* did = create_usb_hiddev_display_identifier(hiddev_devno);
    *did_loc = did;
@@ -347,15 +345,13 @@ ddca_did_repr(DDCA_Display_Identifier ddca_did) {
 //
 
 DDCA_Status
-ddca_get_display_ref(
+ddci_get_display_ref(
       DDCA_Display_Identifier did,
       DDCA_Display_Ref*       dref_loc)
 {
-   free_thread_error_detail();
    bool debug = false;
-   API_PROLOGX(debug, NORESPECT_QUIESCE, "did=%p, dref_loc=%p", did, dref_loc);
-   assert(library_initialized);
-   API_PRECOND_W_EPILOG(dref_loc);
+   DBGTRC_STARTING(debug, TRACE_GROUP, "did=%s, dref_loc=%p", did_repr(did), dref_loc);
+
    *dref_loc = NULL;
    DDCA_Status rc = 0;
    ddc_ensure_displays_detected();
@@ -366,21 +362,39 @@ ddca_get_display_ref(
    }
    else {
       Display_Ref* dref = get_display_ref_for_display_identifier(pdid, CALLOPT_NONE);
-      if (debug)
-         DBGMSG("get_display_ref_for_display_identifier() returned %p", dref);
-      if (dref)
-         *dref_loc = dref;
+      DBGMSF(debug, "get_display_ref_for_display_identifier() returned %p", dref);
+      if (dref) {
+         DDCA_Display_Ref ddca_dref = dref_to_ddca_dref(dref);
+         add_published_dref_id_by_dref(dref);
+         *dref_loc = ddca_dref;
+      }
       else
          rc = DDCRC_INVALID_DISPLAY;
    }
 
-   API_EPILOG_BEFORE_RETURN(debug, NORESPECT_QUIESCE, rc,
-                        "*dref_loc=%p", psc_name_code(rc), *dref_loc);
+   DBGTRC_RET_DDCRC(debug, TRACE_GROUP, rc, "*dref_loc=%p", *dref_loc);
    TRACED_ASSERT( (rc==0 && *dref_loc) || (rc!=0 && !*dref_loc) );
    return rc;
 }
 
 
+DDCA_Status
+ddca_get_display_ref(
+      DDCA_Display_Identifier did,
+      DDCA_Display_Ref*       dref_loc)
+{
+   bool debug = false;
+   free_thread_error_detail();
+   API_PROLOGX(debug, RESPECT_QUIESCE, "did=%p, dref_loc=%p", did, dref_loc);
+   assert(library_initialized);
+   API_PRECOND_W_EPILOG(dref_loc);
+
+  DDCA_Status rc = ddci_get_display_ref(did, dref_loc);
+
+  API_EPILOG_BEFORE_RETURN(debug, RESPECT_QUIESCE, rc,
+                       "*dref_loc=%p", psc_name_code(rc), *dref_loc);
+  return rc;
+}
 
 
 // deprecated
@@ -389,69 +403,67 @@ ddca_create_display_ref(
       DDCA_Display_Identifier did,
       DDCA_Display_Ref*       dref_loc)
 {
-   return ddca_get_display_ref(did, dref_loc);
-}
-
-#ifdef REMOVED
-/** @deprecated All display references are persistent
- *
- *  Frees a display reference.
- *
- *  Use this function to safely release a #DDCA_Display_Ref.
- *  If the display reference was dynamically created, it is freed.
- *  If the display reference was permanently allocated (normal case), does nothing.
- *
- *  @param[in] dref  display reference to free
- *  @retval DDCRC_OK     success, or dref == NULL
- *  @retval DDCRC_ARG    dref does not point to a valid display reference
- *  @retval DDCRC_LOCKED dref is to a transient instance, and it is referenced
- *                       by an open display handle
- *
- *  @ingroup api_display_spec
- */
-// __attribute__ ((deprecated ("DDCA_Display_Refs are always persistent")))
-DDCA_Status
-ddca_free_display_ref(
-      DDCA_Display_Ref dref);
-#endif
-
-#ifdef REMOVED
-// deprecated, not needed, in library there are no transient display refs
-DDCA_Status
-ddca_free_display_ref(DDCA_Display_Ref ddca_dref) {
    bool debug = false;
-   API_PROLOG(debug, "ddca_dref=%p", ddca_dref);
-   DDCA_Status psc = 0;
    free_thread_error_detail();
-   if (ddca_dref) {
-      WITH_VALIDATED_DR3(ddca_dref, psc,
-         {
-            if (dref->flags & DREF_TRANSIENT)
-               psc = free_display_ref(dref);
-         }
-      );
-   }
-   API_EPILOG_BEFORE_RETURN(debug, psc, "");
-   return psc;
-}
-#endif
+   API_PROLOGX(debug, RESPECT_QUIESCE, "did=%p, dref_loc=%p", did, dref_loc);
+   assert(library_initialized);
+   API_PRECOND_W_EPILOG(dref_loc);
 
+   DDCA_Status rc = ddci_get_display_ref(did, dref_loc);
+
+   API_EPILOG_BEFORE_RETURN(debug, RESPECT_QUIESCE, rc,
+                        "*dref_loc=%p", psc_name_code(rc), *dref_loc);
+   return rc;
+}
+
+
+// GMutex ddca_redetect_active_mutex;
+static bool ddca_redetect_active = false;
 
 DDCA_Status
 ddca_redetect_displays() {
    bool debug = false;
    API_PROLOGX(debug, NORESPECT_QUIESCE, "");
-   quiesce_api();
-   dw_redetect_displays();
-   unquiesce_api();
-   API_EPILOG_RET_DDCRC(debug, NORESPECT_QUIESCE, 0, "");
+
+   DDCA_Status ddcrc = 0;
+   bool perform_detect = true;
+
+//   g_mutex_lock(ddca_redetect_active_mutex);
+   if (ddca_redetect_active) {
+	   SYSLOG2(DDCA_SYSLOG_ERROR, "Calling ddca_redetect_displays() when already active");
+	   perform_detect = false;
+	   ddcrc = DDCRC_INVALID_OPERATION;
+   }
+   // g_mutex_unlock(ddca_redetect_active_mutex);
+
+#ifdef WATCH_DISPLAYS
+   if (perform_detect) {
+      if (active_callback_thread_ct() > 0) {
+         SYSLOG2(DDCA_SYSLOG_ERROR, "Calling ddca_redetect_display() when callback threads are active");
+         SYSLOG2(DDCA_SYSLOG_ERROR, "Behavior is indeterminate.");
+         // ddcrc = DDCRC_INVALID_OPERATION;
+         // perform_detect = false;
+      }
+   }
+#endif
+
+   if (perform_detect) {
+	  ddca_redetect_active = true;
+      quiesce_api();
+      dw_redetect_displays();
+      unquiesce_api();
+      ddca_redetect_active = false;
+   }
+
+   API_EPILOG_RET_DDCRC(debug, NORESPECT_QUIESCE, ddcrc, "");
 }
 
 
 const char *
 ddca_dref_repr(DDCA_Display_Ref ddca_dref) {
    bool debug = false;
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+      reset_current_traced_function_stack();
    DBGTRC_STARTING(debug, DDCA_TRC_NONE, "ddca_dref=%p", ddca_dref);
 
    Display_Ref * dref = dref_from_published_ddca_dref(ddca_dref);
@@ -468,7 +480,8 @@ ddca_dbgrpt_display_ref(
       int              depth)
 {
    bool debug = false;
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+      reset_current_traced_function_stack();
    DBGMSF(debug, "Starting.  ddca_dref = %p, depth=%d", ddca_dref, depth);
    Display_Ref * dref = ddca_dref;
    if (dref && memcmp(dref->marker, DISPLAY_REF_MARKER, 4) == 0) {
@@ -555,113 +568,71 @@ ddca_validate_display_ref(DDCA_Display_Ref ddca_dref, bool require_not_asleep)
 // Open and close display
 //
 
-#ifdef DEPRECATED
-
-/** \deprecated Use #ddca_open_display2()
- * Open a display
- * @param[in]  ddca_dref    display reference for display to open
- * @param[out] ddca_dh_loc  where to return display handle
- * @return     status code
- *
- * Fails if display is already opened by another thread.
- * \ingroup api_display_spec
- */
-// __attribute__ ((deprecated ("use ddca_open_display2()")))
-DDCA_Status
-ddca_open_display(
-      DDCA_Display_Ref      ddca_dref,
-      DDCA_Display_Handle * ddca_dh_loc);
-
-DDCA_Status
-ddca_open_display(
-      DDCA_Display_Ref      ddca_dref,
-      DDCA_Display_Handle * dh_loc)
-{
-   return ddca_open_display2(ddca_dref, false, dh_loc);
-}
-#endif
-
-
-// unpublished extension
-
-
 /** Options for opening a DDCA_Display_Ref
+ *
+ *  This is a vestigial remnant of what was once a larger set of options.
  */
 
 typedef enum {
-   DDCA_OPENOPT_NONE             = 0,
-   DDCA_OPENOPT_WAIT             = 1,
-   DDCA_OPENOPT_FORCE_SLAVE_ADDR = 2
-} DDCA_Open_Options;
+   DDC_OPENOPT_NONE             = 0,
+   DDC_OPENOPT_WAIT             = 1,
+} DDC_Open_Options;
 
 
 STATIC
-DDCA_Status
-ddca_open_display3(
+Error_Info *
+ddci_open_display3(
       DDCA_Display_Ref      ddca_dref,
-      DDCA_Open_Options     options,
+      DDC_Open_Options      options,
       DDCA_Display_Handle * dh_loc)
 {
    bool debug = false;
-   free_thread_error_detail();
-   API_PROLOGX(debug, RESPECT_QUIESCE,
+   DBGTRC_STARTING(debug, DDCA_TRC_API,
           "ddca_dref=%p, options=0x%02x, dh_loc=%p, on thread %d",
           ddca_dref, options, dh_loc, get_thread_id());
+
    DBGTRC_NOPREFIX(debug, DDCA_TRC_API,
-          "library_initialized=%s, ddc_displays_already_detected() = %ld",
-          sbool(library_initialized), ddc_displays_already_detected());
+          "library_initialized=%s, ddc_displays_already_detected() = %s",
+          sbool(library_initialized), sbool(ddc_displays_already_detected()));
    TRACED_ASSERT(library_initialized);
    TRACED_ASSERT(ddc_displays_already_detected());
 
-   API_PRECOND_W_EPILOG(dh_loc);
    Display_Ref * dref = NULL;
    *dh_loc = NULL;        // in case of error
    DDCA_Status rc = 0;
    Error_Info * err = NULL;
    Display_Ref * dref0 = dref_from_published_ddca_dref(ddca_dref);
-   if (dref0) {
-      // rc  = ddci_validate_ddca_display_ref(ddca_dref, /* basic_only*/ false, /* require_not_asleep */ true, &dref);
-      rc  = ddci_validate_ddca_display_ref2(ddca_dref, DREF_VALIDATE_EDID | DREF_VALIDATE_AWAKE, &dref);
-      if (!rc) {
-        Display_Handle* dh = NULL;
-        Call_Options callopts = CALLOPT_NONE;
-        if (options & DDCA_OPENOPT_WAIT)
-           callopts |= CALLOPT_WAIT;
-        if (options & DDCA_OPENOPT_FORCE_SLAVE_ADDR)
-           callopts |= CALLOPT_FORCE_SLAVE_ADDR;
-        err = ddc_open_display(dref,  callopts, &dh);
-        if (!err)
-           *dh_loc = dh;
-        else {
-           rc = err->status_code;
-           char * detail2 = g_strdup_printf("%s, Internal display ref: %s", err->detail, dref_reprx_t(dref));
-           free(err->detail);
-           err->detail = detail2;
-
-           DDCA_Error_Detail * public_error_detail = error_info_to_ddca_detail(err);
-           errinfo_free_with_report(err, debug, __func__);
-           save_thread_error_detail(public_error_detail);
-        }
-      }
-      else {
-         Error_Info * err = ERRINFO_NEW(DDCRC_INVALID_DISPLAY, "Invalid display ref");
-         DDCA_Error_Detail * public_error_detail = error_info_to_ddca_detail(err);
-         errinfo_free_with_report(err, debug, __func__);
-         save_thread_error_detail(public_error_detail);
-      }
+   if (!dref0) {
+      err = ERRINFO_NEW(DDCRC_INVALID_DISPLAY, "Unknown display ref");
    }
    else {
-      Error_Info * err = ERRINFO_NEW(DDCRC_INVALID_DISPLAY, "Unknown display ref");
-      DDCA_Error_Detail * public_error_detail = error_info_to_ddca_detail(err);
-      errinfo_free_with_report(err, debug, __func__);
-      save_thread_error_detail(public_error_detail);
+      rc = ddci_validate_ddca_display_ref2(
+            ddca_dref, DREF_VALIDATE_EDID | DREF_VALIDATE_AWAKE, &dref);
+      if (rc) {
+         err = ERRINFO_NEW(DDCRC_INVALID_DISPLAY,
+               "Invalid display ref, ddci_validate_ddca_display_ref2 returned %s", psc_desc(rc));
+      }
+      else {
+         Display_Handle * dh;
+         Call_Options callopts = CALLOPT_NONE;
+         if (options & DDC_OPENOPT_WAIT)
+            callopts |= CALLOPT_WAIT;
+         err = ddc_open_display(dref,  callopts, &dh);
+         if (err) {
+            char * detail2 = g_strdup_printf("%s, Internal display ref: %s",
+                                             err->detail, dref_reprx_t(dref));
+            free(err->detail);
+            err->detail = detail2;
+         }
+         else {
+            *dh_loc = dh;
+         }
+      }
    }
 
-
-   API_EPILOG_BEFORE_RETURN(debug, RESPECT_QUIESCE, rc,
-                        "*dh_loc=%p -> %s", *dh_loc, dh_repr(*dh_loc));
-   TRACED_ASSERT_IFF(rc==0, *dh_loc);
-   return rc;
+   DBGTRC_RET_ERRINFO(debug, DDCA_TRC_API, err, "*dh_loc=%p -> %s", *dh_loc, dh_repr(*dh_loc));
+   TRACED_ASSERT_IFF(!err, *dh_loc);
+   return err;
 }
 
 
@@ -671,9 +642,29 @@ ddca_open_display2(
       bool                  wait,
       DDCA_Display_Handle * dh_loc)
 {
-   return ddca_open_display3(ddca_dref,
-                             (wait) ? DDCA_OPENOPT_WAIT : DDCA_OPENOPT_NONE,
+   bool debug = false;
+   free_thread_error_detail();
+   API_PROLOGX(debug, RESPECT_QUIESCE,
+           "ddca_dref=%p, wait=%s, dh_loc=%p, on thread %d",
+           ddca_dref, SBOOL(wait), dh_loc, get_thread_id());
+   API_PRECOND_W_EPILOG(dh_loc);
+
+   Error_Info * err = ddci_open_display3(ddca_dref,
+                             (wait) ? DDC_OPENOPT_WAIT : DDC_OPENOPT_NONE,
                              dh_loc);
+   int ddcrc = 0;
+   if (err) {
+      ddcrc = err->status_code;
+      DDCA_Error_Detail * public_error_detail = error_info_to_ddca_detail(err);
+      errinfo_free_with_report(err, debug, __func__);
+      save_thread_error_detail(public_error_detail);
+   }
+
+   API_EPILOG_BEFORE_RETURN(debug, RESPECT_QUIESCE, ddcrc,
+                        "*dh_loc=%p -> %s", *dh_loc, dh_repr(*dh_loc));
+   TRACED_ASSERT_IFF(ddcrc==0, *dh_loc);
+   return ddcrc;
+
 }
 
 
@@ -684,7 +675,7 @@ ddca_close_display(DDCA_Display_Handle ddca_dh) {
    DDCA_Status rc = 0;
    Error_Info * err = NULL;
    Display_Handle * dh = (Display_Handle *) ddca_dh;
-   API_PROLOGX(debug, RESPECT_QUIESCE, "dh = %s", dh_repr(dh));
+   API_PROLOGX(debug, NORESPECT_QUIESCE, "dh = %s", dh_repr(dh));
    if (dh) {
       if (memcmp(dh->marker, DISPLAY_HANDLE_MARKER, 4) != 0 )  {
          err = ERRINFO_NEW(DDCRC_ARG, "Invalid display handle");
@@ -702,7 +693,7 @@ ddca_close_display(DDCA_Display_Handle ddca_dh) {
       save_thread_error_detail(public_error_detail);
    }
 
-   API_EPILOG_BEFORE_RETURN(debug, RESPECT_QUIESCE, rc, "");
+   API_EPILOG_BEFORE_RETURN(debug, NORESPECT_QUIESCE, rc, "");
    return rc;
 }
 
@@ -712,7 +703,7 @@ ddca_close_display(DDCA_Display_Handle ddca_dh) {
 //
 
 const char *
-ddca_dh_repr(DDCA_Display_Handle ddca_dh) {
+ddci_dh_repr(DDCA_Display_Handle ddca_dh) {
    char * repr = NULL;
    Display_Handle * dh = (Display_Handle *) ddca_dh;
    if (valid_display_handle(dh))
@@ -721,6 +712,12 @@ ddca_dh_repr(DDCA_Display_Handle ddca_dh) {
 }
 
 
+const char *
+ddca_dh_repr(DDCA_Display_Handle ddca_dh) {
+	return ddci_dh_repr(ddca_dh);
+}
+
+
 DDCA_Display_Ref
 ddca_display_ref_from_handle(
       DDCA_Display_Handle   ddca_dh)
@@ -898,24 +895,6 @@ ddca_mmk_from_dh(
 // Display Info
 //
 
-#ifdef DEPRECATED
-/** @deprecated use #ddca_get_display_info_list2()
- * Gets a list of the detected displays.
- *
- *  Displays that do not support DDC are not included.
- *
- *  @return list of display summaries
- */
-__attribute__ ((deprecated ("use ddca_get_display_info_list2()")))
-DDCA_Display_Info_List *
-ddca_get_display_info_list(void)
-{
-   DDCA_Display_Info_List * result = NULL;
-   ddca_get_display_info_list2(false, &result);
-   return result;
-}
-#endif
-
 STATIC
  void ddci_init_display_info(Display_Ref * dref, DDCA_Display_Info * curinfo) {
    bool debug = false;
@@ -933,21 +912,11 @@ STATIC
    if (dref->dispno > 0 && (dref->flags&DREF_DDC_COMMUNICATION_WORKING)) {
       vspec = get_vcp_version_by_dref(dref);
    }
-   memcpy(curinfo->edid_bytes,    dref->pedid->bytes, 128);
-
-#if __GNUC__ >= 8
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wstringop-truncation"
-   STRLCPY(curinfo->mfg_id,     dref->pedid->mfg_id,       EDID_MFG_ID_FIELD_SIZE);
-   STRLCPY(curinfo->model_name, dref->pedid->model_name,   EDID_MODEL_NAME_FIELD_SIZE);
-   STRLCPY(curinfo->sn,         dref->pedid->serial_ascii, DDCA_EDID_SN_ASCII_FIELD_SIZE);
-#pragma GCC diagnostic pop
-#else
-   STRLCPY(curinfo->mfg_id,     dref->pedid->mfg_id,       EDID_MFG_ID_FIELD_SIZE);
-   STRLCPY(curinfo->model_name, dref->pedid->model_name,   EDID_MODEL_NAME_FIELD_SIZE);
-   STRLCPY(curinfo->sn,         dref->pedid->serial_ascii, DDCA_EDID_SN_ASCII_FIELD_SIZE);
-#endif
-   curinfo->product_code  = dref->pedid->product_code;
+   memcpy(   curinfo->edid_bytes, dref->pedid->bytes, 128);
+   G_STRLCPY(curinfo->mfg_id,     dref->pedid->mfg_id,       EDID_MFG_ID_FIELD_SIZE);
+   G_STRLCPY(curinfo->model_name, dref->pedid->model_name,   EDID_MODEL_NAME_FIELD_SIZE);
+   G_STRLCPY(curinfo->sn,         dref->pedid->serial_ascii, DDCA_EDID_SN_ASCII_FIELD_SIZE);
+   curinfo->product_code   = dref->pedid->product_code;
    curinfo->vcp_version    = vspec;
    curinfo->dref           = dref_to_ddca_dref(dref);
 
@@ -967,6 +936,65 @@ STATIC
 }
 
 
+STATIC
+DDCA_Drm_Connector_Found_By drm_to_ddca_connector_found_by(Drm_Connector_Found_By drm_value) {
+   DDCA_Drm_Connector_Found_By ddca_value = DDCA_DRM_CONNECTOR_NOT_FOUND;  // pointless initialization to avoid compiler warning
+   assert(drm_value != DRM_CONNECTOR_NOT_CHECKED);
+   switch (drm_value) {
+   case DRM_CONNECTOR_NOT_FOUND:      ddca_value = DDCA_DRM_CONNECTOR_NOT_FOUND;      break;
+   case DRM_CONNECTOR_FOUND_BY_BUSNO: ddca_value = DDCA_DRM_CONNECTOR_FOUND_BY_BUSNO; break;
+   case DRM_CONNECTOR_FOUND_BY_EDID:  ddca_value = DDCA_DRM_CONNECTOR_FOUND_BY_EDID;  break;
+   case DRM_CONNECTOR_NOT_CHECKED:    assert(false);
+   }
+   return ddca_value;
+}
+
+
+STATIC
+ void ddci_init_display_info2(Display_Ref * dref, DDCA_Display_Info2 * curinfo) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_API, "dref=%s, curinfo=%p", dref_reprx_t(dref),curinfo);
+   memcpy(curinfo->marker, DDCA_DISPLAY_INFO_MARKER, 4);
+   curinfo->dispno        = dref->dispno;
+
+   curinfo->path = dref->io_path;
+   if (dref->io_path.io_mode == DDCA_IO_USB) {
+      curinfo->usb_bus    = dref->usb_bus;
+      curinfo->usb_device = dref->usb_device;
+   }
+
+   DDCA_MCCS_Version_Spec vspec = DDCA_VSPEC_UNKNOWN;
+   if (dref->dispno > 0 && (dref->flags&DREF_DDC_COMMUNICATION_WORKING)) {
+      vspec = get_vcp_version_by_dref(dref);
+   }
+   memcpy(curinfo->edid_bytes,    dref->pedid->bytes, 128);
+   G_STRLCPY(curinfo->mfg_id,     dref->pedid->mfg_id,       EDID_MFG_ID_FIELD_SIZE);
+   G_STRLCPY(curinfo->model_name, dref->pedid->model_name,   EDID_MODEL_NAME_FIELD_SIZE);
+   G_STRLCPY(curinfo->sn,         dref->pedid->serial_ascii, DDCA_EDID_SN_ASCII_FIELD_SIZE);
+   curinfo->product_code  = dref->pedid->product_code;
+   curinfo->vcp_version   = vspec;
+   curinfo->dref          = dref_to_ddca_dref(dref);
+
+   // Additional fields in DDCA_Display_Info2 but not DDCA_Display_Info
+
+   if (dref->io_path.io_mode == DDCA_IO_I2C) {
+      I2C_Bus_Info * businfo = dref->detail;
+      if (businfo->drm_connector_name) {
+         G_STRLCPY(curinfo->drm_card_connector, businfo->drm_connector_name, DDCA_DRM_CONNECTOR_FIELD_SIZE);
+         curinfo->drm_card_connector_found_by =  drm_to_ddca_connector_found_by(businfo->drm_connector_found_by);
+         curinfo->drm_connector_id = businfo->drm_connector_id;
+      }
+      else {
+         curinfo->drm_card_connector[0] = '\0';
+         curinfo->drm_card_connector_found_by = DDCA_DRM_CONNECTOR_NOT_FOUND;
+         curinfo->drm_connector_id = -1;
+      }
+   }
+
+   DBGTRC_DONE(debug, DDCA_TRC_API, "dref=%s", dref_reprx_t(dref));
+}
+
+
 DDCA_Status
 ddca_get_display_info(
       DDCA_Display_Ref  ddca_dref,
@@ -997,6 +1025,37 @@ ddca_get_display_info(
 }
 
 
+
+DDCA_Status
+ddca_get_display_info2(
+      DDCA_Display_Ref      ddca_dref,
+      DDCA_Display_Info2 ** dinfo_loc)
+{
+   bool debug = false;
+
+   Display_Ref * dref0 = dref_from_published_ddca_dref(ddca_dref);
+
+   // causes return DDCRC_UNINTIALIZED: called after explicit ddca_init()/init2() call failed
+   API_PROLOGX(debug, RESPECT_QUIESCE, "ddca_dref=%p, dref0=%s", ddca_dref, dref_reprx_t(dref0));
+   // causes return DDCRC_ARG if dinfo_loc == NULL
+   API_PRECOND_W_EPILOG(dinfo_loc);
+   DDCA_Status ddcrc = 0;
+
+   // if ddc_validate_display_ref() fails, returns its status code
+   WITH_VALIDATED_DR4(
+         ddca_dref, ddcrc, DREF_VALIDATE_EDID | DREF_VALIDATE_DDC_COMMUNICATION_FAILURE_OK,
+         {
+            DDCA_Display_Info2 * info = calloc(1, sizeof(DDCA_Display_Info2));
+            ddci_init_display_info2(dref, info);
+            *dinfo_loc = info;
+         }
+   )
+
+   API_EPILOG_BEFORE_RETURN(debug, RESPECT_QUIESCE, ddcrc, "ddca_dref=%p, dref=%s", ddca_dref, dref_reprx_t(dref0));
+   return ddcrc;
+}
+
+
 STATIC void
 set_ddca_error_detail_from_open_errors() {
    bool debug = false;
@@ -1142,15 +1201,28 @@ ddca_get_display_info_list2(
 void
 ddca_free_display_info(DDCA_Display_Info * info_rec) {
    bool debug = false;
-   API_PROLOG(debug, "info_rec->%p", info_rec);
-   // DDCA_Display_Info contains no pointers, can simply be free'd
-   // data structures.  Nothing to free.
+   API_PROLOG_NO_DISPLAY_IO(debug, "info_rec=%p", info_rec);
    if (info_rec && memcmp(info_rec->marker, DDCA_DISPLAY_INFO_MARKER, 4) == 0) {
+      // DDCA_IO_Path path = info_rec->path;
+      // DDCA_Display_Info contains no pointers, can simply be free'd.
+      info_rec->marker[3] = 'x';
+      free(info_rec);
+   }
+   API_EPILOG_NO_RETURN(debug, false, "");
+   DISABLE_API_CALL_TRACING();
+}
+
+
+void
+ddca_free_display_info2(DDCA_Display_Info2 * info_rec) {
+   bool debug = false;
+   API_PROLOG_NO_DISPLAY_IO(debug, "info_rec=%p", info_rec);
+   if (info_rec && memcmp(info_rec->marker, DDCA_DISPLAY_INFO_MARKER, 4) == 0) {
+      // DDCA_Display_Info contains no pointers, can simply be free'd
       info_rec->marker[3] = 'x';
       free(info_rec);
    }
    API_EPILOG_NO_RETURN(debug, false, "");
-   // DBGTRC_DONE(debug, DDCA_TRC_API,"");
    DISABLE_API_CALL_TRACING();
 }
 
@@ -1175,8 +1247,8 @@ ddca_free_display_info_list(DDCA_Display_Info_List * dlist) {
 }
 
 
-DDCA_Status
-ddca_report_display_info(
+static DDCA_Status
+ddci_report_display_info(
       DDCA_Display_Info * dinfo,
       int                 depth)
 {
@@ -1198,14 +1270,15 @@ ddca_report_display_info(
          rpt_label(  d0, "Invalid display - Does not support DDC");
       // rpt_vstring(      d1, "Display ref:         %p -> %s", dinfo->dref, dref_repr_t(dinfo->dref) );
       // rpt_vstring(d1, "IO mode:             %s", io_mode_name(dinfo->path.io_mode));
+      int tw = 22;   // title width
       switch(dinfo->path.io_mode) {
       case (DDCA_IO_I2C):
-            rpt_vstring(d1, "I2C bus:              /dev/i2c-%d", dinfo->path.path.i2c_busno);
+            rpt_vstring(d1, "%-*s /dev/i2c-%d", tw, "I2C bus:", dinfo->path.path.i2c_busno);
             break;
       case (DDCA_IO_USB):
-            rpt_vstring(d1, "USB bus.device:       %d.%d",
+            rpt_vstring(d1, "%-*s %d.%d", tw, "USB bus device:",
                             dinfo->usb_bus, dinfo->usb_device);
-            rpt_vstring(d1, "USB hiddev device:   /dev/usb/hiddev%d", dinfo->path.path.hiddev_devno);
+            rpt_vstring(d1, "%-*s /dev/usb/hiddev%d", tw, "USB hiddev device:", dinfo->path.path.hiddev_devno);
             break;
       }
 
@@ -1213,23 +1286,25 @@ ddca_report_display_info(
       Display_Ref * dref = dref_from_published_ddca_dref(dinfo->dref);
       if (dref) {   // should never fail, but just in case
          if (dref->drm_connector_id > 0)
-            // rpt_vstring(d1, "DRM connector id:        %d",  dref->drm_connector_id);
-            rpt_vstring(d1, "DRM connector:        %s (id: %d)",  dref->drm_connector, dref->drm_connector_id);
+            // rpt_vstring(d1, "%-*s %d", tw, "DRM connector id:", dref->drm_connector_id);
+            rpt_vstring(d1, "%-*s %s (id: %d)",  tw, "DRM connector:", dref->drm_connector, dref->drm_connector_id);
          else
-            rpt_vstring(d1, "DRM connector:        %s",  dref->drm_connector);
+            rpt_vstring(d1, "%-*s %s",  tw, "DRM connector:", dref->drm_connector);
+         // TMI:
+         // rpt_vstring(d1, "%-*s %s",  tw, "DRM connector found by:",  drm_connector_found_by_public_name(dref->drm_connector_found_by));
       }
 
-      rpt_vstring(d1, "Mfg Id:               %s", dinfo->mfg_id);
-      rpt_vstring(d1, "Model:                %s", dinfo->model_name);
-      rpt_vstring(d1, "Product code:         %u", dinfo->product_code);
-      rpt_vstring(d1, "Serial number:        %s", dinfo->sn);
+      rpt_vstring(d1, "%-*s %s", tw, "Mfg id:",dinfo->mfg_id);
+      rpt_vstring(d1, "%-*s %s", tw, "Model:", dinfo->model_name);
+      rpt_vstring(d1, "%-*s %u", tw, "Product code:", dinfo->product_code);
+      rpt_vstring(d1, "%-*s %s", tw, "Serial number:",  dinfo->sn);
 
       // binary SN is not part of DDCA_Display_Info
       Parsed_Edid * edid = create_parsed_edid(dinfo->edid_bytes);
       if (edid) {     // should never fail, but being ultra-cautious
          // Binary serial number is typically 0x00000000 or 0x01010101, but occasionally
          // useful for differentiating displays that share a generic ASCII "serial number"
-         rpt_vstring(d1,"Binary serial number: %"PRIu32" (0x%08x)",
+         rpt_vstring(d1,"%-*s %"PRIu32" (0x%08x)", tw, "Binary serial number:",
                         edid->serial_binary, edid->serial_binary);
          free_parsed_edid(edid);
       }
@@ -1256,7 +1331,7 @@ ddca_report_display_info(
       // OLD: rpt_hex_dump(dinfo->edid_bytes, 128, d2);
 
       // rpt_vstring(d1, "dref:                %p", dinfo->dref);
-      rpt_vstring(d1, "VCP Version:          %s", format_vspec(dinfo->vcp_version));
+      rpt_vstring(d1, "%-*s %s", tw, "VCP Version:", format_vspec(dinfo->vcp_version));
       // rpt_vstring(d1, "VCP Version Id:      %s", format_vcp_version_id(dinfo->vcp_version_id) );
 
       if (dinfo->dispno == DISPNO_BUSY) {
@@ -1294,6 +1369,24 @@ ddca_report_display_info(
 }
 
 
+DDCA_Status
+ddca_report_display_info(
+      DDCA_Display_Info * dinfo,
+      int                 depth)
+{
+	return ddci_report_display_info(dinfo, depth);
+}
+
+
+DDCA_Status
+ddca_report_display_info2(
+      DDCA_Display_Info2 * dinfo,
+      int                  depth)
+{
+   return ddci_report_display_info((DDCA_Display_Info*) dinfo, depth);
+}
+
+
 STATIC void
 dbgrpt_display_info(
       DDCA_Display_Info * dinfo,
@@ -1301,7 +1394,7 @@ dbgrpt_display_info(
 {
    bool debug = false;
    DBGMSF(debug, "Starting. dinfo=%p");
-   ddca_report_display_info(dinfo, depth);
+   ddci_report_display_info(dinfo, depth);
    int d1 = depth+1;
 
    rpt_vstring(d1, "dref:                %s", dref_repr_t(dinfo->dref));
@@ -1311,6 +1404,25 @@ dbgrpt_display_info(
    DBGMSF(debug, "Done.");
 }
 
+#ifdef UNUSED
+STATIC void
+dbgrpt_display_info2(
+      DDCA_Display_Info2 * dinfo,
+      int                  depth)
+{
+   bool debug = false;
+   DBGMSF(debug, "Starting. dinfo=%p");
+   ddci_report_display_info((DDCA_Display_Info*)dinfo, depth);
+   int d1 = depth+1;
+
+   rpt_vstring(d1, "dref:                %s", dref_repr_t(dinfo->dref));
+   if (dinfo->dref) {  // paranoid, should never be NULL
+      rpt_vstring(d1, "VCP Version (dref xdf): %s", format_vspec_verbose(((Display_Ref*)dinfo->dref)->vcp_version_xdf));
+   }
+   DBGMSF(debug, "Done.");
+}
+#endif
+
 
 void
 ddca_report_display_info_list(
@@ -1324,7 +1436,7 @@ ddca_report_display_info_list(
    int d1 = depth+1;
    rpt_vstring(depth, "Found %d displays", dlist->ct);
    for (int ndx=0; ndx<dlist->ct; ndx++) {
-      ddca_report_display_info(&dlist->info[ndx], d1);
+      ddci_report_display_info(&dlist->info[ndx], d1);
    }
    API_EPILOG_NO_RETURN(debug, false, "");
 }
@@ -1347,90 +1459,10 @@ dbgrpt_display_info_list(
 }
 
 
-//
-// Miscellaneous
-//
-
-#ifdef DEPRECATED
-
-// /** \deprecated */
-__attribute__ ((deprecated))
-DDCA_Status
-ddca_get_edid_by_dref(
-      DDCA_Display_Ref ddca_dref,
-      uint8_t **       pbytes_loc);   // pointer into ddcutil data structures, do not free
-
-
-// deprecated
-DDCA_Status
-ddca_get_edid_by_dref(
-      DDCA_Display_Ref ddca_dref,
-      uint8_t**        p_bytes)
-{
-   DDCA_Status rc = 0;
-   *p_bytes = NULL;
-   free_thread_error_detail();
-
-   assert(library_initialized);
-
-   Display_Ref * dref = (Display_Ref *) ddca_dref;
-   // if (dref == NULL || memcmp(dref->marker, DISPLAY_REF_MARKER, 4) != 0 )  {
-   if ( !valid_display_ref(dref) )  {
-      rc = DDCRC_ARG;
-      goto bye;
-   }
-
-   // Parsed_Edid*  edid = ddc_get_parsed_edid_by_dref(dref);
-   Parsed_Edid * edid = dref->pedid;
-   assert(edid);
-   *p_bytes = edid->bytes;
-
-bye:
-   return rc;
-}
-#endif
-
-
-#ifdef UNIMPLEMENTED
-// Use ddca_get_edid_by_dref() instead
-// n. edid_buffer must be >= 128 bytes
-
-DDCA_Status
-ddca_get_edid(DDCA_Display_Handle * dh, uint8_t* edid_buffer);
-#endif
-
-
 //
 // Reports
 //
 
-#ifdef DEPRECATED
-/** \deprecated use #ddca_report_displays()
- * Reports on all active displays.
- *  This function hooks into the code used by command "ddcutil detect"
- *
- *  @param[in] depth  logical indentation depth
- *  @return    number of MCCS capable displays
- */
-__attribute__ ((deprecated ("use ddca_report_displays()")))
-int
-ddca_report_active_displays(
-      int depth);
-
-
-// deprecated, use ddca_report_displays()
-int
-ddca_report_active_displays(int depth) {
-   bool debug = false;
-   API_PROLOG(debug, "");
-   int display_ct = ddc_report_displays(false, depth);
-   DBGTRC_DONE(debug, DDCA_TRC_API, "Returning %d", display_ct);
-   DISABLE_API_CALL_TRACING();
-   return display_ct;
-}
-#endif
-
-
 // TODO: deprecate, does not respect quiesced
 int
 ddca_report_displays(bool include_invalid_displays, int depth) {
@@ -1467,7 +1499,6 @@ ddca_register_display_status_callback(DDCA_Display_Status_Callback_Func func);
 #endif
 
 
-
 //
 // Display Status Change Communication
 //
@@ -1478,11 +1509,15 @@ ddca_register_display_status_callback(DDCA_Display_Status_Callback_Func func) {
    free_thread_error_detail();
    API_PROLOGX(debug, RESPECT_QUIESCE, "func=%p", func);
 
+#ifdef WATCH_DISPLAYS
    DDCA_Status result = DDCRC_INVALID_OPERATION;
  #ifdef ENABLE_UDEV
     if (check_all_video_adapters_implement_drm())
        result = dw_register_display_status_callback(func);
  #endif
+#else
+    DDCA_Status result = DDCRC_UNIMPLEMENTED;
+#endif
 
    API_EPILOG_RET_DDCRC(debug, RESPECT_QUIESCE, result, "func=%p", func);
    return result;
@@ -1495,7 +1530,11 @@ ddca_unregister_display_status_callback(DDCA_Display_Status_Callback_Func func)
    free_thread_error_detail();
    API_PROLOGX(debug, RESPECT_QUIESCE, "func=%p", func);
 
+#ifdef WATCH_DISPLAYS
    DDCA_Status result = dw_unregister_display_status_callback(func);
+#else
+   DDCA_Status result = DDCRC_UNIMPLEMENTED;
+#endif
 
    API_EPILOG_RET_DDCRC(debug, RESPECT_QUIESCE, result, "func=%p", func);
    return result;
@@ -1504,7 +1543,11 @@ ddca_unregister_display_status_callback(DDCA_Display_Status_Callback_Func func)
 
 const char *
    ddca_display_event_type_name(DDCA_Display_Event_Type event_type) {
+#ifdef WATCH_DISPLAYS
       return dw_display_event_type_name(event_type);
+#else
+      return NULL;
+#endif
 }
 
 //
@@ -1595,15 +1638,19 @@ void init_api_displays() {
    RTTI_ADD_FUNC(ddca_close_display);
    RTTI_ADD_FUNC(ddca_get_display_info_list2);
    RTTI_ADD_FUNC(ddca_get_display_info);
+   RTTI_ADD_FUNC(ddca_get_display_info2);
+   RTTI_ADD_FUNC(ddci_get_display_ref);
    RTTI_ADD_FUNC(ddca_get_display_ref);
+   RTTI_ADD_FUNC(ddca_create_display_ref);
    RTTI_ADD_FUNC(ddca_get_display_refs);
    RTTI_ADD_FUNC(ddca_open_display2);
-   RTTI_ADD_FUNC(ddca_open_display3);
+   RTTI_ADD_FUNC(ddci_open_display3);
    RTTI_ADD_FUNC(ddca_redetect_displays);
    RTTI_ADD_FUNC(ddca_report_display_by_dref);
    RTTI_ADD_FUNC(ddca_register_display_status_callback);
    RTTI_ADD_FUNC(ddca_unregister_display_status_callback);
    RTTI_ADD_FUNC(ddci_init_display_info);
+   RTTI_ADD_FUNC(ddci_init_display_info2);
 #ifdef OLD
    RTTI_ADD_FUNC(ddci_validate_ddca_display_ref);
 #endif
