@@ -64,7 +64,7 @@ bool ddca_feature_list_contains(DDCA_Feature_List vcplist, uint8_t vcp_code) {
 
 
 const char *
-ddca_feature_list_id_name(
+ddci_feature_list_id_name(
       DDCA_Feature_Subset_Id  feature_subset_id)
 {
    char * result = NULL;
@@ -98,6 +98,14 @@ ddca_feature_list_id_name(
 }
 
 
+const char *
+ddca_feature_list_id_name(
+      DDCA_Feature_Subset_Id  feature_subset_id)
+{
+   return ddci_feature_list_id_name(feature_subset_id);
+}
+
+
 #ifdef NEVER_PUBLISHED
 DDCA_Status
 ddca_get_feature_list(
@@ -174,8 +182,9 @@ DDCA_Feature_List
 feature_list_from_dyn_feature_set(Dyn_Feature_Set * fset)
 {
    bool debug = false;
-   if (debug || IS_TRACING()) {
-      DBGMSG("Starting. feature_set = %p -> %s", (void*)fset, feature_subset_name(fset->subset));
+   DBGTRC_STARTING(debug, TRACE_GROUP, "feature_set = %p -> %s",
+         (void*)fset, feature_subset_name(fset->subset));
+   if (IS_DBGTRC(debug, TRACE_GROUP)) {
       // show_backtrace(2);
       dbgrpt_dyn_feature_set(fset, false, 1);
    }
@@ -202,10 +211,13 @@ feature_list_from_dyn_feature_set(Dyn_Feature_Set * fset)
 #endif
    }
 
+   DBGTRC_RET_STRING(debug, TRACE_GROUP, feature_list_string(&vcplist, "", ","), "");
+#ifdef OLD
    if (debug || IS_TRACING()) {
       DBGMSG("Returning: %s", feature_list_string(&vcplist, "", ","));
       // rpt_hex_dump(vcplist.bytes, 32, 1);
    }
+#endif
 
    return vcplist;
 }
@@ -223,7 +235,7 @@ ddca_get_feature_list_by_dref(
    free_thread_error_detail();
    API_PROLOGX(debug, RESPECT_QUIESCE, "feature_subset_id=%d=0x%08x=%s, ddca_dref=%p, "
               "include_table_features=%s, feature_list_loc=%p",
-              feature_set_id, feature_set_id, ddca_feature_list_id_name(feature_set_id),
+              feature_set_id, feature_set_id, ddci_feature_list_id_name(feature_set_id),
           ddca_dref,
           sbool(include_table_features),
           feature_list_loc);
@@ -291,7 +303,7 @@ ddca_get_feature_list_by_dref(
           "Feature list: %s", feature_list_string(feature_list_loc, "", ","));
       // rpt_hex_dump((Byte*) p_feature_list, 32, 1);
    API_EPILOG_RET_DDCRC(debug, RESPECT_QUIESCE, psc, "feature_set_id=%d=0x%08x=%s, subset=%d=%s",
-         feature_set_id, feature_set_id, ddca_feature_list_id_name(feature_set_id),
+         feature_set_id, feature_set_id, ddci_feature_list_id_name(feature_set_id),
          subset, feature_subset_name(subset));
 }
 
@@ -925,7 +937,8 @@ ddca_dbgrpt_feature_metadata(
       int                     depth)
 {
    bool debug = false;
-   reset_current_traced_function_stack();
+   if (traced_function_stack_enabled)
+      reset_current_traced_function_stack();
    DBGTRC_STARTING(debug, TRACE_GROUP, "");
    // rpt_push_output_dest(stdout);
    dbgrpt_ddca_feature_metadata(md, depth);
