@@ -23,9 +23,7 @@
 #include "util/coredefs.h"
 #include "util/data_structures.h"
 #include "util/debug_util.h"
-#ifdef USE_LIBDRM
-#include "util/drm_common.h"
-#endif
+#include "util/drm_card_connector_util.h"
 #include "util/edid.h"
 #include "util/file_util.h"
 #include "util/glib_string_util.h"
@@ -67,6 +65,7 @@ bool fn_any(const char * filename, const char * ignore) {
 }
 #endif
 
+
 //
 // *** Common Functions
 //
@@ -111,7 +110,7 @@ GPtrArray * get_sys_video_devices() {
 
 
 //
-// Extract bus numbers connetor_id, and name from card-connector directories
+// Extract bus numbers, connetor_id, and name from card-connector directories
 //
 
 void dbgrpt_connector_bus_numbers(Connector_Bus_Numbers * cbn, int depth) {
@@ -131,7 +130,7 @@ void free_connector_bus_numbers(Connector_Bus_Numbers * cbn) {
 
 
 /** Attempts to extract an I2C bus number and additional information from a
- * card-connector directory. This may not always be successful:
+ *  DRM card-connector directory. This is not always successful:
  *  - connector is on MST hub
  *  - Nvidia proprietary driver
  *
@@ -161,6 +160,8 @@ void get_connector_bus_numbers(
    bool debug = false;
    int d = (IS_DBGTRC(debug, DDCA_TRC_NONE)) ? 1 : -1;
    DBGTRC_STARTING(debug, TRACE_GROUP, "dirname=%s, fn=|%s|", dirname, fn);
+   assert(dirname);
+   assert(fn);
    int d0 = (debug) ? 1 : -1;
    bool validate_name = debug;
 
@@ -392,6 +393,7 @@ void dbgrpt_sysfs_basic_connector_attributes(int depth) {
 
 //
 // Get DRM connector name given an I2C bus number or connector id.
+//
 
 typedef struct {
    int    connector_id;
@@ -491,8 +493,8 @@ char * get_sys_drm_connector_name_by_connector_id(int connector_id) {
 }
 
 
-/** Given a I2C bus number, return the name of the
- *  connector for that bus number.
+/** Given a I2C bus number, return the name of the connector for that
+ *  bus number.
  *
  *  @param  busno  i2c bus number
  *  @return connector name
@@ -775,6 +777,54 @@ void get_sysfs_drm_add_one_connector_name(
 }
 
 
+typedef struct {
+   bool found;
+} Found_Accumulator;
+
+bool is_card_connector_dir1(
+   const char *  dirname,
+   const char *  fn,
+   void *        accumulator,
+   int           depth)
+{
+   bool debug = false;
+   DBGF(debug, "dirname=%s, fn=%s, accumulator=%p", dirname, fn, accumulator);
+
+   Found_Accumulator * accum = accumulator;
+   bool found = is_drm_connector(dirname, fn);
+   if (found)
+      accum->found = true;
+
+   DBGF(debug, "Returning %s", sbool(found));
+   return found;
+}
+
+
+bool sysfs_connector_directories_exist() {
+   bool debug = false;
+   static bool executed = false;
+   static bool found = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "executed = %s", sbool(executed));
+
+   if (!executed) {
+      char * dirname = SYS"/class/drm";
+      Found_Accumulator accumulator;
+      accumulator.found = false;
+      dir_foreach_terminatable(
+            dirname,
+            NULL,
+            is_card_connector_dir1,
+            &accumulator,
+            (debug) ? 1 : -1);
+      found = accumulator.found;
+      executed = true;
+   }
+
+   DBGTRC_RET_BOOL(debug, TRACE_GROUP, found, "");
+   return found;
+}
+
+
 /**Checks /sys/class/drm for connectors.
  *
  * @return struct Sysfs_Connector_Names
@@ -784,12 +834,7 @@ void get_sysfs_drm_add_one_connector_name(
  */
 Sysfs_Connector_Names get_sysfs_drm_connector_names() {
    bool debug = false;
-   char * dname =
- #ifdef TARGET_BSD
-              "/compat/linux/sys/class/drm";
- #else
-              "/sys/class/drm";
- #endif
+   const char * dname = SYS"/class/drm";
    DBGTRC_STARTING(debug, TRACE_GROUP, "Examining %s", dname);
 
    Sysfs_Connector_Names connector_names = {NULL, NULL};
@@ -918,7 +963,7 @@ char * find_sysfs_drm_connector_name_by_edid(GPtrArray* connector_names, Byte *
  * status, and enabled attributes as displays are connected and
  * disconnected.
  *
- * Unfortunately depending version, the nvidia driver does not.
+ * Unfortunately depending on version, the nvidia driver does not.
  * Attribute enabled is always "disabled".  It may be the case
  * that the edid value is that of the monitor initially connected.
  * What has been observed is that if the driver does change the
@@ -1057,7 +1102,7 @@ bool is_sysfs_reliable_for_driver(const char * driver) {
          result = known_reliable_driver(driver);
    }
 
-   DBGF(debug, "Executed. Returning %s, driver=%s", SBOOL(result), driver);
+   DBGTRC_EXECUTED(debug, DDCA_TRC_NONE, "Returning %s, driver=%s", SBOOL(result), driver);
    return result;
 }
 
@@ -1461,23 +1506,24 @@ int search_all_businfo_records_by_connector_name(char *connector_name) {
 }
 
 
+/** Module initialization */
 void init_i2c_sysfs_base() {
    RTTI_ADD_FUNC(possibly_write_detect_to_status);
    RTTI_ADD_FUNC(sysfs_find_adapter);
    RTTI_ADD_FUNC(get_i2c_sysfs_driver_by_busno);
    RTTI_ADD_FUNC(get_i2c_device_sysfs_class);
    RTTI_ADD_FUNC(check_connector_reliability);
+   RTTI_ADD_FUNC(is_sysfs_reliable_for_driver);
    RTTI_ADD_FUNC(check_sysfs_reliability);
-   RTTI_ADD_FUNC(dbgrpt_sysfs_basic_connector_attributes);
    RTTI_ADD_FUNC(find_adapter_and_get_driver);
+   RTTI_ADD_FUNC(is_sysfs_reliable);
+   RTTI_ADD_FUNC(dbgrpt_sysfs_basic_connector_attributes);
    RTTI_ADD_FUNC(find_sysfs_drm_connector_name_by_edid);
    RTTI_ADD_FUNC(get_connector_bus_numbers);
    RTTI_ADD_FUNC(get_sys_drm_connector_name_by_connector_id);
-   RTTI_ADD_FUNC(is_sysfs_reliable);
    RTTI_ADD_FUNC(search_all_businfo_records_by_connector_name);
+   RTTI_ADD_FUNC(sysfs_connector_directories_exist);
 #ifdef UNUSED
    RTTI_ADD_FUNC(get_sys_video_devices);
 #endif
-
 }
-
