@@ -1,4 +1,4 @@
-/** @file sysfs_sys_info.c */
+/** @file sysfs_i2c_sys_info.c */
 
 // Copyright (C) 2020-2025 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
@@ -18,16 +18,13 @@
 #include <unistd.h>
 /** \endcond */
 
-
 #include "util/data_structures.h"
 #include "util/debug_util.h"
-#ifdef USE_LIBDRM
-#include "util/drm_common.h"
-#endif
 #include "util/edid.h"
 #include "util/file_util.h"
 #include "util/glib_string_util.h"
 #include "util/i2c_util.h"
+#include "util/libdrm_aux_util.h"
 #include "util/report_util.h"
 #include "util/string_util.h"
 #include "util/subprocess_util.h"
@@ -190,7 +187,6 @@ read_drm_dp_card_connector_node(
 }
 
 
-
 // Process a <controller>/drm/cardN/cardN-<connector> for case when
 // cardN-<connector> is not a DisplayPort connector
 
