@@ -2,11 +2,11 @@
  *  String utility functions header file
  */
 
-// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
-#ifndef STRINGUTIL_H_
-#define STRINGUTIL_H_
+#ifndef STRING_UTIL_H_
+#define STRING_UTIL_H_
 
 /** \cond */
 #include <stdbool.h>
@@ -42,8 +42,8 @@ bool   str_starts_with(const char * value_to_test, const char * start_part);
 bool   str_ends_with(const char * value_to_test, const char * end_part);
 int    str_contains(const char * value_to_test, const char * segment);
 bool   str_all_printable(const char * s);
-char * strupper(char * s);
-char * strlower(char * s);
+void   strupper(char * s);
+void   strlower(char * s);
 char * strdup_uc(const char* s);
 char * strjoin( const char ** pieces, const int ct, const char * sepstr);
 char * chars_to_string(const char * start, int len);
@@ -65,13 +65,15 @@ int matches_by_func(    const char * word, const char ** match_list, String_Comp
 int exactly_matches_any(const char * word, const char ** match_list);
 int starts_with_any(    const char * word, const char ** match_list);
 
+char * int_array_to_string(uint16_t * start, int ct);
+
 /** pointer to null-terminated array of strings */
 typedef char** Null_Terminated_String_Array;    // equivalent to GStrv
 void ntsa_free(  Null_Terminated_String_Array string_array, bool free_strings);
 int  ntsa_length(Null_Terminated_String_Array string_array);
 void ntsa_show(  Null_Terminated_String_Array string_array);
-int  ntsa_findx( Null_Terminated_String_Array string_array, char * value, String_Comp_Func func);
-int  ntsa_find(  Null_Terminated_String_Array string_array, char * value);
+int  ntsa_findx( Null_Terminated_String_Array string_array, const char * value, String_Comp_Func func);
+int  ntsa_find(  Null_Terminated_String_Array string_array, const char * value);
 Null_Terminated_String_Array ntsa_join(  Null_Terminated_String_Array a1, Null_Terminated_String_Array a2, bool dup);
 Null_Terminated_String_Array ntsa_copy(Null_Terminated_String_Array a1, bool dup);
 Null_Terminated_String_Array ntsa_prepend(char * value, Null_Terminated_String_Array string_array, bool dup);
@@ -91,7 +93,8 @@ Null_Terminated_String_Array g_ptr_array_to_ntsa(GPtrArray * garray, bool duplic
 // Numeric conversion
 //
 
-bool str_to_int(const char * sval, int * p_ival, int base);
+bool str_to_long( const char * sval, long *  p_ival, int base);
+bool str_to_int(  const char * sval, int *   p_ival, int base);
 bool str_to_float(const char * sval, float * p_fval);
 
 
@@ -103,7 +106,8 @@ char * canonicalize_possible_hex_value(char * string_value);
 bool hhs_to_byte_in_buf(const char * s,  Byte * result);    // converts null terminated string into buffer
 bool any_one_byte_hex_string_to_byte_in_buf(const char * s, Byte * result);
 bool hhc_to_byte_in_buf(const char * hh, Byte * result);    // converts 2 characters at hh into buffer
-int  hhs_to_byte_array(const char * hhs, Byte** pBa);
+int  hhs_to_byte_array(const char * hhs, Byte** ba_loc);
+bool hhs4_to_uint16(char * hhs4, uint16_t* result);
 
 char * hexstring(const Byte * bytes, int size);  // buffer returned must be freed
 char * hexstring_t(
@@ -123,6 +127,7 @@ char * hexstring3_t(
           uint8_t               hunk_size,  // separator string frequency
           bool                  uppercase); // use upper case hex characters
 
+void hex_dump_indented_collect(GPtrArray * collector, const Byte* data, int size, int indents);
 void fhex_dump_indented(FILE * fh, const Byte* data,  int size, int indents);
 void fhex_dump(         FILE * fh, const Byte* bytes, int size);
 void hex_dump(                     const Byte* bytes, int size);
@@ -150,4 +155,4 @@ bool apply_filter_terms(const char * text, char ** terms, bool ignore_case);
 #endif
 
 
-#endif /* STRINGUTIL_H_ */
+#endif /* STRING_UTIL_H_ */
